
    !!***********************************************************************
    !! Version "$Id: xposemat.f90 1 2017-06-10 18:05:20Z coats $"
    !! EDSS/Models-3 I/O API.
    !! Copyright (C) 2020 UNC Institute for the Environment
    !! Distributed under the GNU LESSER GENERAL PUBLIC LICENSE version 2.1
    !! See file "LGPL.txt" for conditions of use.
    !!.........................................................................
    !!  subroutine XPOSEMAT  body starts at line  56
    !!  subroutine XPOSEMATP body starts at line  56
    !!
    !!  FUNCTION:  Compute the transpose of sparse matrix <NX,IX,CX> and
    !!             return the result as  <NY,IY,CY>
    !!
    !!  PRECONDITIONS REQUIRED:
    !!      XPOSEMAT:   Matrix (and transpose) in incremental-count form
    !!      XPOSEMATP:  Matrix (and transpose) in cumulative-count form
    !!
    !!  SUBROUTINES AND FUNCTIONS CALLED:  none
    !!
    !!  REVISION  HISTORY:
    !!       prototype 2/2020 by CJC
    !!***********************************************************************

SUBROUTINE  XPOSEMAT( NCOLS, NROWS, NCOFF, NX, IX, CX, NY, IY, CY )

    IMPLICIT NONE

    !!...........   ARGUMENTS and their descriptions:

    INTEGER, INTENT(IN   ) :: NCOLS           ! cols in <NX,IX,CX> = rows in <NY,IY,CY>
    INTEGER, INTENT(IN   ) :: NROWS           ! rows in <NX,IX,CX> = cols in <NY,IY,CY>
    INTEGER, INTENT(IN   ) :: NCOFF           ! max number of sparse coefficients

    INTEGER, INTENT(IN   ) :: NX( NROWS )     ! # of entries per row
    INTEGER, INTENT(IN   ) :: IX( NCOFF )     ! columns list
    REAL   , INTENT(IN   ) :: CX( NCOFF )     ! coefficient array

    INTEGER, INTENT(INOUT) :: NY( NCOLS )     ! # of entries per row
    INTEGER, INTENT(INOUT) :: IY( NCOFF )     ! columns list
    REAL   , INTENT(INOUT) :: CY( NCOFF )     ! coefficient array

    !!...........   SCRATCH LOCAL VARIABLES and their descriptions:

    INTEGER         C, R, J, K, L, N

    !......................  begin body of XPOSEMAT  .......................
    
    DO C = 1, NCOLS
        NY( C ) = 0
    END DO
    
    L = 0
    DO C = 1, NCOLS

        N = 0
        K = 0

        DO R = 1, NROWS
        DO J = 1, NX( R )
            K = K + 1
            IF ( IX( K ) .EQ. C ) THEN
                N = N + 1
                L = L + 1
                IY( L ) = R
                CY( L ) = CX( K )
            END IF
        END DO
        END DO

        NY( C ) = N

    END DO      !!  end loop on cols C

    RETURN

END SUBROUTINE  XPOSEMAT

    !!***********************************************************************

SUBROUTINE  XPOSEMATP( NCOLS, NROWS, NCOFF, NX, IX, CX, NY, IY, CY )

    IMPLICIT NONE

    !!...........   ARGUMENTS and their descriptions:

    INTEGER, INTENT(IN   ) :: NCOLS           ! cols in <NX,IX,CX> = rows in <NY,IY,CY>
    INTEGER, INTENT(IN   ) :: NROWS           ! rows in <NX,IX,CX> = cols in <NY,IY,CY>
    INTEGER, INTENT(IN   ) :: NCOFF           ! max number of sparse coefficients

    INTEGER, INTENT(IN   ) :: NX( 0:NROWS )   ! cumulative # of entries per row
    INTEGER, INTENT(IN   ) :: IX(   NCOFF )   ! columns list
    REAL   , INTENT(IN   ) :: CX(   NCOFF )   ! coefficient array

    INTEGER, INTENT(INOUT) :: NY( 0:NCOLS )   ! cumulative # of entries per row
    INTEGER, INTENT(INOUT) :: IY(   NCOFF )   ! rows list
    REAL   , INTENT(INOUT) :: CY(   NCOFF )   ! coefficient array

    !!...........   SCRATCH LOCAL VARIABLES and their descriptions:

    INTEGER         C, R, J, K, N

    !......................  begin body of XPOSEMAT  .......................
    
    N = 0
    DO C = 1, NCOLS

        DO R = 1, NROWS
        DO K = NX( R-1 )+1, NX( R )
            IF ( IX( K ) .EQ. C ) THEN
                N = N + 1
                IY( N ) = R
                CY( N ) = CX( K )
            END IF
        END DO
        END DO

        NY( C ) = N

    END DO      !!  end loop on cols C

    RETURN

END SUBROUTINE  XPOSEMATP
