
C.........................................................................
C Version "%W% %P% %G% %U%"
C EDSS/Models-3 I/O API.  Portions copyright (C) 1992-1997 MCNC
C See file "COPYRIGHT.txt" for conditions of use.
C.........................................................................

        LOGICAL FUNCTION DDTVAR3( FNAME, VNAME, CALLER,
     &                            JDATE, JTIME, RSIZE, BUFFER )

C
C JEB FOR DLL COMPILATION
C
!DEC$ IF DEFINED(MAKE_DLL)
!DEC$ ATTRIBUTES DLLEXPORT :: DDTVAR3
!DEC$ ENDIF
C***********************************************************************
C  DDTVAR3 function body starts at line  128
C
C  NOTE:  MACHINE-DEPENDENT CODE !!!  depends upon FORTRAN : C
C       coupling between DDTVAR3() and BUFDDT3()
C
C  FUNCTION:  calculates time-derivative for data from Models-3 file 
C       with logical name FNAME, variable VNAME for all layers for 
C       date and time JDATE (coded YYYYDDD) and time JTIME (HHMMSS), 
C	putting the result in BUFFER.
C       File type must be CUSTOM, GRIDDED, or BOUNDARY.
C       For time-independent files, JDATE:JTIME are ignored (derivative
C	is identically zero).
C       Encapsulates file opens and reads, and all the book-keeping
C       and data structures necessary for that task.
C
C  USES C routine BUFDDT3:  sets up all the data structures for the
C       call so that BUFDDT3 can allocate and manage read-buffers
C       internally, call NCVGT and perform the interpolation.
C       (Actually, DDTVAR3() + BUFDDT3() should be considered as
C       two parts of a single routine)
C
C  RETURN VALUE:  TRUE iff the operation succeeds (and the data is available)
C
C  PRECONDITIONS REQUIRED:  FNAME is a Models-3 data file containing
C       REAL variable VNAME for a set of time steps enclosing JDATE:JTIME
C       File type must be CUSTOM3, GRDDED3, or BNDARY3.
C
C  SUBROUTINES AND FUNCTIONS CALLED:
C       BUFDDT3, CURRSTEP, INDEX1, JSTEP3, M3WARN, NEXTIME, OPEN3,
C       SECSDIFF, TIME2SEC
C
C  REVISION  HISTORY:
C       prototype 7/94 by CJC
C       modified 10/94 by CJC for new WRITE3 -- granularity at the
C       level of individual variables.  FNAME, VNAME are now CHAR*(*).
C       modified  6/95 by CJC to check VTYPE3( VID,FID )
C
C***********************************************************************

      IMPLICIT NONE

C...........   INCLUDES:

        INCLUDE 'PARMS3.EXT'
        INCLUDE 'STATE3.EXT'
        INCLUDE 'NETCDF.EXT'


C...........   ARGUMENTS and their descriptions:

        CHARACTER*(*)   FNAME           !  logical file name
        CHARACTER*(*)   VNAME           !  variable name, or 'ALL'
        CHARACTER*(*)   CALLER          !  name of caller
        INTEGER         JDATE           !  date, formatted YYYYDDD
        INTEGER         JTIME           !  time, formatted HHMMSS
        INTEGER         RSIZE           !  record dim, words, for error-check
        REAL            BUFFER( RSIZE ) !  interpolation-output buffer array


C...........   EXTERNAL FUNCTIONS and their descriptions:

        INTEGER         BUFDDT3         !  ddt() from circ. bufs. (C)
        INTEGER         BUFVGT3         !  alloc/read for circ. bufs. (C)
        LOGICAL         CURRSTEP        !  finds start of timestep
        INTEGER         INDEX1          !  name-table lookup
        INTEGER         INIT3           !  initialize I/O API
        INTEGER         JSTEP3          !  compute time step record numbers
        LOGICAL         OPEN3           !  opens M3 files
        INTEGER         SECSDIFF        !  computes date&time differences
        INTEGER         TIME2SEC        !  converts time to seconds
        INTEGER         TRIMLEN         !  effective char. string length

        EXTERNAL    BUFDDT3, BUFVGT3, CURRSTEP, INDEX1, INIT3, JSTEP3,
     &              OPEN3, SECSDIFF, TIME2SEC, TRIMLEN


C...........   STATE VARIABLES:  

        LOGICAL         FIRSTIME
        DATA            FIRSTIME        / .TRUE. /

        CHARACTER*16    TYPNAMES( -3 : 5 )
        DATA            TYPNAMES
     &      / 'UNKNOWN',        !  -3:  file type error
     &        'DGRAPH3',        !  -2:  known file types
     &        'CUSTOM3',        !  -1:    "
     &        'DCTNRY3',        !   0:    "
     &        'GRDDED3',        !   1:    "
     &        'BNDARY3',        !   2:    "
     &        'IDDATA3',        !   3:    "
     &        'PROFIL3',        !   4:    "
     &        'GRNEST3' /       !   5:    "

        SAVE            FIRSTIME, TYPNAMES


C...........   SCRATCH LOCAL VARIABLES and their descriptions:

        INTEGER         IDUM            !  scratch variable
        INTEGER         FID             !  file subscript for STATE3 arrays
        INTEGER         VID             !  variable subscript for STATE3 arrays
        INTEGER         TDIM            !  subscript for time step
        INTEGER         STEP            !  time step record number
        INTEGER         DT              !  seconds in TSTEP3
        INTEGER         MDATE, MTIME    !  date:time args for CURRSTEP
        INTEGER         NDATE, NTIME    !  date:time of "next" record
        INTEGER         DTJL            !  difference LDATE:LTIME to JDATE:JTIME
        INTEGER         DTNJ            !  difference JDATE:JTIME to NDATE:NTIME
        CHARACTER*24    NAMBUF          !  buffer for names (CRAY-PORTING HACK)
        INTEGER         DELTA           !  d(INDX) / d(NCVGTcall)
        INTEGER         DIMS( 5 )       !  hyperslab corner   for NCVGT()
        INTEGER         DELS( 5 )       !  hyperslab diagonal for NCVGT()
        INTEGER         DIMT( 5 )       !  hyperslab corner   for NCVGT()
        INTEGER         DELT( 5 )       !  hyperslab diagonal for NCVGT()
        INTEGER         FLAGS( 2,2 )    !  timestep-valid flags
        INTEGER         DATE1, DATE2    !
        INTEGER         TIME1, TIME2    !
        INTEGER         IERR            !  error flag for NCVGT call
        LOGICAL         RFLAG           !  BUFDDT3() read-data flag
        LOGICAL         EFLAG           !  error flag
        CHARACTER*16    FIL16           !  scratch file-name-buffer
        CHARACTER*16    VAR16           !  scratch vble-name-buffer
        CHARACTER*256   MESG            !  for m3warn(), etc


C***********************************************************************
C   begin body of function  DDTVAR3

!$OMP   CRITICAL( S_LOGDEV )
        IF ( .NOT. FINIT3 ) THEN
            LOGDEV = INIT3()
        END IF          !  if not FINIT3
!$OMP   END CRITICAL( S_LOGDEV )

C...........   Check length of name arguments; copy into length=16 buffers
            
        IF ( LEN( FNAME ) .GT. 16 ) THEN
            NAMBUF = CALLER // ':DDTVAR3'
            MESG = 'File "'// FNAME // '" Variable "' // VNAME // '"'
            CALL M3MSG2( MESG )
            WRITE( MESG,91010 )
     &          'Max file name length 16; actual:', LEN( FNAME )
            CALL M3WARN( NAMBUF, JDATE, JTIME, MESG )
            DDTVAR3 = .FALSE.
            RETURN
        ELSE IF ( LEN( VNAME ) .GT. 16 ) THEN
            NAMBUF = CALLER // ':DDTVAR3'
            MESG = 'File "'// FNAME// '" Variable "'// VNAME//'"'
            CALL M3MSG2( MESG )
            WRITE( MESG,91010 )
     &          'Max vble name length 16; actual:', LEN( VNAME )
            CALL M3WARN( NAMBUF, JDATE, JTIME, MESG )
            DDTVAR3 = .FALSE.
            RETURN
        END IF          !  if len( fname ) > 16, or if len( vname ) > 16


C.......   Find netCDF index for the file, and check time step availability:

!$OMP CRITICAL( S_NC )

        FIL16 = FNAME   !  fixed-length-16 scratch copy of name
        FID   = INDEX1( FIL16, COUNT3, FLIST3 )

        IF ( FID .EQ. 0 ) THEN  !  file needs opening

            NAMBUF = CALLER // ':DDTVAR3'
            IF ( .NOT.OPEN3( FIL16, FSREAD3, NAMBUF ) ) THEN

                MESG = 'Could not open ' // FIL16
                CALL M3WARN( NAMBUF, JDATE, JTIME, MESG )
                DDTVAR3 = .FALSE.
                RETURN

            END IF      !  if open3() failed

            FID = INDEX1( FIL16, COUNT3, FLIST3 )

            IF ( FID .EQ. 0 ) THEN      !  lookup failed

                NAMBUF = CALLER // ':DDTVAR3'
                MESG   = 'Error accessing ' // FIL16
                CALL M3WARN( NAMBUF, JDATE, JTIME, MESG )

                DDTVAR3 = .FALSE.
                RETURN

            END IF      !  if file error

        END IF          !  if file needs opening

!$OMP END CRITICAL( S_NC )


        IDUM = FTYPE3( FID )
        IF ( IDUM .EQ. CUSTOM3 ) THEN

            DIMS( 1 ) = 1
            DELS( 1 ) = NCOLS3( FID )

            DIMS( 2 ) = 1
            DELS( 2 ) = NLAYS3( FID )

            TDIM  = 3

        ELSE IF ( IDUM .EQ. GRDDED3 ) THEN

            DIMS( 1 ) = 1
            DELS( 1 ) = NCOLS3( FID )

            DIMS( 2 ) = 1
            DELS( 2 ) = NROWS3( FID )

            DIMS( 3 ) = 1
            DELS( 3 ) = NLAYS3( FID )

            TDIM  = 4

        ELSE IF ( IDUM .EQ. BNDARY3 ) THEN

            DELTA = 2 * NTHIK3( FID )
            DELTA = DELTA*( NCOLS3( FID ) + NROWS3( FID ) + DELTA )

            DIMS( 1 ) = 1
            DELS( 1 ) = DELTA

            DIMS( 2 ) = 1
            DELS( 2 ) = NLAYS3( FID )

            TDIM  = 3

        ELSE        !  bad file type for DDTVAR3()

            IF ( IDUM .LT. -2   .OR.  IDUM .GT. 5 ) IDUM = -3
            NAMBUF = CALLER // ':DDTVAR3'
            MESG   = 'Can not interpolate:  file "' //
     &          FIL16( 1:TRIMLEN( FIL16 ) ) // 
     &          '" of type ' // TYPNAMES( IDUM )
            CALL M3WARN( NAMBUF, JDATE, JTIME, MESG )

            DDTVAR3 = .FALSE.
            RETURN

        END IF              !  check file types; set DIMS, DELS, TDIM

        IF ( RSIZE .NE. NLAYS3( FID ) * BSIZE3( FID ) ) THEN

            WRITE( MESG,94010 )
     &      'Size error for "' //VAR16( 1:TRIMLEN( VAR16 ) )//
     &      '" from "' // FIL16( 1:TRIMLEN( FIL16 ) ) //
     &      '--REQ:', RSIZE, ' ACT:', NLAYS3( FID ) * BSIZE3( FID )
            NAMBUF = CALLER // ':DDTVAR3'
            CALL M3WARN( NAMBUF, JDATE, JTIME, MESG )
            DDTVAR3 = .FALSE.
            RETURN

        END IF          !  requested record-size in error


C.......   Find index for variable, and if necessary allocate circular
C.......   buffers for interpolation, and initialize related data structures.

        VAR16 = VNAME
        VID   = INDEX1( VAR16, NVARS3( FID ), VLIST3( 1,FID ) )

        IF ( VID .EQ. 0 ) THEN  !  variable not available in that file

            NAMBUF = CALLER // ':DDTVAR3'
            MESG = 'Variable "' // VAR16( 1:TRIMLEN( VAR16 ) ) //
     &          '" not in file "' // FIL16( 1:TRIMLEN( FIL16 ) ) // '"'
            CALL M3WARN( NAMBUF, JDATE, JTIME, MESG )
            DDTVAR3 = .FALSE.
            RETURN

        ELSE IF ( VTYPE3( VID,FID ) .NE. M3REAL ) THEN  !  variable of wrong type

            NAMBUF = CALLER // ':DDTVAR3'
            MESG =  'Variable "' // VAR16( 1:TRIMLEN( VAR16 ) ) //
     &          '" not of type REAL in file "' // 
     &          FIL16( 1:TRIMLEN( FIL16 ) ) // '"'
            CALL M3WARN( NAMBUF, JDATE, JTIME, MESG )
            DDTVAR3 = .FALSE.
            RETURN

        END IF    !  if variable not available; else if not already set up


C.......   Interpolate/copy to JDATE:JTIME

        IF ( TSTEP3( FID ) .EQ. 0 ) THEN        !  time-independent file

            DO  11  IDUM = 1, RSIZE
                BUFFER( IDUM ) = 0.0        
11          CONTINUE
            DDTVAR3 = .TRUE.
            RETURN

        ELSE IF ( CDFID3( FID ) .LT. 0 ) THEN	!  "buffered" virtual file
            
            RFLAG = .FALSE.
            
            IF( LDATE3( VID,FID ) .LT. 0 ) THEN
                MESG = 'Variable "' // VAR16( 1:TRIMLEN( VAR16 ) ) //
     &                   '" not yet written to BUFFERED file "' //
     &                   FIL16( 1:TRIMLEN( FIL16 ) ) // '".'
                NAMBUF = CALLER // ':DDTVAR3'
                CALL M3WARN( NAMBUF, JDATE, JTIME, MESG )
                DDTVAR3 = .FALSE.
                RETURN
            END IF

            DT   = TIME2SEC( TSTEP3( FID ) )
            DTJL = SECSDIFF( LDATE3( VID,FID ), LTIME3( VID,FID ),
     &                       JDATE,             JTIME )

            IF ( DTJL .LT. 0 ) THEN	!  ldate:ltime not OK for interp

                NAMBUF = CALLER // ':DDTVAR3'
                WRITE( MESG, 94030 )
     &                  JDATE, JTIME,
     &                  ' n/a for ' // VAR16( 1:TRIMLEN( VAR16 ) ) //
     &                  ' from '    // FIL16( 1:TRIMLEN( FIL16 ) ) //
     &                  ' (',
     &                  SDATE3( FID ), STIME3( FID ),
     &                  ' by', TSTEP3( FID ), ')'
                CALL M3WARN( NAMBUF, JDATE, JTIME, MESG )
                DDTVAR3 = .FALSE.
                RETURN

            ELSE IF ( DTJL .GT. 0 ) THEN !  interp between ld:lt and nd:nt

                DTNJ = SECSDIFF( JDATE,             JTIME,
     &                           NDATE3( VID,FID ), NTIME3( VID,FID ) )
                IF ( ( DTNJ .LT. 0 ) .OR. DTNJ .GT. DT ) THEN
                    WRITE( MESG, 94030 )
     &                  JDATE, JTIME,
     &                  ' n/a for ' // VAR16( 1:TRIMLEN( VAR16 ) ) //
     &                  ' from '    // FIL16( 1:TRIMLEN( FIL16 ) ) //
     &                  ' (',
     &                  SDATE3( FID ), STIME3( FID ),
     &                  ' by', TSTEP3( FID ), ')'
                    CALL M3WARN( NAMBUF, JDATE, JTIME, MESG )
                    DDTVAR3 = .FALSE.
                    RETURN

                END IF	!  if ndate:ntime not OK for interp

            END IF	!  if dtjl < 0; else if dtjl > 0 (just copy if dtjl=0)

        ELSE            !  time-stepped file.

C...........   Check to see relationship between currently circular buffer
C...........   start and ending times, and requested time.  Two cases
C...........   require updates:  (1) need new data at both ends; and
C...........   (2) need new data at the futureward end only. (Needing
C...........   data at the pastward end is still treated under (1).)

            DT   = TIME2SEC( TSTEP3( FID ) )

            IF ( LDATE3( VID,FID ) .GT. 0 ) THEN
                DTJL = SECSDIFF( LDATE3( VID,FID ), LTIME3( VID,FID ),
     &                           JDATE,             JTIME )
            ELSE                !  ldate set to "invalid"
                DTJL = IMISS3   !  dtjl "invalid"
            END IF

            IF ( ( DTJL .LT. 0 ) .OR. ( DTJL .GT. 2*DT ) ) THEN ! read both

                IF ( .NOT. CURRSTEP( JDATE, JTIME,
     &                               SDATE3( FID ), STIME3( FID ),
     &                               TSTEP3( FID ),
     &                               MDATE, MTIME ) ) THEN

                    NAMBUF = CALLER // ':DDTVAR3'
                    WRITE( MESG, 94030 )
     &                  JDATE, JTIME,
     &                  ' n/a for ' // VAR16( 1:TRIMLEN( VAR16 ) ) //
     &                  ' from '    // FIL16( 1:TRIMLEN( FIL16 ) ) //
     &                  ' (',
     &                  SDATE3( FID ), STIME3( FID ),
     &                  ' by', TSTEP3( FID ), ')'
                    CALL M3WARN( NAMBUF, JDATE, JTIME, MESG )
                    DDTVAR3 = .FALSE.
                    RETURN

                END IF          !  if requested date&time before start of file

                RFLAG = .TRUE.
                DELTA =  0                      !  read starts at 2
                ILAST3( VID, FID ) = 0          !  buffer starts at 0
                LDATE3( VID, FID ) = MDATE
                LTIME3( VID, FID ) = MTIME
                DELS( TDIM ) = 2               !  read BOTH time steps
                NDATE = LDATE3( VID,FID )
                NTIME = LTIME3( VID,FID )
                CALL NEXTIME( NDATE, NTIME, TSTEP3( FID ) )
                DATE1 = LDATE3( VID,FID )
                TIME1 = LTIME3( VID,FID )
                DATE2 = NDATE
                TIME2 = NTIME
                NDATE3( VID,FID ) = DATE2
                NTIME3( VID,FID ) = TIME2

                STEP = JSTEP3( LDATE3( VID,FID ), LTIME3( VID,FID ),
     &                         SDATE3(     FID ), STIME3(     FID ),
     &                         TSTEP3(     FID ) )

                DTJL = SECSDIFF( LDATE3( VID,FID ),
     &                           LTIME3( VID,FID ),
     &                           JDATE, JTIME )

                IF ( TSTEP3( FID ) .LT. 0 ) THEN
                    DIMS( TDIM ) = 1 + MOD( STEP-1, 2 )
                ELSE
                    DIMS( TDIM ) = STEP
                END IF

            ELSE IF ( DTJL .GT. DT ) THEN      !  advance by one time step

                CALL NEXTIME( LDATE3( VID, FID ),
     &                        LTIME3( VID, FID ),
     &                        TSTEP3( FID ) )

                NDATE3( VID,FID ) = LDATE3( VID, FID )
                NTIME3( VID,FID ) = LTIME3( VID, FID )
                CALL NEXTIME( NDATE3( VID,FID ), 
     &                        NTIME3( VID,FID ), 
     &                        TSTEP3( FID ) )
                
                DELTA = ILAST3( VID,FID )        !  overwrite ILAST entry
                ILAST3( VID,FID ) = 1 - DELTA    !  new ILAST after read:
                                                !  formula swaps 0,1
                DTJL  = DTJL - DT
                RFLAG = .TRUE.
                DELS( TDIM ) = 1                      !  read one time
                STEP = 1 + JSTEP3( LDATE3( VID,FID ),  !  step for the
     &                             LTIME3( VID,FID ),  !  record
     &                             SDATE3( FID ),      !  one step
     &                             STIME3( FID ),      !  BEYOND
     &                             TSTEP3( FID ) )     !  ldate:ltime
                    
                IF ( TSTEP3( FID ) .LT. 0 ) THEN
                    DIMS( TDIM ) = 1 + MOD( STEP-1, 2 )
                ELSE
                    DIMS( TDIM ) = STEP
                END IF

            ELSE        !  do not need to read this variable from this file:

                RFLAG = .FALSE.

            END IF      !  if circular-buffer structure needed updating

        END IF          ! time stepped file or not


C..........   If read will be necessary, check timestep flags first:

        IF ( RFLAG ) THEN       !  check record-available flag
        
            IF ( CDFID3( FID ) .EQ. BUFFIL3 ) THEN
                
                MESG = 'BUFFERED virtual-file not yet written to ' 
     &                   // FLIST3( FID )
                NAMBUF = CALLER // ':DDTVAR3'
                CALL M3WARN( NAMBUF, JDATE, JTIME, MESG )
                DDTVAR3 = .FALSE.
                RETURN
                
            END IF

            DIMT( 1 ) = 1               !  field:  date or time
            DELT( 1 ) = 2               !  extent:  both date and time
            DIMT( 2 ) = VID             !  initial variable-number
            DELT( 2 ) = 1               !  extent:  1 variable
            DIMT( 3 ) = DIMS( TDIM )    !  initial time step number
            DELT( 3 ) = DELS( TDIM )    !  time step extent

	    EFLAG = .FALSE.
            
!$OMP       CRITICAL( S_NC )
            CALL NCVGT( CDFID3( FID ),
     &                  TINDX3( FID ),
     &                  DIMT, DELT, FLAGS, IERR )
!$OMP       END CRITICAL( S_NC )
            IF ( IERR .EQ. 8 ) THEN     !  timestep flag not yet written

                NAMBUF = CALLER // ':DDTVAR3'
                CALL M3WARN( NAMBUF, JDATE, JTIME,
     &                       'Time step not yet written for ' //
     &                       VAR16( 1:TRIMLEN( VAR16 ) ) //
     &                       ' from ' // FIL16( 1:TRIMLEN( FIL16 ) ) )
                EFLAG   = .TRUE.
                GO TO 900

            ELSE IF ( IERR .NE. 0 ) THEN

                WRITE( MESG,94020 )
     &                      'netCDF error', IERR,
     &                      'reading TFLAG for '
     &                      // FIL16( 1:TRIMLEN( FIL16 ) )
                NAMBUF = CALLER // ':DDTVAR3'
                CALL M3WARN( NAMBUF, JDATE, JTIME, MESG )
                EFLAG   = .TRUE.
                GO TO 900

C...........   Check validity of timestep flag(s)

            ELSE IF ( DELS( TDIM ) .EQ. 1 ) THEN        ! request 1 timestep

                IF ( FLAGS( 1,1 ) .NE. NDATE3( VID,FID )   .OR.
     &               FLAGS( 2,1 ) .NE. NTIME3( VID,FID ) ) THEN

                    NAMBUF = CALLER // ':DDTVAR3'
                    CALL M3WARN( NAMBUF, JDATE, JTIME,
     &                  'Data not available for ' //
     &                  VAR16( 1:TRIMLEN( VAR16 ) ) //
     &                  ' from ' // FIL16( 1:TRIMLEN( FIL16 ) ) )
                    EFLAG   = .TRUE.
                    GO TO 900

                END IF

            ELSE        !  dims( tdim ) = 2:  request both time-steps

                IF ( FLAGS( 1,1 ) .NE. DATE1  .OR.
     &               FLAGS( 2,1 ) .NE. TIME1  .OR.
     &               FLAGS( 1,2 ) .NE. DATE2  .OR.
     &               FLAGS( 2,2 ) .NE. TIME2 ) THEN

                    NAMBUF = CALLER // ':DDTVAR3'
                    CALL M3WARN( NAMBUF, JDATE, JTIME,
     &                  'Time step not available for ' //
     &                  VAR16( 1:TRIMLEN( VAR16 ) ) //
     &                  ' from ' // FIL16( 1:TRIMLEN( FIL16 ) ) )
                    EFLAG   = .TRUE.
                    GO TO 900

                END IF      !  if this timestep flag bad

            END IF          !  if bad ierr for NCVGT(), or if bad timestep
            
            IF ( 0 .EQ. BUFVGT3( FID, VID, RFLAG, DIMS, DELS,
     &                           NLAYS3( FID ) * BSIZE3( FID ), 
     &                           DELTA, TSTEP3( FID ) ) ) THEN

                WRITE( MESG,94040 ) 
     &              'Error reading ' //
     &              VAR16( 1:TRIMLEN( VAR16 ) ) //
     &              ' from ' // FIL16( 1:TRIMLEN( FIL16 ) ) // 
     &              '--LAST',  LDATE3( VID,FID ), LTIME3( VID,FID ),
     &              '; NEXT',  NDATE3( VID,FID ), NTIME3( VID,FID )
                CALL M3WARN( NAMBUF, JDATE, JTIME, MESG )
                EFLAG   = .TRUE.
                
            END IF              !  if bufvgt() fails

900         CONTINUE

            IF ( EFLAG ) THEN   !  return FALSE if error in critical section

                DDTVAR3 = .FALSE.
                RETURN

            END IF
        
       END IF          !  if RFLAG:  read will be necessary


C...........   Call BUFDDT3 to read & interpolate; check its success/failure
C...........   status and return accordingly:

        IF ( 0 .NE. BUFDDT3( FID, VID, NLAYS3( FID ) * BSIZE3( FID ),
     &                       ILAST3( VID,FID ), TSTEP3( FID ),
     &                       1.0 / FLOAT( TIME2SEC( TSTEP3( FID ) ) ),
     &                       BUFFER ) ) THEN

            DDTVAR3 = .TRUE.

        ELSE

            DDTVAR3 = .FALSE.
            LDATE3( VID, FID ) = IMISS3   !  "invalid"

            NAMBUF = CALLER // ':DDTVAR3'
            MESG = 'Could not read "' // 
     &             VAR16( 1:TRIMLEN( VAR16 ) ) // '" from "' // 
     &             FIL16( 1:TRIMLEN( FIL16 ) ) // '"'
            CALL M3WARN( NAMBUF, JDATE, JTIME, MESG )

        END IF          !  if bufint3() succeeded, or failed

        RETURN          !  from DDTVAR3

C******************  FORMAT  STATEMENTS   ******************************

C...........   Error and warning message formats..... 91xxx

91010   FORMAT ( A , I5 )

C...........   Internal buffering formats............ 94xxx

94010   FORMAT( A, I7, A, I7, :, A )

94020   FORMAT( A, I4, 2X, A )

94030   FORMAT( I7.7, ':', I6.6, A, I7.7, ':', I6.6, A, I8 , A )

94040   FORMAT( A, :, I9.7, ':', I6.6 )


        END             !  logical function DDTVAR3

