
C.........................................................................
C Version "%W% %P% %G% %U%"
C EDSS/Models-3 I/O API.  Portions copyright (C) 1992-1997 MCNC
C See file "COPYRIGHT.txt" for conditions of use.
C.........................................................................

        REAL   FUNCTION GETREAL ( LO , HI , DEFAULT , PROMPT )

C
C JEB FOR DLL COMPILATION
C
!DEC$ IF DEFINED(MAKE_DLL)
!DEC$ ATTRIBUTES DLLEXPORT :: GETREAL
!DEC$ ENDIF
C********************************************************************
C       function body starts at line  81
C
C   CALLS:  TRIMLEN  
C
C   FUNCTION:
C
C       Display the  PROMPT  for an integer between  LO  and  HI,
C       get the user's response and check that it is within range.
C       Return DEFAULT if the user hits <RET>.  Reprompts on error
C       for up to 5 attempts.
C
C  REVISION HISTORY:
C
C       Adapted  9/90 by CJC from GETNUM
C       Version  2/93 by CJC for CRAY
C       Version  8/96 by CJC treats ! as a terminator
C       Modified 1/97 by CJC:  logs result
C       Modified 8/98 by CJC:  PROMPT2 and strict f77 string-handling
C
C  ARGUMENT LIST DESCRIPTION:
C
C    Input arguments:
C
C        LO       Minimum allowed return value
C        HI       Maximum allowed return value
C        DEFAULT  Default return value
C        PROMPT   Prompt for user
C
C    Output arguments:  none
C
C  RETURNS   user response after checking its range; or default.
C
C********************************************************************

        IMPLICIT NONE

C.......   ARGUMENTS:

        REAL            LO , HI
        REAL     	DEFAULT
        CHARACTER*(*)   PROMPT


C.......   EXTERNAL FUNCTION:  interpret I/O errors:

        LOGICAL         ENVYN
        INTEGER         TRIMLEN
        EXTERNAL        ENVYN, TRIMLEN


C.......   LOCAL VARIABLES:

        INTEGER         K , L , M , N
        REAL            LLO , LHI , LDF
        REAL            ANSWER
        INTEGER         ERRCNT
        INTEGER         IOS
        CHARACTER*64    BUFFER
        CHARACTER*256   MESG
        CHARACTER*256   PROMPT2
        CHARACTER*16	FMTSTR
        CHARACTER*1	CH

        LOGICAL         PROMPTON

        LOGICAL         FIRSTIME
        DATA            FIRSTIME / .TRUE. /
        SAVE            FIRSTIME, PROMPTON

C*********************************************************************
C       begin GETREAL

        IF( FIRSTIME ) THEN

            PROMPTON = ENVYN( 'PROMPTFLAG', 'Prompt for input flag',
     &                      .TRUE., IOS )
            FIRSTIME = .FALSE.
 
        END IF

        PROMPT2 = PROMPT

        IF( .NOT. PROMPTON ) THEN
            M  =  TRIMLEN( PROMPT )
            GETREAL = DEFAULT
            WRITE( MESG,94030 ) DEFAULT, PROMPT2( 1:M )
            CALL M3MSG2( MESG )
            RETURN
        END IF

        ERRCNT =  0

        LLO  =  MIN ( LO , HI )
        LHI  =  MAX ( LO , HI )
        LDF  =  MIN ( LHI , MAX ( LLO , DEFAULT ) )


100     CONTINUE

        M  =  TRIMLEN( PROMPT2 )

        WRITE ( *,93000 ) PROMPT2( 1:M ) , LDF

        BUFFER =  '           '
        READ ( *, 93100, ERR=900, END=999, IOSTAT=IOS ) BUFFER

        N = INDEX( BUFFER, '!' )
        IF ( N .GT. 0 ) BUFFER( N : LEN( BUFFER ) ) = ' '

        IF ( BUFFER ( 1:1 )  .EQ. ' ' )  THEN
            GETREAL =  LDF
            WRITE( MESG,94020 ) PROMPT2( 1:M ), LDF
            CALL M3MSG2( MESG )
        ELSE

C...........   upcase and remove excess white space

            L = 0
            M = 0
            N = 0

            DO  111  K = 1, LEN( BUFFER )

                CH = BUFFER( K:K )
                IF ( CH .EQ. ' '       ) GO TO 111

                L = L + 1
                IF ( CH .EQ. 'e' ) CH = 'E'
                IF ( CH .EQ. 'E' )  M = L
                IF ( CH .EQ. '.' )  N = L
                BUFFER( L:L ) = CH

111         CONTINUE

            DO  122  K = L+1, LEN( BUFFER )
                BUFFER( K:K ) = ' '
122         CONTINUE

            IF ( N .EQ. 0 ) THEN		!  no decimal:  integer response
                IF ( M .EQ. 0 ) THEN		!  no exponent
                    WRITE( FMTSTR, 94010 ) L
                    READ ( BUFFER, FMTSTR, ERR=400 )  N
                    ANSWER = FLOAT( N )
                ELSE
                    WRITE( FMTSTR, 94010 ) M - 1
                    READ ( BUFFER( 1:M-1 ), FMTSTR, ERR=400 )  N
                    WRITE( FMTSTR, 94010 ) L - M
                    READ ( BUFFER( M+1:L ), FMTSTR, ERR=400 )  M
                    ANSWER = FLOAT( N ) * 10.0 ** M
                END IF
            ELSE
                IF ( M .EQ. 0 ) THEN		!  F format response
                    WRITE( FMTSTR, 94011 ) 'F', L, L - N
                ELSE IF ( M .GT. N ) THEN	!  E format response
                    WRITE( FMTSTR, 94011 ) 'E',  L, M - N
                ELSE				!  bad response
                    GO TO  900
                END IF
                READ ( BUFFER, FMTSTR, ERR=400 )  ANSWER
            END IF


            IF ( ANSWER .LT. LLO  .OR.  ANSWER .GT. LHI )  THEN
                ERRCNT  =  ERRCNT + 1
                WRITE ( *,92100 ) ANSWER , LLO , LHI , ERRCNT
                IF ( ERRCNT .LT. 5 )  GO TO  100

                GO TO  500      !  max error count exceeded

            END IF

            GETREAL =  ANSWER
            WRITE( MESG,94020 ) PROMPT2( 1:M ), ANSWER
            CALL M3MSG2( MESG )

        END IF

        RETURN


400     CONTINUE        !  error in read from BUFFER

        ERRCNT  =  ERRCNT + 1
        WRITE ( *,92200 )  ERRCNT
        IF ( ERRCNT .LT. 5 )  GO TO  100


500     CONTINUE        !  more than 5 entry errors

        CALL M3EXIT( 'GETREAL',0,0, 
     &               'Maximum entry-error count exceeded', 2 )


900     CONTINUE        !  error in read from terminal

        ERRCNT  =  ERRCNT + 1
        IF ( IOS .GT. 0 ) THEN
            IF ( ERRCNT .LT. 5 ) THEN
                WRITE ( *,92000 ) IOS , ERRCNT
                GO TO  100
            ELSE
                CALL M3EXIT( 'GETREAL',0,0, 
     &                       'Maximum error count exceeded', 1 )
            END IF
        END IF

999     CONTINUE        !  exit:  end-of-file condition

        CALL M3EXIT( 'GETREAL',0,0, 'END-OF-FILE encountered', 1 )


C................   end body of GETREAL  .......................................


92000   FORMAT ( /5X , '>>> ERROR IN ROUTINE GETREAL <<< ' ,
     &         //10X , 'Error reading response'               ,
     &         /10X  , 'I/O error status number = ' , I3                 ,
     &         /10X  , 'This is error ' , I1 , ' of 5 errors allowed ' ,
     &         /10X  , 'Please try again.'
     &         )

92100   FORMAT ( /5X , '>>>  RESPONSE ERROR  <<<'      ,
     &         //10X , 'Your response '           , 1PE12.5 ,
     &                 ' not in the range '       , 1PE12.5 ,
     &                 ' ... ' , 1PE12.5 ,
     &         /10X  , 'This is error ' , I1 , ' of 5 errors allowed ' ,
     &         /10X  , 'Please try again.'            , /
     &         )

92200   FORMAT ( /5X , '>>>  RESPONSE ERROR  <<<'         ,
     &         //10X , 'Did not understand your response' ,
     &         /10X  , 'This is error ' , I1 , ' of 5 errors allowed ' ,
     &         /10X  , 'Please try again.'                , /
     &         )

92300   FORMAT ( /5X, A , 2X, 1PE12.5, :, 2X, A, /5X, 3A )

93000   FORMAT ( /5X , A , ' [' , 1PE12.5 , '] >> ' , $ )

93100   FORMAT ( A64 )

94010	FORMAT ( '(I', I3, ')' )

94011	FORMAT ( '(', A1, I3, '.', I3 , ')' )

94020	FORMAT ( 10 ( A, :, 1PE12.5 , :, 2X ) )

94030	FORMAT ( 'Using default response', 1PE12.5 , 2X, 
     &           'for query "', A, '"' )

        END

C................   end   GETREAL  ....................................

