
C Version "%W% %P% %G% %U%
C.........................................................................
C Version "%W% %P% %G% %U%"
C MCPL output module for MM5.  Copyright (C) 1999 MCNC
C Distributed under the GNU LESSER GENERAL PUBLIC LICENSE version 2.1
C See file "LGPL.txt" for conditions of use.
C.........................................................................

        SUBROUTINE CLDFRAC( NCOLS, NROWS, NLAYS,
     &                      PRES, RHLW, CFRACL, CFRACM, CFRACH )

C***********************************************************************
C  subroutine body starts at line  77
C
C  FUNCTION:
C       Compute the cross point cloud fractions for the following
C       pressure groupings:
C               low (970-800 mb), 
C               middle (800-450 mb),
C               high level (above 450 mb) 
C       using the MM5 sfcrad.F relative humidity based algorithm,
C       from 3D pressure and relative humidity, 
C
C  PRECONDITIONS REQUIRED:
C       PRES and RHLW already computed in I/O API order by MM2THERM3().
C
C  SUBROUTINES AND FUNCTIONS CALLED:
C       none
C
C  REVISION  HISTORY:
C       Prototype 12/2001 by Carlie J. Coats, Jr., MCNC EMC.
C***********************************************************************

      IMPLICIT NONE

C...........   ARGUMENTS and their descriptions:

        INTEGER         NCOLS   !  output dimension
        INTEGER         NROWS   !  " 
        INTEGER         NLAYS   !  "
        REAL            PRES( NCOLS, NROWS, NLAYS ) !  pressure (cb)
        REAL            RHLW( NCOLS, NROWS, NLAYS ) !  rh, 2-phase version
        REAL            CFRACL( NCOLS, NROWS ) !  cloud fraction
        REAL            CFRACM( NCOLS, NROWS ) !  rh, liq. water version
        REAL            CFRACH( NCOLS, NROWS ) !  rh, liq. water version

C...........   PARAMETERs

        REAL	    	MBTOPA
        REAL            PRESBOT
        REAL            PRESMID
        REAL            PRESTOP
        REAL            BLIQ
        REAL            CLIQ
        REAL            DLIQ
        REAL            AICE
        REAL            BICE
        REAL            CICE
        REAL            DICE
        REAL            F622

        PARAMETER     ( MBTOPA  = 1.0E2 ,
     &                  PRESBOT =  970.0 * MBTOPA ,
     &                  PRESMID =  800.0 * MBTOPA ,
     &                  PRESTOP =  450.0 * MBTOPA  )

C...........   SCRATCH LOCAL VARIABLES and their descriptions:

	INTEGER		C, R, L		        !  IOAPI col subscript
        REAL            RHBOT, RHMID, RHTOP


C***********************************************************************
C   begin body of subroutine  MM2DPT

        DO  R = 1, NROWS
        DO  C = 1, NCOLS

            RHBOT = 0.0
            RHMID = 0.0
            RHTOP = 0.0

            DO  L = 1, NLAYS
                IF      (  PRES( C,R,L ) .LE. PRESTOP ) THEN
                    RHTOP = MAX( RHLW( C,R,L ), RHTOP )
                ELSE IF (  PRES( C,R,L ) .LE. PRESMID ) THEN
                    RHMID = MAX( RHLW( C,R,L ), RHMID )
                ELSE IF (  PRES( C,R,L ) .LE. PRESBOT ) THEN
                    RHBOT = MAX( RHLW( C,R,L ), RHBOT )
                END IF
            END DO

            !!  Note rescaling ("...E-2") of humidity from percent
            !!  to humidity-fraction...

            CFRACL( C,R ) = MIN( MAX( 4.0E-2 * RHBOT - 3.0, 0.0 ), 1.0 )
            CFRACM( C,R ) = MIN( MAX( 4.0E-2 * RHMID - 3.0, 0.0 ), 1.0 )
            CFRACH( C,R ) = MIN( MAX( 2.5E-2 * RHTOP - 1.5, 0.0 ), 1.0 )
            
        END DO
        END DO

        RETURN
        END

