
C Version "%W% %P% %G% %U%

        SUBROUTINE  GETVBLES( VFILE, NLIST, VLIST, 
     &                        NVARS, VNAMES, VFLAG )

C***********************************************************************
C  subroutine body starts at line  70
C
C  FUNCTION:  Open file VFILE containing list of variable-names
C	(one per line), read it, check the names produced against
C	VLIST, save the matches in VNAMES and count them in NVARS.
C	Log warning message for each mis-match.
C
C  PRECONDITIONS REQUIRED:
C
C
C  SUBROUTINES AND FUNCTIONS CALLED:
C	GETEFILE(), INDEX1()
C
C  REVISION  HISTORY:
C	Prototype 10/17/97 by Carlie J. Coats, Jr.
C       Bug-fix   02/09/99 by CJC:  need "GT" at line 121
C***********************************************************************

      IMPLICIT NONE

C...........   INCLUDES:

      INCLUDE 'PARMS3.EXT'      ! I/O API constants

C...........   ARGUMENTS and their descriptions:

        CHARACTER*16	VFILE		!  logical name for input file
        INTEGER		NLIST		!  input dimension
        CHARACTER*16	VLIST( NLIST )	!  input names to check against
        INTEGER         NVARS		!  number of output names
        CHARACTER*16    VNAMES( MXVARS3 )	  !  output names
        LOGICAL		VFLAG

C...........   PARAMETER

        CHARACTER*16    NONE
        PARAMETER (     NONE  = 'NONE' )

C...........   EXTERNAL FUNCTIONS and their descriptions:

        INTEGER		GETEFILE
        INTEGER		INDEX1
        INTEGER		LBLANK
        INTEGER		TRIMLEN
        
        EXTERNAL	GETEFILE, INDEX1, LBLANK, TRIMLEN


C...........   SCRATCH LOCAL VARIABLES and their descriptions:

        INTEGER		IUNIT
        INTEGER		IREC
        INTEGER		ISTAT
        INTEGER		LO, HI

        CHARACTER*80    LINE
        CHARACTER*256	ANAME
        CHARACTER*256	MESG

C***********************************************************************
C   begin body of subroutine  GETVBLES

        NVARS = 0
        CALL ENVSTR( VFILE, 'Output variables list', 
     &               NONE, ANAME, ISTAT )
        IF ( ISTAT .GT. 0 ) THEN
            MESG = 'Bad environment variable "' //
     &             VFILE( 1:TRIMLEN( VFILE ) ) // '"'
            VFLAG = .FALSE.
            RETURN
        END IF
        
        CALL UPCASE( ANAME )
        IF ( ANAME( 1:16 ) .EQ. NONE ) THEN
            VFLAG = .FALSE.
            RETURN
        END IF
        
        IREC  = 0

        IUNIT = GETEFILE( VFILE, .TRUE., .TRUE., 'GETVBLES' )
        IF ( IUNIT .LT. 0 ) THEN
            MESG = 'Could not open "' // 
     &             VFILE( 1:TRIMLEN( VFILE ) ) // '"'
            CALL M3MESG( MESG ) 
            VFLAG = .FALSE.
            RETURN
        END IF

11      CONTINUE

            READ( IUNIT, 93000, END=99, IOSTAT=ISTAT ) LINE
            IREC = IREC + 1
            IF ( ISTAT .NE. 0 ) THEN
                WRITE( MESG,94000 ) 
     &              'Error', ISTAT, 
     &              'at line', IREC, 
     &              'in file "' , VFILE( 1:TRIMLEN( VFILE ) ) , '"'
                CALL M3MESG( MESG )
            END IF

            LO = LBLANK( LINE ) + 1
            HI = TRIMLEN( LINE )

            IF ( HI - LO .GT. NAMLEN3 ) THEN	!  too-long name

                    WRITE( MESG,94020 )
     &                  'Variable name "' , LINE( 1:TRIMLEN( LINE ) ),
     &                  'at line', IREC,
     &                  'in file "' , VFILE( 1:TRIMLEN( VFILE ) ) , 
     &                  '" too long'
                    CALL M3MESG( MESG )

            ELSE IF ( LO .GT. HI ) THEN	!  blank name

                WRITE( MESG,94010 ) 
     &              'No entry at line', IREC,
     &              'in file "' , VFILE( 1:TRIMLEN( VFILE ) ) , '"'
                CALL M3MESG( MESG )

            ELSE		!  else name OK

                ANAME = LINE( LO:HI )
                IF ( INDEX1( ANAME, NLIST, VLIST ) .GT. 0 ) THEN
                    NVARS = NVARS + 1
                    IF ( NVARS .LE. NLIST ) THEN
                        VNAMES( NVARS ) = ANAME
                    ELSE
                        WRITE( MESG,94020 )
     &                      'Table overflow in file "' ,
     &                      VFILE( 1:TRIMLEN( VFILE ) ) ,
     &                      '" at line', IREC
                        CALL M3MESG( MESG )
                    END IF
                ELSE
                    WRITE( MESG,94020 )
     &                  'Variable "' , ANAME( 1:TRIMLEN( ANAME ) ),
     &                  'at line', IREC,
     &                  'in file "' , VFILE( 1:TRIMLEN( VFILE ) ) , 
     &                  '" not found in input-list'
                    CALL M3MESG( MESG )
                END IF

            END IF	!  if name nonblank, or not

            GO TO  11	! to head of loop

99      CONTINUE

        CLOSE( IUNIT )
        VFLAG = ( ( NVARS .GT. 0 ) .AND. ( NVARS .LE. NLIST ) )

        RETURN

C******************  FORMAT  STATEMENTS   ******************************

C...........   Formatted file I/O formats............ 93xxx

93000	FORMAT ( A )

C...........   Internal buffering formats............ 94xxx

94000	FORMAT ( A, :, I9, :, 2X, A, :, I9, :, 2X, 3 A )

94010	FORMAT ( A, :, I9, :, 2X, 3 A )

94020	FORMAT ( 3 A, :, I9, :, 2X, 3 A )

        END

