C.........................................................................
C   Version "%W% %P% %G% %U%"
C   KFCPL output module for MM5.
C   Copyright (C) 2004 Baron Advanced Meteorological Systems
C   All rights reserved.
C.........................................................................

        SUBROUTINE KFCPL

C----------------------------------------------------------------
C  ENTRY        KF_INIT starts at line   258
C  ENTRY        KF_PUT  starts at line   896
C  ENTRY        KF_CHK  starts at line  1243
C  ENTRY        KF_OUT  starts at line  1275
C
C  FUNCTION:
C
C       Buffering system for KF events.  Accumulates up to NRECS=10000
C       KF-events, and then write them out.
C
C       KF_INIT:
C           Initialize KF_SCA and KF_PRO files for MCPL, and
C           sets up internal buffers and counters.
C
C       KF_PUT:
C           Accumulate KF events internally, and return.
C
C       KF_CHK:
C           Writes out accumulated KF event data if appropriate 
C           to prevent overflow (conditional fall-through to KFOUT)
C
C       KF_OUT:
C           Writes out accumulated KF event data, and resets internal buffers.
C           Should be called at least once per simulation hour.
C
C  PRECONDITIONS REQUIRED:
C       Layer-structure already set up in FDESC3 data structures.
C
C  REVISION  HISTORY:
C       Protoytype 5/2004 by Carlie J. Coats, Jr. and John N. McHenry, BAMS
C
C----------------------------------------------------------------

        IMPLICIT NONE

C...........   INCLUDES:

        INCLUDE 'PARMS3.EXT'      ! I/O API constants
        INCLUDE 'FDESC3.EXT'      ! I/O API file description data structure
        INCLUDE 'IODECL3.EXT'     ! I/O API function declarations
        INCLUDE 'STATE3.EXT'      ! I/O API internal state
#include "parame.incl"
#include "various.incl"
#include "addmdl3.incl"


C...........   ARGUMENTS and their descriptions:
C...............   For KF_INIT

        INTEGER         SDATE0          ! in:  model-run starting date
        INTEGER         STIME0          ! in:  model-run starting time
        CHARACTER*16    GNAMES( 16 ) ! in:  cross-grid names
        INTEGER         SDATES( 16 ) ! in:  starting dates
        INTEGER         STIMES( 16 ) ! in:  starting times
        INTEGER         CCRO0( 16 )  ! in:  col-offsets
        INTEGER         RCRO0( 16 )  ! in:  starting corner rows
        INTEGER         NCCRO( 16 )  ! in:  col-offsets
        INTEGER         NRCRO( 16 )  ! in:  starting corner rows
        INTEGER         NTHIK( 16 )  ! in:  kf events-per-cell

C...............   For KF_PUT, KF_CHK, KF_OUT

        INTEGER         NEST    ! in:  MM5 nest number, 1=coarse domain
        INTEGER         II, JJ  !  MM5 I and J values
        REAL            P300KF  !  pressure 300 mb above sfc at cloud inception
        REAL            TCKF    !  duration of this cloud in seconds
        REAL            RKF     !  Precip originating as liquid water in cloud
        REAL            ICEKF   !  Precip originating as ice in cloud
        REAL            TADVEC  !  advection transit time, seconds
        INTEGER         NIC     !  number of MM5 advection steps for TADVEC
        REAL            CLDTP   !  KF cloud top, meters a.g.l.
        REAL            TKF0( MKX )     !  Temperature, K
        REAL            QKF0( MKX )     !  Water Vapor, kg/kg
        REAL            UKF0( MKX )     !  U-component of wind velocity, M/S
        REAL            VKF0( MKX )     !  V-", M/S
        REAL            WAV0( MKX )     !  averaged W-", M/S
        REAL            PKF0( MKX )     !  Presssure, PA
        REAL            DZQKF( MKX )    !  full-sigma layer thickness, m
        REAL            DPKF0( MKX )    !  full-sigma pressure thickess, PA


C...........   EXTERNAL FUNCTIONS and their descriptions:

        LOGICAL     DSCGRID, KFOPEN
        INTEGER     ENVYN, INDEX1, RDVARS, SEC2TIME, SECSDIFF

        EXTERNAL    DSCGRID, ENVYN, INDEX1, KFOPEN, RDVARS,
     &              SEC2TIME, SECSDIFF


C...........   PARAMETERS and their descriptions:

        INTEGER         KFRECS            !  max # of kf events in memory
        INTEGER         KFTHIK            !  max # of kf events per cell per call
        PARAMETER (     KFRECS = 60 000,
     &                  KFTHIK = 60 )

        CHARACTER*16    KF_SCA_G1, KF_PRO_G1
        CHARACTER*16    KF_SCA_G3, KF_PRO_G3
        CHARACTER*16    KF_SCA_G4, KF_PRO_G4
        CHARACTER*16    KF_SCA_G5, KF_PRO_G5
        CHARACTER*16    KF_SCA_G6, KF_PRO_G6

        PARAMETER     ( KF_SCA_G1 = 'MET_KF_SCA_G1',
     &                  KF_PRO_G1 = 'MET_KF_PRO_G1',
     &                  KF_SCA_G3 = 'MET_KF_SCA_G3',
     &                  KF_PRO_G3 = 'MET_KF_PRO_G3',
     &                  KF_SCA_G4 = 'MET_KF_SCA_G4',
     &                  KF_PRO_G4 = 'MET_KF_PRO_G4',
     &                  KF_SCA_G5 = 'MET_KF_SCA_G5',
     &                  KF_PRO_G5 = 'MET_KF_PRO_G5',
     &                  KF_SCA_G6 = 'MET_KF_SCA_G6',
     &                  KF_PRO_G6 = 'MET_KF_PRO_G6' )

        INTEGER         NCOLS1, NROWS1, NTHIK1
        INTEGER         NCOLSC, NROWSC, NTHIKC  ! not "*3"because of STATE3
        INTEGER         NCOLS4, NROWS4, NTHIK4
        INTEGER         NCOLS5, NROWS5, NTHIK5
        INTEGER         NCOLS6, NROWS6, NTHIK6

        PARAMETER     ( NCOLS1 = 63, NROWS1 = 67, NTHIK1 = 200,
     &                  NCOLSC = 70, NROWSC = 90, NTHIKC = 200,
     &                  NCOLS4 = 40, NROWS4 = 45, NTHIK4 = 200,
     &                  NCOLS5 = 42, NROWS5 = 60, NTHIK5 = 200,
     &                  NCOLS6 = 72, NROWS6 = 93, NTHIK6 = 200 )

        REAL            D60    	        !  conversion factor:  min::sec
        PARAMETER (     D60 = 1.0 / 60.0 )


C...........   STATE VARIABLES and their descriptions:
C...............   KFCPL-active flag:

        LOGICAL         KFCPL_ACTIVE
        DATA            KFCPL_ACTIVE        / .FALSE. /
        SAVE            KFCPL_ACTIVE

C...............   Model-run starting date&time

        INTEGER         MDATE0, MTIME0
        SAVE            MDATE0, MTIME0

C...............   Per-window state variables:

        INTEGER         G1RECS, G3RECS, G4RECS, G5RECS, G6RECS
        INTEGER         G1REC0, G3REC0, G4REC0, G5REC0, G6REC0

        !!  Per-window grid bounds for I, J

        INTEGER         G1I0, G1J0, G1I1, G1J1
        INTEGER         G3I0, G3J0, G3I1, G3J1
        INTEGER         G4I0, G4J0, G4I1, G4J1
        INTEGER         G5I0, G5J0, G5I1, G5J1
        INTEGER         G6I0, G6J0, G6I1, G6J1

        INTEGER         G1AFID,  G3AFID,  G4AFID,  G5AFID,   G6AFID
        INTEGER         G1PFID,  G3PFID,  G4PFID,  G5PFID,   G6PFID
        INTEGER         SDATEG1, SDATEG3, SDATEG4, SDATEG5,  SDATEG6
        INTEGER         STIMEG1, STIMEG3, STIMEG4, STIMEG5,  STIMEG6

        INTEGER         G1NRECS( NCOLS1, NROWS1 )
        INTEGER         G1COUNT( NCOLS1, NROWS1 )
        INTEGER         G1EVENT( KFTHIK, NCOLS1, NROWS1 )
        INTEGER         G1SDATE( KFTHIK, NCOLS1, NROWS1 )
        INTEGER         G1STIME( KFTHIK, NCOLS1, NROWS1 )
        INTEGER         G1LNGTH( KFTHIK, NCOLS1, NROWS1 )

        INTEGER         G3NRECS( NCOLSC, NROWSC )
        INTEGER         G3COUNT( NCOLSC, NROWSC )
        INTEGER         G3EVENT( KFTHIK, NCOLSC, NROWSC )
        INTEGER         G3SDATE( KFTHIK, NCOLSC, NROWSC )
        INTEGER         G3STIME( KFTHIK, NCOLSC, NROWSC )
        INTEGER         G3LNGTH( KFTHIK, NCOLSC, NROWSC )

        INTEGER         G4NRECS( NCOLS4, NROWS4 )
        INTEGER         G4COUNT( NCOLS4, NROWS4 )
        INTEGER         G4EVENT( KFTHIK, NCOLS4, NROWS4 )
        INTEGER         G4SDATE( KFTHIK, NCOLS4, NROWS4 )
        INTEGER         G4STIME( KFTHIK, NCOLS4, NROWS4 )
        INTEGER         G4LNGTH( KFTHIK, NCOLS4, NROWS4 )

        INTEGER         G5NRECS( NCOLS5, NROWS5 )
        INTEGER         G5COUNT( NCOLS5, NROWS5 )
        INTEGER         G5EVENT( KFTHIK, NCOLS5, NROWS5 )
        INTEGER         G5SDATE( KFTHIK, NCOLS5, NROWS5 )
        INTEGER         G5STIME( KFTHIK, NCOLS5, NROWS5 )
        INTEGER         G5LNGTH( KFTHIK, NCOLS5, NROWS5 )

        INTEGER         G6NRECS( NCOLS6, NROWS6 )
        INTEGER         G6COUNT( NCOLS6, NROWS6 )
        INTEGER         G6EVENT( KFTHIK, NCOLS6, NROWS6 )
        INTEGER         G6SDATE( KFTHIK, NCOLS6, NROWS6 )
        INTEGER         G6STIME( KFTHIK, NCOLS6, NROWS6 )
        INTEGER         G6LNGTH( KFTHIK, NCOLS6, NROWS6 )

        COMMON / KF_GRIDS /
     &          G1RECS,  G3RECS,  G4RECS,  G5RECS,  G6RECS,
     &          G1REC0,  G3REC0,  G4REC0,  G5REC0,  G6REC0,
     &          G1AFID,  G3AFID,  G4AFID,  G5AFID,  G6AFID,
     &          G1PFID,  G3PFID,  G4PFID,  G5PFID,  G6PFID,
     &          SDATEG1, SDATEG3, SDATEG4, SDATEG5, SDATEG6,
     &          STIMEG1, STIMEG3, STIMEG4, STIMEG5, STIMEG6,
     &          G1I0,    G3I0,    G4I0,    G5I0,    G6I0,
     &          G1J0,    G3J0,    G4J0,    G5J0,    G6J0,
     &          G1I1,    G3I1,    G4I1,    G5I1,    G6I1,
     &          G1J1,    G3J1,    G4J1,    G5J1,    G6J1,
     &          G1NRECS, G1COUNT, G1EVENT, G1SDATE, G1STIME, G1LNGTH,
     &          G3NRECS, G3COUNT, G3EVENT, G3SDATE, G3STIME, G3LNGTH,
     &          G4NRECS, G4COUNT, G4EVENT, G4SDATE, G4STIME, G4LNGTH,
     &          G5NRECS, G5COUNT, G5EVENT, G5SDATE, G5STIME, G5LNGTH,
     &          G6NRECS, G6COUNT, G6EVENT, G6SDATE, G6STIME, G6LNGTH

C...............   Data, time-flag buffers:

        REAL    KABUF(      KFRECS, 7, 5 )
        REAL    KPBUF( MKX, KFRECS, 8, 5 )
        INTEGER TFLAG( 5,   KFRECS, 5 )

        COMMON / KF_STEPS /    KABUF, KPBUF, TFLAG
        
        SAVE   / KF_GRIDS /, / KF_STEPS /


C...........   SCRATCH VARIABLES and their descriptions:

        INTEGER         FNUM, SCANUM, PRONUM, IREC, IERR, MXROWS2
        INTEGER         I, J, L, N, NN, C, R, CC, RR, V
        INTEGER         KDATE, KTIME, KSTEP, KFLEN, KSECS
        INTEGER         DIMS( 5 ), DELS( 5 )
        LOGICAL         EFLAG
        CHARACTER*16    NAMBUF
        CHARACTER*256   MESG


C......................................................................
C...............   Begin subroutine KFCPL:
C...............   Should never call KFCPL() directly:

        MESG = 'Incorrect direct call to KFCPL():  ' //
     &  'call KF_INIT(), then KF_PUT() and KF_OUT() instead'
        CALL M3EXIT( 'MM5:KFCPL', 0, 0, MESG, 2 )


C......................................................................
C...............   KF Initialization Segment:

        ENTRY KF_INIT( SDATE0, STIME0, GNAMES, SDATES, STIMES,
     &                 CCRO0, RCRO0, NCCRO, NRCRO, NTHIK )

        MDATE0 = SDATE0
        MTIME0 = STIME0

C..........   KF_SCA files for nests 1,2:

        FTYPE3D = KFEVNT3
        NVARS3D = 7
        NLAYS3D = 1

        VTYPE3D(1) = M3REAL
        VNAME3D(1) = 'P300'
        UNITS3D(1) = 'Pa'
        VDESC3D(1) = 'pressure 300 mb above surface'

        VTYPE3D(2) = M3REAL
        VNAME3D(2) = 'RAINKFCM'
        UNITS3D(2) = 'cm'
        VDESC3D(2) =
     &        'Precipitation originating as liquid water in cloud'

        VTYPE3D(3) = M3REAL
        VNAME3D(3) = 'PRICEKFCM'
        UNITS3D(3) = 'cm'
        VDESC3D(3) = 'Precip originating as ice in cloud'

        VTYPE3D(4) = M3REAL
        VNAME3D(4) = 'XTIME'
        UNITS3D(4) = 'sec'
        VDESC3D(4) = 'MM5 elapsed time since start of forecast'

        VTYPE3D(5) = M3REAL
        VNAME3D(5) = 'TIMEC'
        UNITS3D(5) = 'sec'
        VDESC3D(5) = 'duration of this cloud in seconds'

        VTYPE3D(6) = M3REAL
        VNAME3D(6) = 'TADVEC'
        UNITS3D(6) = 'sec'
        VDESC3D(6) = 'transit time needed to advect cloud through cell'

        VTYPE3D(7) = M3REAL
        VNAME3D(7) = 'NIC'
        UNITS3D(7) = 'none'
        VDESC3D(7) = 'number of MM5 advection steps for TADVEC'

        FDESC3D( 1 ) = 'Scalar KF cloud event inception data'

!$OMP       PARALLEL DO DEFAULT( NONE ),
!$OMP&                   SHARED( KFTOP, KFTIM, KFEXP ),
!$OMP&                  PRIVATE( I,J )

        DO  J = 1, MJX
        DO  I = 1, MIX
            KFTOP( I,J,1 ) = 0
            KFTIM( I,J,1 ) = 0
            KFEXP( I,J,1 ) = 0
            KFTOP( I,J,2 ) = 0
            KFTIM( I,J,2 ) = 0
            KFEXP( I,J,2 ) = 0
        END DO
        END DO

        !!~~~~~~~~~~~~~~~~~~>>  open file  KF_SCA_G1:

        NN = 1

        IF ( NCOLS1 .NE. NCCRO( 2 ) ) THEN
            WRITE( MESG, '( A, I10, 2X, A, I10 )')
     &          'Cols mismatch:  dim=', NCOLS1, 'caller=', NCCRO(2)
            CALL M3EXIT( 'MM5/KFCPL', 0, 0, MESG, 2 )
        ELSE IF ( NROWS1 .NE. NRCRO( 2 ) ) THEN
            WRITE( MESG, '( A, I10, 2X, A, I10 )')
     &          'Rows mismatch:  dim=', NROWS1, 'caller=', NRCRO(2)
            CALL M3EXIT( 'MM5/KFCPL', 0, 0, MESG, 2 )
        END IF

        G1J0   = CCRO0( 2 ) - 1
        G1I0   = RCRO0( 2 ) - 1
        G1J1   = G1J0 + NCOLS1
        G1I1   = G1I0 + NROWS1

        IF ( .NOT. DSCGRID( GNAMES( 2 ), NAMBUF, GDTYP3D,
     &                      P_ALP3D, P_BET3D, P_GAM3D,
     &                      XCENT3D, YCENT3D, XORIG3D, YORIG3D,
     &                      XCELL3D, YCELL3D,
     &                      NCOLS3D, NROWS3D, NTHIK3D  ) ) THEN
            MESG = 'No grid description for ' // GNAMES( 1 )
            CALL M3EXIT( 'MM5/KFCPL:KF_INIT', 0, 0, MESG, 2 )
        END IF          !  if dscgrid() failed

        GDNAM3D = GNAMES( 2 )
        NLAYS3D = 1
        NTHIK3D = NTHIK ( 2 )
        SDATE3D = SDATES( 2 )
        STIME3D = STIMES( 2 )
        SDATEG1 = SDATE3D
        STIMEG1 = STIME3D
        KSTEP   = -10000
        CALL NEXTIME( SDATE3D, STIME3D, KSTEP )

        IF ( .NOT. KFOPEN( KF_SCA_G1, FSUNKN3, 'MM5/KFCPL',
     &                     G1NRECS ) ) THEN
            MESG = 'Could not open MET_KF_SCA_G1'
            CALL M3EXIT( 'MM5/KFCPL', 0, 0, MESG, 2 )
        END IF
        G1AFID = INDEX1( KF_SCA_G1, MXFILE3, FLIST3 )

        DIMS( 1 ) = 1
        DELS( 1 ) = NTHIK1
        DIMS( 2 ) = 1
        DELS( 2 ) = NCOLS1
        DIMS( 3 ) = 1
        DELS( 3 ) = NROWS1

        G1REC0 = MXREC3( G1AFID )
        G1RECS = 0


        NN = 2     !  KF_SCA for MM5 nest 1, windows G3, G4, G5, G6

        !!~~~~~~~~~~~~~~~~~~>>  open file  KF_SCA_G3:

        IF ( NCOLSC .NE. NCCRO( 4 ) ) THEN
            WRITE( MESG, '( A, I10, 2X, A, I10 )')
     &        'Cols mismatch:  dim=', NCOLSC, 'caller=', NCCRO(4)
            CALL M3EXIT( 'MM5/KFCPL', 0, 0, MESG, 2 )
        ELSE IF ( NROWSC .NE. NRCRO( 4 ) ) THEN
            WRITE( MESG, '( A, I10, 2X, A, I10 )')
     &        'Rows mismatch:  dim=', NROWSC, 'caller=', NRCRO(4)
            CALL M3EXIT( 'MM5/KFCPL', 0, 0, MESG, 2 )
        END IF

        G3J0   = CCRO0( 4 ) - 1
        G3I0   = RCRO0( 4 ) - 1
        G3J1   = G3J0 + NCOLSC
        G3I1   = G3I0 + NROWSC

        IF ( .NOT. DSCGRID( GNAMES( 4 ), NAMBUF, GDTYP3D,
     &                      P_ALP3D, P_BET3D, P_GAM3D,
     &                      XCENT3D, YCENT3D, XORIG3D, YORIG3D,
     &                      XCELL3D, YCELL3D,
     &                      NCOLS3D, NROWS3D, NTHIK3D  ) ) THEN
            MESG = 'No grid description for ' // GNAMES( 4 )
            CALL M3EXIT( 'MM5/KFCPL:KF_INIT', 0, 0, MESG, 2 )
        END IF          !  if dscgrid() failed

        GDNAM3D = GNAMES( 4 )
        NLAYS3D = 1
        NTHIK3D = NTHIK ( 4 )
        SDATE3D = SDATES( 4 )
        STIME3D = STIMES( 4 )
        SDATEG3 = SDATE3D
        STIMEG3 = STIME3D
        KSTEP   = -10000
        CALL NEXTIME( SDATE3D, STIME3D, KSTEP )

        IF ( .NOT. KFOPEN( KF_SCA_G3, FSUNKN3, 'MM5/KFCPL',
     &                     G3NRECS ) ) THEN
            MESG = 'Could not open MET_KF_SCA_G3'
            CALL M3EXIT( 'MM5/KFCPL', 0, 0, MESG, 2 )
        END IF
        G3AFID = INDEX1( KF_SCA_G3, MXFILE3, FLIST3 )

        DIMS( 1 ) = 1
        DELS( 1 ) = NTHIKC
        DIMS( 2 ) = 1
        DELS( 2 ) = NCOLSC
        DIMS( 3 ) = 1
        DELS( 3 ) = NROWSC

        G3REC0 = MXREC3( G3AFID )
        G3RECS = 0


        !!~~~~~~~~~~~~~~~~~~>>  open file  KF_SCA_G4:

        IF ( NCOLS4 .NE. NCCRO( 5 ) ) THEN
            WRITE( MESG, '( A, I10, 2X, A, I10 )')
     &        'Cols mismatch:  dim=', NCOLS4, 'caller=', NCCRO(5)
            CALL M3EXIT( 'MM5/KFCPL', 0, 0, MESG, 2 )
        ELSE IF ( NROWS4 .NE. NRCRO( 5 ) ) THEN
            WRITE( MESG, '( A, I10, 2X, A, I10 )')
     &        'Rows mismatch:  dim=', NROWS4, 'caller=', NRCRO(5)
            CALL M3EXIT( 'MM5/KFCPL', 0, 0, MESG, 2 )
        END IF

        G4J0   = CCRO0( 5 ) - 1
        G4I0   = RCRO0( 5 ) - 1
        G4J1   = G4J0 + NCOLS4
        G4I1   = G4I0 + NROWS4

        IF ( .NOT. DSCGRID( GNAMES( 5 ), NAMBUF, GDTYP3D,
     &                      P_ALP3D, P_BET3D, P_GAM3D,
     &                      XCENT3D, YCENT3D, XORIG3D, YORIG3D,
     &                      XCELL3D, YCELL3D,
     &                      NCOLS3D, NROWS3D, NTHIK3D  ) ) THEN
            MESG = 'No grid description for ' // GNAMES( 5 )
            CALL M3EXIT( 'MM5/KFCPL:KF_INIT', 0, 0, MESG, 2 )
        END IF          !  if dscgrid() failed

        GDNAM3D = GNAMES( 5 )
        NLAYS3D = 1
        NTHIK3D = NTHIK ( 5 )
        SDATE3D = SDATES( 5 )
        STIME3D = STIMES( 5 )
        SDATEG4 = SDATE3D
        STIMEG4 = STIME3D
        KSTEP   = -10000
        CALL NEXTIME( SDATE3D, STIME3D, KSTEP )

        IF ( .NOT. KFOPEN( KF_SCA_G4, FSUNKN3, 'MM5/KFCPL',
     &                     G4NRECS ) ) THEN
            MESG = 'Could not open MET_KF_SCA_G4'
            CALL M3EXIT( 'MM5/KFCPL', 0, 0, MESG, 2 )
        END IF
        G4AFID = INDEX1( KF_SCA_G4, MXFILE3, FLIST3 )

        DIMS( 1 ) = 1
        DELS( 1 ) = NTHIK4
        DIMS( 2 ) = 1
        DELS( 2 ) = NCOLS4
        DIMS( 3 ) = 1
        DELS( 3 ) = NROWS4

        G4REC0 = MXREC3( G4AFID )
        G4RECS = 0


        !!~~~~~~~~~~~~~~~~~~>>  open file  KF_SCA_G5:

        IF ( NCOLS5 .NE. NCCRO( 6 ) ) THEN
            WRITE( MESG, '( A, I10, 2X, A, I10 )')
     &        'Cols mismatch:  dim=', NCOLS5, 'caller=', NCCRO(6)
            CALL M3EXIT( 'MM5/KFCPL', 0, 0, MESG, 2 )
        ELSE IF ( NROWS5 .NE. NRCRO( 6 ) ) THEN
            WRITE( MESG, '( A, I10, 2X, A, I10 )')
     &        'Rows mismatch:  dim=', NROWS5, 'caller=', NRCRO(6)
            CALL M3EXIT( 'MM5/KFCPL', 0, 0, MESG, 2 )
        END IF

        G5J0   = CCRO0( 6 ) - 1
        G5I0   = RCRO0( 6 ) - 1
        G5J1   = G5J0 + NCOLS5
        G5I1   = G5I0 + NROWS5

        IF ( .NOT. DSCGRID( GNAMES( 6 ), NAMBUF, GDTYP3D,
     &                      P_ALP3D, P_BET3D, P_GAM3D,
     &                      XCENT3D, YCENT3D, XORIG3D, YORIG3D,
     &                      XCELL3D, YCELL3D,
     &                      NCOLS3D, NROWS3D, NTHIK3D  ) ) THEN
            MESG = 'No grid description for ' // GNAMES( 6 )
            CALL M3EXIT( 'MM5/KFCPL:KF_INIT', 0, 0, MESG, 2 )
        END IF          !  if dscgrid() failed

        GDNAM3D = GNAMES( 6 )
        NLAYS3D = 1
        NTHIK3D = NTHIK ( 6 )
        SDATE3D = SDATES( 6 )
        STIME3D = STIMES( 6 )
        SDATEG5 = SDATE3D
        STIMEG5 = STIME3D
        KSTEP   = -10000
        CALL NEXTIME( SDATE3D, STIME3D, KSTEP )

        IF ( .NOT. KFOPEN( KF_SCA_G5, FSUNKN3, 'MM5/KFCPL',
     &                     G5NRECS ) ) THEN
            MESG = 'Could not open MET_KF_SCA_G5'
            CALL M3EXIT( 'MM5/KFCPL', 0, 0, MESG, 2 )
        END IF
        G5AFID = INDEX1( KF_SCA_G5, MXFILE3, FLIST3 )

        DIMS( 1 ) = 1
        DELS( 1 ) = NTHIK5
        DIMS( 2 ) = 1
        DELS( 2 ) = NCOLS5
        DIMS( 3 ) = 1
        DELS( 3 ) = NROWS5

        G5REC0 = MXREC3( G5AFID )
        G5RECS = 0


        !!~~~~~~~~~~~~~~~~~~>>  open file  KF_SCA_G6:

        IF ( NCOLS6 .NE. NCCRO( 7 ) ) THEN
            WRITE( MESG, '( A, I10, 2X, A, I10 )')
     &        'Cols mismatch:  dim=', NCOLS6, 'caller=', NCCRO(7)
            CALL M3EXIT( 'MM5/KFCPL', 0, 0, MESG, 2 )
        ELSE IF ( NROWS6 .NE. NRCRO( 7 ) ) THEN
            WRITE( MESG, '( A, I10, 2X, A, I10 )')
     &        'Rows mismatch:  dim=', NROWS6, 'caller=', NRCRO(7)
            CALL M3EXIT( 'MM5/KFCPL', 0, 0, MESG, 2 )
        END IF

        G6J0   = CCRO0( 7 ) - 1
        G6I0   = RCRO0( 7 ) - 1
        G6J1   = G6J0 + NCOLS6
        G6I1   = G6I0 + NROWS6
        IF ( .NOT. DSCGRID( GNAMES( 7 ), NAMBUF, GDTYP3D,
     &                      P_ALP3D, P_BET3D, P_GAM3D,
     &                      XCENT3D, YCENT3D, XORIG3D, YORIG3D,
     &                      XCELL3D, YCELL3D,
     &                      NCOLS3D, NROWS3D, NTHIK3D  ) ) THEN
            MESG = 'No grid description for ' // GNAMES( 7 )
            CALL M3EXIT( 'MM5/KFCPL:KF_INIT', 0, 0, MESG, 2 )
        END IF          !  if dscgrid() failed

        GDNAM3D = GNAMES( 7 )
        NLAYS3D = 1
        NTHIK3D = NTHIK ( 7 )
        SDATE3D = SDATES( 7 )
        STIME3D = STIMES( 7 )
        SDATEG6 = SDATE3D
        STIMEG6 = STIME3D
        KSTEP   = -10000
        CALL NEXTIME( SDATE3D, STIME3D, KSTEP )

        IF ( .NOT. KFOPEN( KF_SCA_G6, FSUNKN3, 'MM5/KFCPL',
     &                     G6NRECS ) ) THEN
            MESG = 'Could not open MET_KF_SCA_G6'
            CALL M3EXIT( 'MM5/KFCPL', 0, 0, MESG, 2 )
        END IF
        G6AFID = INDEX1( KF_SCA_G6, MXFILE3, FLIST3 )

        DIMS( 1 ) = 1
        DELS( 1 ) = NTHIK6
        DIMS( 2 ) = 1
        DELS( 2 ) = NCOLS6
        DIMS( 3 ) = 1
        DELS( 3 ) = NROWS6

        G6REC0 = MXREC3( G6AFID )
        G6RECS = 0


C..........   KF_PRO files for nests 1,2:

        FTYPE3D = KFEVNT3
        NVARS3D = 8
        NLAYS3D = MKX

        VTYPE3D(1) = M3REAL
        VNAME3D(1) = 'T0'
        UNITS3D(1) = 'K'
        VDESC3D(1) = 'Temperature profile at KF cloud inception'

        VTYPE3D(2) = M3REAL
        VNAME3D(2) = 'Q0'
        UNITS3D(2) = 'kg/kg'
        VDESC3D(2) = 'Water Vapor profile at KF cloud inception'

        VTYPE3D(3) = M3REAL
        VNAME3D(3) = 'U0'
        UNITS3D(3) = 'M/S'
        VDESC3D(3) =
     &    'U-component of wind velocity profile at KF cloud inception'

        VTYPE3D(4) = M3REAL
        VNAME3D(4) = 'V0'
        UNITS3D(4) = 'M/S'
        VDESC3D(4) =
     &    'V-component of wind velocity profile at KF cloud inception'

        VTYPE3D(5) = M3REAL
        VNAME3D(5) = 'W0AVG'
        UNITS3D(5) = 'M/S'
        VDESC3D(5) =
     &    'Running avg. W-component of wind vel at KF cloud inception'

        VTYPE3D(6) = M3REAL
        VNAME3D(6) = 'P0'
        UNITS3D(6) = 'Pa'
        VDESC3D(6) =
     &         'Pressure profile at KF cloud inception'

        VTYPE3D(7) = M3REAL
        VNAME3D(7) = 'DZQ'
        UNITS3D(7) = 'm'
        VDESC3D(7) =
     &         'Internal KF representation of thickness of ' //
     &         'full-sigma layers'

        VTYPE3D(8) = M3REAL
        VNAME3D(8) = 'DP'
        UNITS3D(8) = 'Pa'
        VDESC3D(8) =
     &         'Internal KF representation of pressure depth of ' //
     &         'full-sigma layers'

        FDESC3D( 1 ) = 'KF cloud event inception vertical profiles'

        NN = 1     !  KF_PRO for nest 1, window G1


        !!~~~~~~~~~~~~~~~~~~>>  open file  KF_PRO_G1:

        IF ( .NOT. DSCGRID( GNAMES( 2 ), NAMBUF, GDTYP3D,
     &                      P_ALP3D, P_BET3D, P_GAM3D,
     &                      XCENT3D, YCENT3D, XORIG3D, YORIG3D,
     &                      XCELL3D, YCELL3D,
     &                      NCOLS3D, NROWS3D, NTHIK3D  ) ) THEN
            MESG = 'No grid description for ' // GNAMES( 1 )
            CALL M3EXIT( 'MM5/KFCPL:KF_INIT', 0, 0, MESG, 2 )
        END IF          !  if dscgrid() failed

        GDNAM3D = GNAMES( 2 )
        NLAYS3D = MKX
        NTHIK3D = NTHIK ( 2 )
        SDATE3D = SDATES( 2 )
        STIME3D = STIMES( 2 )
        SDATEG1 = SDATE3D
        STIMEG1 = STIME3D
        KSTEP   = -10000
        CALL NEXTIME( SDATE3D, STIME3D, KSTEP )

        IF ( .NOT. KFOPEN( KF_PRO_G1, FSUNKN3, 'MM5/KFCPL',
     &                     G1COUNT ) ) THEN
            MESG = 'Could not open MET_KF_PRO_G1'
            CALL M3EXIT( 'MM5/MCPL', 0, 0, MESG, 2 )
        END IF

!$OMP       CRITICAL( S_KFG1 )
        EFLAG = .FALSE.
        DO RR = 1, NROWS1
        DO CC = 1, NCOLS1
            IF ( G1COUNT(CC,RR) .NE. G1NRECS(CC,RR) ) EFLAG=.TRUE.
            G1COUNT(CC,RR) = 0
        END DO
        END DO
!$OMP       END CRITICAL( S_KFG1 )

        IF ( EFLAG ) THEN
            MESG = 'Inconsistent MET_KF_SCA_G1, MET_KF_PRO_G1'
            CALL M3EXIT( 'MM5/KFCPL', 0, 0, MESG, 2 )
        END IF

        G1PFID = INDEX1( KF_PRO_G1, MXFILE3, FLIST3 )


        NN = 2     !  KF_PRO for nest 2, windows G3, G4, G5, G6

        !!~~~~~~~~~~~~~~~~~~>>  open file  KF_PRO_G3:

        SDATE3D = SDATES( 4 )
        STIME3D = STIMES( 4 )
        KSTEP   = -10000
        CALL NEXTIME( SDATE3D, STIME3D, KSTEP )

        IF ( .NOT. DSCGRID( GNAMES( 4 ), NAMBUF, GDTYP3D,
     &                      P_ALP3D, P_BET3D, P_GAM3D,
     &                      XCENT3D, YCENT3D, XORIG3D, YORIG3D,
     &                      XCELL3D, YCELL3D,
     &                      NCOLS3D, NROWS3D, NTHIK3D  ) ) THEN
            MESG = 'No grid description for ' // GNAMES( 4 )
            CALL M3EXIT( 'MM5/KFCPL:KF_INIT', 0, 0, MESG, 2 )
        END IF          !  if dscgrid() failed

        GDNAM3D = GNAMES( 4 )
        NLAYS3D = MKX
        NTHIK3D = NTHIK(4)

        IF ( .NOT. KFOPEN( KF_PRO_G3, FSUNKN3, 'MM5/KFCPL',
     &                     G3COUNT ) ) THEN
            MESG = 'Could not open MET_KF_PRO_G3'
            CALL M3EXIT( 'MM5/MCPL', 0, 0, MESG, 2 )
        END IF

!$OMP       CRITICAL( S_KFG3 )
        EFLAG = .FALSE.
        DO RR = 1, NROWSC
        DO CC = 1, NCOLSC
            IF ( G3COUNT(CC,RR) .NE. G3NRECS(CC,RR) ) EFLAG=.TRUE.
            G3COUNT(CC,RR) = 0
        END DO
        END DO
!$OMP       END CRITICAL( S_KFG3 )

        IF ( EFLAG ) THEN
            MESG = 'Inconsistent MET_KF_SCA_G3, MET_KF_PRO_G3'
            CALL M3EXIT( 'MM5/KFCPL', 0, 0, MESG, 2 )
        END IF

        G3PFID = INDEX1( KF_PRO_G3, MXFILE3, FLIST3 )


        !!~~~~~~~~~~~~~~~~~~>>  open file  KF_PRO_G4:

        SDATE3D = SDATES( 5 )
        STIME3D = STIMES( 5 )
        KSTEP   = -10000
        CALL NEXTIME( SDATE3D, STIME3D, KSTEP )

        IF ( .NOT. DSCGRID( GNAMES( 5 ), NAMBUF, GDTYP3D,
     &                      P_ALP3D, P_BET3D, P_GAM3D,
     &                      XCENT3D, YCENT3D, XORIG3D, YORIG3D,
     &                      XCELL3D, YCELL3D,
     &                      NCOLS3D, NROWS3D, NTHIK3D  ) ) THEN
            MESG = 'No grid description for ' // GNAMES( 5 )
            CALL M3EXIT( 'MM5/KFCPL:KF_INIT', 0, 0, MESG, 2 )
        END IF          !  if dscgrid() failed

        GDNAM3D = GNAMES( 5 )
        NLAYS3D = MKX
        NTHIK3D = NTHIK( 5 )

        IF ( .NOT. KFOPEN( KF_PRO_G4, FSUNKN3, 'MM5/KFCPL',
     &                     G4COUNT ) ) THEN
            MESG = 'Could not open MET_KF_PRO_G4'
            CALL M3EXIT( 'MM5/MCPL', 0, 0, MESG, 2 )
        END IF

!$OMP       CRITICAL( S_KFG4 )
            EFLAG = .FALSE.

CC!$OMP  PARALLEL DO  DEFAULT( NONE ),
CC!$OMP&               SHARED( G4COUNT, G4NRECS ),
CC!$OMP&              PRIVATE( CC, RR ),
CC!$OMP&            REDUCTION( .OR.:  EFLAG )

            DO RR = 1, NROWS4
            DO CC = 1, NCOLS4
                EFLAG = EFLAG .OR. (G4COUNT(CC,RR) .NE. G4NRECS(CC,RR))
                G4COUNT(CC,RR) = 0
            END DO
            END DO
!$OMP       END CRITICAL( S_KFG4 )

            IF ( EFLAG ) THEN
            MESG = 'Inconsistent MET_KF_SCA_G4, MET_KF_PRO_G4'
            CALL M3EXIT( 'MM5/KFCPL', 0, 0, MESG, 2 )
        END IF

        G4PFID = INDEX1( KF_PRO_G4, MXFILE3, FLIST3 )


        !!~~~~~~~~~~~~~~~~~~>>  open file  KF_PRO_G5:

        SDATE3D = SDATES( 6 )
        STIME3D = STIMES( 6 )
        KSTEP   = -10000
        CALL NEXTIME( SDATE3D, STIME3D, KSTEP )

        IF ( .NOT. DSCGRID( GNAMES( 6 ), NAMBUF, GDTYP3D,
     &                      P_ALP3D, P_BET3D, P_GAM3D,
     &                      XCENT3D, YCENT3D, XORIG3D, YORIG3D,
     &                      XCELL3D, YCELL3D,
     &                      NCOLS3D, NROWS3D, NTHIK3D  ) ) THEN
            MESG = 'No grid description for ' // GNAMES( 6 )
            CALL M3EXIT( 'MM5/KFCPL:KF_INIT', 0, 0, MESG, 2 )
        END IF          !  if dscgrid() failed

        GDNAM3D = GNAMES( 6 )
        NLAYS3D = MKX
        NTHIK3D = NTHIK( 6 )

        IF ( .NOT. KFOPEN( KF_PRO_G5, FSUNKN3, 'MM5/KFCPL',
     &                     G5COUNT ) ) THEN
            MESG = 'Could not open MET_KF_PRO_G5'
            CALL M3EXIT( 'MM5/MCPL', 0, 0, MESG, 2 )
        END IF

!$OMP       CRITICAL( S_KFG5 )
            EFLAG = .FALSE.
            DO RR = 1, NROWS5
            DO CC = 1, NCOLS5
                IF ( G5COUNT(CC,RR) .NE. G5NRECS(CC,RR) ) EFLAG=.TRUE.
                G5COUNT(CC,RR) = 0
            END DO
            END DO
!$OMP       END CRITICAL( S_KFG5 )

        IF ( EFLAG ) THEN
            MESG = 'Inconsistent MET_KF_SCA_G5, MET_KF_PRO_G5'
            CALL M3EXIT( 'MM5/KFCPL', 0, 0, MESG, 2 )
        END IF

        G5PFID = INDEX1( KF_PRO_G5, MXFILE3, FLIST3 )


        !!~~~~~~~~~~~~~~~~~~>>  open file  KF_PRO_G6:

        SDATE3D = SDATES( 7 )
        STIME3D = STIMES( 7 )
        KSTEP   = -10000
        CALL NEXTIME( SDATE3D, STIME3D, KSTEP )

        IF ( .NOT. DSCGRID( GNAMES( 7 ), NAMBUF, GDTYP3D,
     &                      P_ALP3D, P_BET3D, P_GAM3D,
     &                      XCENT3D, YCENT3D, XORIG3D, YORIG3D,
     &                      XCELL3D, YCELL3D,
     &                      NCOLS3D, NROWS3D, NTHIK3D  ) ) THEN
            MESG = 'No grid description for ' // GNAMES( 7 )
            CALL M3EXIT( 'MM5/KFCPL:KF_INIT', 0, 0, MESG, 2 )
        END IF          !  if dscgrid() failed

        GDNAM3D = GNAMES( 7 )
        NLAYS3D = MKX
        NTHIK3D = NTHIK( 7 )

        IF ( .NOT. KFOPEN( KF_PRO_G6, FSUNKN3, 'MM5/KFCPL',
     &                     G6COUNT ) ) THEN
            MESG = 'Could not open MET_KF_PRO_G6'
            CALL M3EXIT( 'MM5/MCPL', 0, 0, MESG, 2 )
        END IF

!$OMP       CRITICAL( S_KFG6 )
            EFLAG = .FALSE.
            DO RR = 1, NROWS6
            DO CC = 1, NCOLS6
                IF ( G6COUNT(CC,RR) .NE. G6NRECS(CC,RR) ) EFLAG=.TRUE.
                G6COUNT(CC,RR) = 0
            END DO
            END DO
!$OMP       END CRITICAL( S_KFG6 )

        IF ( EFLAG ) THEN
            MESG = 'Inconsistent MET_KF_SCA_G6, MET_KF_PRO_G6'
            CALL M3EXIT( 'MM5/KFCPL', 0, 0, MESG, 2 )
        END IF

        G6PFID = INDEX1( KF_PRO_G6, MXFILE3, FLIST3 )

        KFCPL_ACTIVE = ENVYN( 'KFCPL_ACTIVE', 'Turn on KF processing?',
     &                        .TRUE., IERR )
        IF ( IERR .GT.0 ) THEN
            MESG = 'Invalid environment variable "KFCPL_ACTIVE"'
            CALL M3EXIT( 'MM5/KFCPL', 0, 0, MESG, 2 )
        END IF

        RETURN


C......................................................................
C...............   KF Accumulation Segment:

        ENTRY KF_PUT( NEST, II, JJ,                             !  what cell?
     &      P300KF, TCKF, RKF, ICEKF, TADVEC, NIC, CLDTP,       !  scalar output
     &      TKF0, QKF0, UKF0, VKF0, WAV0, PKF0,
     &      DZQKF, DPKF0 )                   !  vertical profile  output

        KFTOP( II,JJ,NEST ) = CLDTP
        KFTIM( II,JJ,NEST ) = XTIME + D60 * MIN( TADVEC, TCKF )  ! now minutes

        IF ( .NOT. KFCPL_ACTIVE ) RETURN

C...........   Determine date and time in I/O API format and decide whether
C.......   we are after the starting date&time for the KF files (which
C.......   is set to 1 hour before the start of MCPL_OUT(), in order to
C.......   pick up cloud events which are already in existence at the
C.......   time MCPL_OUT() begins):

        KDATE = MDATE0
        KTIME = MTIME0
        KSTEP = SEC2TIME( NINT( 60.0 * XTIME ) )   ! xtime ~~> hhmmss
        CALL NEXTIME( KDATE, KTIME, KSTEP )        ! current simulation clock

        KFLEN = SEC2TIME( NINT( TCKF ) )           ! tckf ~~> hhmmss (duration)

C...............   Per-Nest Output segment:
C.......  Store input scalar  variables into KABUF, where KABUF(7)=float(nic)
C.......        input profile variables into KPBUF,
C.......        cloud top and expiration into KFTOP, KFTIM
C.......  and   then loop through this MM5 domain's output grids,
C.......        calling KFWRITE() for both SCA and PRO

        IF ( NEST .EQ. 1 ) THEN            !  window G1

            !!  buffering for G1 output window:

            KSECS = SECSDIFF( SDATEG1, STIMEG1, KDATE, KTIME )

            IF ( KSECS .GE. -3600 .AND.
     &           II    .GT.  G1I0 .AND. JJ .GT. G1J0 .AND.
     &           II    .LE.  G1I1 .AND. JJ .LE. G1J1 ) THEN

!$OMP           CRITICAL( S_KFCPL )
                G1RECS = G1RECS + 1
                N      = G1RECS
!$OMP           END CRITICAL( S_KFCPL )

                KABUF( N,1,1 ) = P300KF
                KABUF( N,2,1 ) = RKF
                KABUF( N,3,1 ) = ICEKF
                KABUF( N,4,1 ) = XTIME * 60.0
                KABUF( N,5,1 ) = TCKF
                KABUF( N,6,1 ) = TADVEC
                KABUF( N,7,1 ) = FLOAT( NIC )

                DO  L = 1, MKX      !  note mkx  = nlays; copy layered vbles.
                    KPBUF( L,N,1,1 ) = TKF0( L )
                    KPBUF( L,N,2,1 ) = QKF0( L )
                    KPBUF( L,N,3,1 ) = UKF0( L )
                    KPBUF( L,N,4,1 ) = VKF0( L )
                    KPBUF( L,N,5,1 ) = WAV0( L )
                    KPBUF( L,N,6,1 ) = PKF0( L )
                    KPBUF( L,N,7,1 ) = DZQKF( L )
                    KPBUF( L,N,8,1 ) = DPKF0( L )
                END DO

                CC = JJ - G1J0
                RR = II - G1I0

                TFLAG( 1,N,1 ) = CC
                TFLAG( 2,N,1 ) = RR
                TFLAG( 3,N,1 ) = KDATE
                TFLAG( 4,N,1 ) = KTIME
                TFLAG( 5,N,1 ) = KFLEN

                NN               = G1COUNT( CC,RR ) + 1
                G1COUNT( CC,RR ) = NN

                IF ( NN .GT. NTHIK1 ) THEN
                    WRITE( MESG, '()' )
     &                  'KF G1 Event overflow at NEST,I,J,JDATE,JTIME=',
     &                  NEST,II,JJ,KDATE, KTIME
                    CALL M3EXIT( 'MM5/KFCPL:KF_PUT', 0, 0, MESG, 2 )
                END IF

                G1EVENT( NN,CC,RR ) = N + G1REC0
                G1SDATE( NN,CC,RR ) = KDATE
                G1STIME( NN,CC,RR ) = KTIME
                G1LNGTH( NN,CC,RR ) = KFLEN

            END IF      !  if (I,J) in bounds

            RETURN


        ELSE IF ( NEST .EQ. 2 ) THEN            !  windows G3, G4, G5, G6

            !!  buffering for G3 output window:

            KSECS = SECSDIFF( SDATEG3, STIMEG3, KDATE, KTIME )

            IF ( KSECS .GE. -3600 .AND.
     &           II    .GT. G3I0  .AND. JJ .GT. G3J0 .AND.
     &           II    .LE. G3I1  .AND. JJ .LE. G3J1 ) THEN

!$OMP           CRITICAL( S_KFCPL )
                G3RECS = G3RECS + 1
                N      = G3RECS
!$OMP           END CRITICAL( S_KFCPL )

                KABUF( N,1,2 ) = P300KF
                KABUF( N,2,2 ) = RKF
                KABUF( N,3,2 ) = ICEKF
                KABUF( N,4,2 ) = XTIME * 60.0
                KABUF( N,5,2 ) = TCKF
                KABUF( N,6,2 ) = TADVEC
                KABUF( N,7,2 ) = FLOAT( NIC )

                DO  L = 1, MKX      !  note mkx  = nlays; copy layered vbles.
                    KPBUF( L,N,1,2 ) = TKF0( L )
                    KPBUF( L,N,2,2 ) = QKF0( L )
                    KPBUF( L,N,3,2 ) = UKF0( L )
                    KPBUF( L,N,4,2 ) = VKF0( L )
                    KPBUF( L,N,5,2 ) = WAV0( L )
                    KPBUF( L,N,6,2 ) = PKF0( L )
                    KPBUF( L,N,7,2 ) = DZQKF( L )
                    KPBUF( L,N,8,2 ) = DPKF0( L )
                END DO

                CC = JJ - G3J0
                RR = II - G3I0

                TFLAG( 1,N,2 ) = CC
                TFLAG( 2,N,2 ) = RR
                TFLAG( 3,N,2 ) = KDATE
                TFLAG( 4,N,2 ) = KTIME
                TFLAG( 5,N,2 ) = KFLEN

                NN               = G3COUNT( CC,RR ) + 1
                G3COUNT( CC,RR ) = NN

                IF ( NN .GT. NTHIKC ) THEN
                    WRITE( MESG, '()' )
     &                  'KF G3 Event overflow at NEST,I,J,JDATE,JTIME=',
     &                  NEST,II,JJ,KDATE, KTIME
                    CALL M3EXIT( 'MM5/KFCPL:KF_PUT', 0, 0, MESG, 2 )
                END IF

                G3EVENT( NN,CC,RR ) = N + G3REC0
                G3SDATE( NN,CC,RR ) = KDATE
                G3STIME( NN,CC,RR ) = KTIME
                G3LNGTH( NN,CC,RR ) = KFLEN

            END IF      !  if (I,J) in bounds


            !!  buffering for G4 output window:

            KSECS = SECSDIFF( SDATEG4, STIMEG4, KDATE, KTIME )

            IF ( KSECS .GE. -3600 .AND.
     &           II    .GT. G4I0  .AND. JJ .GT. G4J0 .AND.
     &           II    .LE. G4I1  .AND. JJ .LE. G4J1 ) THEN

!$OMP           CRITICAL( S_KFCPL )
                G4RECS = G4RECS + 1
                N      = G4RECS
!$OMP           END CRITICAL( S_KFCPL )

                KABUF( N,1,3 ) = P300KF
                KABUF( N,2,3 ) = RKF
                KABUF( N,3,3 ) = ICEKF
                KABUF( N,4,3 ) = XTIME * 60.0
                KABUF( N,5,3 ) = TCKF
                KABUF( N,6,3 ) = TADVEC
                KABUF( N,7,3 ) = FLOAT( NIC )

                DO  L = 1, MKX      !  note mkx  = nlays; copy layered vbles.
                    KPBUF( L,N,1,3 ) = TKF0( L )
                    KPBUF( L,N,2,3 ) = QKF0( L )
                    KPBUF( L,N,3,3 ) = UKF0( L )
                    KPBUF( L,N,4,3 ) = VKF0( L )
                    KPBUF( L,N,5,3 ) = WAV0( L )
                    KPBUF( L,N,6,3 ) = PKF0( L )
                    KPBUF( L,N,7,3 ) = DZQKF( L )
                    KPBUF( L,N,8,3 ) = DPKF0( L )
                END DO

                CC = JJ - G4J0
                RR = II - G4I0

                TFLAG( 1,N,3 ) = CC
                TFLAG( 2,N,3 ) = RR
                TFLAG( 3,N,3 ) = KDATE
                TFLAG( 4,N,3 ) = KTIME
                TFLAG( 5,N,3 ) = KFLEN

                NN               = G4COUNT( CC,RR ) + 1
                G4COUNT( CC,RR ) = NN

                IF ( NN .GT. NTHIK4 ) THEN
                    WRITE( MESG, '()' )
     &                 'KF G4 Event overflow at NEST,I,J,JDATE,JTIME=',
     &                 NEST,II,JJ,KDATE, KTIME
                    CALL M3EXIT( 'MM5/KFCPL:KF_PUT', 0, 0, MESG, 2 )
                END IF

                G4EVENT( NN,CC,RR ) = N + G4REC0
                G4SDATE( NN,CC,RR ) = KDATE
                G4STIME( NN,CC,RR ) = KTIME
                G4LNGTH( NN,CC,RR ) = KFLEN

            END IF      !  if (I,J) in bounds


            !!  buffering for G5 output window:

            KSECS = SECSDIFF( SDATEG5, STIMEG5, KDATE, KTIME )

            IF ( KSECS .GE. -3600 .AND.
     &           II    .GT.  G5I0 .AND. JJ .GT. G5J0 .AND.
     &           II    .LE.  G5I1 .AND. JJ .LE. G5J1 ) THEN

!$OMP           CRITICAL( S_KFCPL )
                G5RECS = G5RECS + 1
                N      = G5RECS
!$OMP           END CRITICAL( S_KFCPL )

                KABUF( N,1,4 ) = P300KF
                KABUF( N,2,4 ) = RKF
                KABUF( N,3,4 ) = ICEKF
                KABUF( N,4,4 ) = XTIME * 60.0
                KABUF( N,5,4 ) = TCKF
                KABUF( N,6,4 ) = TADVEC
                KABUF( N,7,4 ) = FLOAT( NIC )

                DO  L = 1, MKX      !  note mkx  = nlays; copy layered vbles.
                    KPBUF( L,N,1,4 ) = TKF0( L )
                    KPBUF( L,N,2,4 ) = QKF0( L )
                    KPBUF( L,N,3,4 ) = UKF0( L )
                    KPBUF( L,N,4,4 ) = VKF0( L )
                    KPBUF( L,N,5,4 ) = WAV0( L )
                    KPBUF( L,N,6,4 ) = PKF0( L )
                    KPBUF( L,N,7,4 ) = DZQKF( L )
                    KPBUF( L,N,8,4 ) = DPKF0( L )
                END DO

                CC = JJ - G5J0
                RR = II - G5I0

                TFLAG( 1,N,4 ) = CC
                TFLAG( 2,N,4 ) = RR
                TFLAG( 3,N,4 ) = KDATE
                TFLAG( 4,N,4 ) = KTIME
                TFLAG( 5,N,4 ) = KFLEN

                NN               = G5COUNT( CC,RR ) + 1
                G5COUNT( CC,RR ) = NN

                IF ( NN .GT. NTHIK5 ) THEN
                    WRITE( MESG, '()' )
     &                 'KF G5 Event overflow at NEST,I,J,JDATE,JTIME=',
     &                 NEST,II,JJ,KDATE, KTIME
                    CALL M3EXIT( 'MM5/KFCPL:KF_PUT', 0, 0, MESG, 2 )
                END IF

                G5EVENT( NN,CC,RR ) = N + G5REC0
                G5SDATE( NN,CC,RR ) = KDATE
                G5STIME( NN,CC,RR ) = KTIME
                G5LNGTH( NN,CC,RR ) = KFLEN

            END IF      !  if (I,J) in bounds


            !!  buffering for G6 output window:

            KSECS = SECSDIFF( SDATEG6, STIMEG6, KDATE, KTIME )

            IF ( KSECS .GE. -3600 .AND.
     &           II    .GT. G6I0  .AND. JJ .GT. G6J0 .AND.
     &           II    .LE. G6I1  .AND. JJ .LE. G6J1 ) THEN

!$OMP           CRITICAL( S_KFCPL )
                G6RECS = G6RECS + 1
                N      = G6RECS
!$OMP           END CRITICAL( S_KFCPL )

                KABUF( N,1,5 ) = P300KF
                KABUF( N,2,5 ) = RKF
                KABUF( N,3,5 ) = ICEKF
                KABUF( N,4,5 ) = XTIME * 60.0
                KABUF( N,5,5 ) = TCKF
                KABUF( N,6,5 ) = TADVEC
                KABUF( N,7,5 ) = FLOAT( NIC )

                DO  L = 1, MKX      !  note mkx  = nlays; copy layered vbles.
                    KPBUF( L,N,1,5 ) = TKF0( L )
                    KPBUF( L,N,2,5 ) = QKF0( L )
                    KPBUF( L,N,3,5 ) = UKF0( L )
                    KPBUF( L,N,4,5 ) = VKF0( L )
                    KPBUF( L,N,5,5 ) = WAV0( L )
                    KPBUF( L,N,6,5 ) = PKF0( L )
                    KPBUF( L,N,7,5 ) = DZQKF( L )
                    KPBUF( L,N,8,5 ) = DPKF0( L )
                END DO

                CC = JJ - G6J0
                RR = II - G6I0

                TFLAG( 1,N,5 ) = CC
                TFLAG( 2,N,5 ) = RR
                TFLAG( 3,N,5 ) = KDATE
                TFLAG( 4,N,5 ) = KTIME
                TFLAG( 5,N,5 ) = KFLEN

                NN               = G6COUNT( CC,RR ) + 1
                G6COUNT( CC,RR ) = NN

                IF ( NN .GT. NTHIK4 ) THEN
                    WRITE( MESG, '()' )
     &                 'KF G6 Event overflow at NEST,I,J,JDATE,JTIME=',
     &                 NEST,II,JJ,KDATE, KTIME
                    CALL M3EXIT( 'MM5/KFCPL:KF_PUT', 0, 0, MESG, 2 )
                END IF

                G6EVENT( NN,CC,RR ) = N + G6REC0
                G6SDATE( NN,CC,RR ) = KDATE
                G6STIME( NN,CC,RR ) = KTIME
                G6LNGTH( NN,CC,RR ) = KFLEN

            END IF      !  if (I,J) in bounds

            RETURN


        ELSE            !  else nest not 1,2:

           WRITE( MESG, '( A, I10 )')
     &           'Unsupported nest for KF_PUT: NEST=', NEST
           CALL M3EXIT( 'MM5:KFCPL/KF_PUT', 0, 0, MESG, 2 )

        END IF
        
        RETURN


C......................................................................
C...............   KF Output Segment:

        ENTRY KF_CHK( NEST )

        !!  Check whether there is risk of overflow at next time step:

        IF ( .NOT. KFCPL_ACTIVE ) THEN

            RETURN

        ELSE IF ( NEST .EQ. 1 ) THEN                 !  Window G1:

            IF ( KFRECS-G1RECS .GT. NCOLS1*NROWS1 ) RETURN

        ELSE IF ( NEST .EQ. 2 ) THEN            !  Windows G3,4,5,6:

            IF ( KFRECS-G3RECS .GT. NCOLSC*NROWSC .AND.
     &           KFRECS-G4RECS .GT. NCOLS4*NROWS4 .AND.
     &           KFRECS-G5RECS .GT. NCOLS5*NROWS5 .AND.
     &           KFRECS-G6RECS .GT. NCOLS6*NROWS6 ) RETURN

        ELSE            !  else nest not 1,2:

           RETURN

        END IF
        
        !!  else don't return, but fall through to KF_OUT:
        

C......................................................................
C...............   KF Output Segment:

        ENTRY KF_OUT( NEST )

        !!  Write out all accumulated data for this nest:

        IF ( .NOT. KFCPL_ACTIVE ) RETURN

        KDATE = MDATE0
        KTIME = MTIME0
        KSTEP = SEC2TIME( NINT( 60.0 * XTIME ) )   ! xtime ~~> hhmmss
        CALL NEXTIME( KDATE, KTIME, KSTEP )        ! current simulation clock

        IF ( NEST .EQ. 1 .AND. G1RECS .GT. 0 ) THEN            !  Window G1:

!$OMP       CRITICAL( S_NC )

C...............   KF_SCA_G1, KF_PRO_G1  files:
C...........   Write the variables in the event-records:

            SCANUM = CDFID3( G1AFID )
            PRONUM = CDFID3( G1PFID )

            DIMS( 1 ) = 1
            DELS( 1 ) = 1

            DIMS( 2 ) = G1REC0 + 1
            DELS( 2 ) = G1RECS

            DO  V = 1, NVARS3( G1AFID )

                CALL NCVPT( SCANUM, VINDX3( V,G1AFID ), DIMS, DELS,
     &                      KABUF( 1,V,1 ), IERR )

                IF ( IERR .NE. 0 ) THEN
                    WRITE( MESG, '( 5 A, I10 )' )
     &                  'Error writing variable ', VLIST3( V,G1AFID ),
     &                  'from file ', FLIST3( G1AFID ) ,
     &                  'netCDF error number', IERR
                    CALL M3EXIT( 'KFCPL:KF_OUT', 0, 0, MESG, 2 )
                END IF                  !  ierr nonzero:  NCVPT() failed

            END DO

            DELS( 1 ) = MKX

            DO  V = 1, NVARS3( G1PFID )

                CALL NCVPT( PRONUM, VINDX3( V,G1PFID ), DIMS, DELS,
     &                      KPBUF( 1,1,V,1 ), IERR )

                IF ( IERR .NE. 0 ) THEN
                    WRITE( MESG, '( 5 A, I10 )' )
     &                  'Error writing variable ', VLIST3( V,G1PFID ),
     &                  'from file ', FLIST3( G1PFID ) ,
     &                  'netCDF error number', IERR
                    CALL M3EXIT( 'KFCPL:KF_OUT', 0, 0, MESG, 2 )
                END IF                  !  ierr nonzero:  NCVPT() failed

            END DO


C...........   Write TFLAG for the G1 event-records:

            DIMS( 1 ) = 1
            DELS( 1 ) = 5
            DIMS( 2 ) = G1REC0 + 1
            DELS( 2 ) = G1RECS

            CALL NCVPT( SCANUM, TINDX3(G1AFID), DIMS, DELS,
     &                  TFLAG(1,1,1), IERR )

            IF ( IERR .NE. 0 ) THEN
                WRITE( MESG, '( 5 A, I10 )' )
     &                'Error writing TFLAG to file ', FLIST3(G1AFID) ,
     &                'netCDF error number', IERR
                CALL M3EXIT( 'KFCPL:KF_OUT', 0, 0, MESG, 2 )
            END IF                  !  ierr nonzero:  NCVPT() failed

            CALL NCVPT( PRONUM, TINDX3(G1AFID), DIMS, DELS,
     &                  TFLAG(1,1,1), IERR )

            IF ( IERR .NE. 0 ) THEN
                WRITE( MESG, '( 5 A, I10 )' )
     &                'Error writing TFLAG to file ', FLIST3(G1PFID) ,
     &                'netCDF error number', IERR
                CALL M3EXIT( 'KFCPL:KF_OUT', 0, 0, MESG, 2 )
            END IF                  !  ierr nonzero:  NCVPT() failed


C...........   Update KFEVENT, KFSDATE, KFSTIME, KFLNGTH, KFCOUNT
C...........   for KF_SCA_G1, KF_PRO_G1:

            DO  R = 1, NROWS1
            DO  C = 1, NCOLS1

                IF ( G1COUNT(C,R) .GT. 0 ) THEN

C...................   Update KFEVENT, KFSDATE, KFSTIME, KFLNGTH, KFCOUNT

                    DIMS( 1 ) = G1NRECS(C,R) + 1
                    DELS( 1 ) = G1COUNT(C,R)
                    DIMS( 2 ) = C
                    DELS( 2 ) = 1
                    DIMS( 3 ) = R
                    DELS( 3 ) = 1

                    CALL NCVPT( SCANUM, SINDX3(G1AFID), DIMS, DELS,
     &                          G1EVENT(1,C,R), IERR )

                    IF ( IERR .NE. 0 ) THEN
                        WRITE( MESG, '( 5 A, I10 )' )
     &                    'Error writing KFEVENT to ', FLIST3(G1AFID),
     &                    'netCDF error number', IERR
                        CALL M3EXIT( 'KFCPL:KF_OUT', 0, 0, MESG, 2 )
                    END IF                  !  ierr nonzero:  NCVPT() failed

                    CALL NCVPT( PRONUM, SINDX3(G1PFID), DIMS, DELS,
     &                          G1EVENT(1,C,R), IERR )

                    IF ( IERR .NE. 0 ) THEN
                        WRITE( MESG, '( 5 A, I10 )' )
     &                    'Error writing KFEVENT to ', FLIST3(G1PFID),
     &                    'netCDF error number', IERR
                        CALL M3EXIT( 'KFCPL:KF_OUT', 0, 0, MESG, 2 )
                    END IF                  !  ierr nonzero:  NCVPT() failed

                    CALL NCVPT( SCANUM, LINDX3(G1AFID), DIMS, DELS,
     &                          G1SDATE(1,C,R), IERR )

                    IF ( IERR .NE. 0 ) THEN
                        WRITE( MESG, '( 5 A, I10 )' )
     &                    'Error writing KFSDATE to ', FLIST3(G1AFID),
     &                    'netCDF error number', IERR
                        CALL M3EXIT( 'KFCPL:KF_OUT', 0, 0, MESG, 2 )
                    END IF                  !  ierr nonzero:  NCVPT() failed

                    CALL NCVPT( PRONUM, LINDX3(G1PFID), DIMS, DELS,
     &                          G1SDATE(1,C,R), IERR )

                    IF ( IERR .NE. 0 ) THEN
                        WRITE( MESG, '( 5 A, I10 )' )
     &                    'Error writing KFSDATE to ', FLIST3(G1PFID),
     &                    'netCDF error number', IERR
                        CALL M3EXIT( 'KFCPL:KF_OUT', 0, 0, MESG, 2 )
                    END IF                  !  ierr nonzero:  NCVPT() failed

                    CALL NCVPT( SCANUM, XINDX3(G1AFID), DIMS, DELS,
     &                          G1STIME(1,C,R), IERR )

                    IF ( IERR .NE. 0 ) THEN
                        WRITE( MESG, '( 5 A, I10 )' )
     &                    'Error writing KFSTIME to ', FLIST3(G1AFID),
     &                    'netCDF error number', IERR
                        CALL M3EXIT( 'KFCPL:KF_OUT', 0, 0, MESG, 2 )
                    END IF                  !  ierr nonzero:  NCVPT() failed

                    CALL NCVPT( PRONUM, XINDX3(G1PFID), DIMS, DELS,
     &                          G1STIME(1,C,R), IERR )

                    IF ( IERR .NE. 0 ) THEN
                        WRITE( MESG, '( 5 A, I10 )' )
     &                    'Error writing KFSTIME to ', FLIST3(G1PFID),
     &                    'netCDF error number', IERR
                        CALL M3EXIT( 'KFCPL:KF_OUT', 0, 0, MESG, 2 )
                    END IF                  !  ierr nonzero:  NCVPT() failed

                    CALL NCVPT( SCANUM, YINDX3(G1AFID), DIMS, DELS,
     &                          G1LNGTH(1,C,R), IERR )

                    IF ( IERR .NE. 0 ) THEN
                        WRITE( MESG, '( 5 A, I10 )' )
     &                    'Error writing KFLNGTH to ', FLIST3(G1AFID),
     &                    'netCDF error number', IERR
                        CALL M3EXIT( 'KFCPL:KF_OUT', 0, 0, MESG, 2 )
                    END IF                  !  ierr nonzero:  NCVPT() failed

                    CALL NCVPT( PRONUM, YINDX3(G1PFID), DIMS, DELS,
     &                          G1LNGTH(1,C,R), IERR )

                    IF ( IERR .NE. 0 ) THEN
                        WRITE( MESG, '( 5 A, I10 )' )
     &                    'Error writing KFLNGTH to ', FLIST3(G1PFID),
     &                    'netCDF error number', IERR
                        CALL M3EXIT( 'KFCPL:KF_OUT', 0, 0, MESG, 2 )
                    END IF                  !  ierr nonzero:  NCVPT() failed


C...................   Update KFCOUNT for KF_SCA_G1, KF_PRO_G1:

                    G1NRECS(C,R) = G1COUNT(C,R) + G1NRECS(C,R)
                    G1COUNT(C,R) = 0    !  reset for next output step

                END IF

            END DO
            END DO

C...........   Write G1NRECS to KF_SCA_G1, KF_PRO_G1:

            DIMS( 1 ) = 1
            DELS( 1 ) = NCOLS1
            DIMS( 2 ) = 1
            DELS( 2 ) = NROWS1

            CALL NCVPT( SCANUM, NINDX3(G1AFID), DIMS, DELS,
     &                  G1NRECS, IERR )

            IF ( IERR .NE. 0 ) THEN
                WRITE( MESG, '( 5 A, I10 )' )
     &                'Error writing KFLNGTH to ', FLIST3(G1AFID),
     &                'netCDF error number', IERR
                CALL M3EXIT( 'KFCPL:KF_OUT', 0, 0, MESG, 2 )
            END IF                  !  ierr nonzero:  NCVPT() failed

            CALL NCVPT( PRONUM, NINDX3(G1PFID), DIMS, DELS,
     &                  G1NRECS, IERR )

            IF ( IERR .NE. 0 ) THEN
                WRITE( MESG, '( 5 A, I10 )' )
     &                'Error writing KFLNGTH to ', FLIST3(G1PFID),
     &                'netCDF error number', IERR
                CALL M3EXIT( 'KFCPL:KF_OUT', 0, 0, MESG, 2 )
            END IF                  !  ierr nonzero:  NCVPT() failed

            CALL NCSNC( SCANUM, IERR )
            IF ( IERR .NE. 0 ) THEN
                WRITE( MESG, '( 5 A, I10 )' )
     &              'NCSNC() error for file ', FLIST3(G1AFID),
     &              'netCDF error number', IERR
                CALL M3EXIT( 'KFCPL:KF_OUT', 0, 0, MESG, 2 )
            END IF                  !  ierr nonzero:  NCSNC() failed

            CALL NCSNC( PRONUM, IERR )
            IF ( IERR .NE. 0 ) THEN
                WRITE( MESG, '( 5 A, I10 )' )
     &              'NCSNC() error for file ', FLIST3(G1PFID),
     &              'netCDF error number', IERR
                CALL M3EXIT( 'KFCPL:KF_OUT', 0, 0, MESG, 2 )
            END IF                  !  ierr nonzero:  NCSNC() failed

C...........   Reset record-count state variables for  output window G1:

            G1REC0 = G1REC0 + G1RECS
            G1RECS = 0

            MXREC3( G1AFID ) =  G1REC0
            MXREC3( G1PFID ) =  G1REC0

            WRITE( MESG, '( 3 A, I9.7, A, I6.6 )' )
     &        'Record written to ', KF_SCA_G1,
     &        ' for ', KDATE, ':', KTIME
            CALL M3MESG( MESG )
            WRITE( MESG, '( 3 A, I9.7, A, I6.6 )' )
     &        'Record written to ', KF_PRO_G1,
     &        ' for ', KDATE, ':', KTIME
            CALL M3MESG( MESG )

!$OMP       END CRITICAL( S_NC )


        ELSE IF ( NEST .EQ. 2 ) THEN    !~~~~~~~~>>  windows G3, G4, G5, G6:

!$OMP       CRITICAL( S_NC )

C...............   G3 window: KF_SCA_G3 and KF_PRO_G3 files
C...........   Write the variables in the event-records:

            IF ( G3RECS .EQ. 0 ) GO TO  44

            SCANUM = CDFID3( G3AFID )
            PRONUM = CDFID3( G3PFID )

            DIMS( 1 ) = 1
            DELS( 1 ) = 1

            DIMS( 2 ) = G3REC0 + 1
            DELS( 2 ) = G3RECS

            DO  V = 1, NVARS3( G3AFID )

                CALL NCVPT( SCANUM, VINDX3( V,G3AFID ), DIMS, DELS,
     &                      KABUF( 1,V,2 ), IERR )

                IF ( IERR .NE. 0 ) THEN
                    WRITE( MESG, '( 5 A, I10 )' )
     &                  'Error writing variable ', VLIST3( V,G3AFID ),
     &                  'from file ', FLIST3( G3AFID ) ,
     &                  'netCDF error number', IERR
                    CALL M3EXIT( 'KFCPL:KF_OUT', 0, 0, MESG, 2 )
                END IF                  !  ierr nonzero:  NCVPT() failed

            END DO

            DELS( 1 ) = MKX

            DO  V = 1, NVARS3( G3PFID )

                CALL NCVPT( PRONUM, VINDX3( V,G3PFID ), DIMS, DELS,
     &                      KPBUF( 1,1,V,2 ), IERR )

                IF ( IERR .NE. 0 ) THEN
                    WRITE( MESG, '( 5 A, I10 )' )
     &                  'Error writing variable ', VLIST3( V,G3PFID ),
     &                  'from file ', FLIST3( G3PFID ) ,
     &                  'netCDF error number', IERR
                    CALL M3EXIT( 'KFCPL:KF_OUT', 0, 0, MESG, 2 )
                END IF                  !  ierr nonzero:  NCVPT() failed

            END DO


C...........   Write TFLAG for the G3 event-records:

            DIMS( 1 ) = 1
            DELS( 1 ) = 5
            DIMS( 2 ) = G3REC0 + 1
            DELS( 2 ) = G3RECS

            CALL NCVPT( SCANUM, TINDX3(G3AFID), DIMS, DELS,
     &                  TFLAG(1,1,2), IERR )

            IF ( IERR .NE. 0 ) THEN
                WRITE( MESG, '( 5 A, I10 )' )
     &                'Error writing TFLAG to file ', FLIST3(G3AFID) ,
     &                'netCDF error number', IERR
                CALL M3EXIT( 'KFCPL:KF_OUT', 0, 0, MESG, 2 )
            END IF                  !  ierr nonzero:  NCVPT() failed

            CALL NCVPT( PRONUM, TINDX3(G3AFID), DIMS, DELS,
     &                  TFLAG(1,1,2), IERR )

            IF ( IERR .NE. 0 ) THEN
                WRITE( MESG, '( 5 A, I10 )' )
     &                'Error writing TFLAG to file ', FLIST3(G3PFID) ,
     &                'netCDF error number', IERR
                CALL M3EXIT( 'KFCPL:KF_OUT', 0, 0, MESG, 2 )
            END IF                  !  ierr nonzero:  NCVPT() failed


C...........   Update KFEVENT, KFSDATE, KFSTIME, KFLNGTH, KFCOUNT
C...........   for KF_SCA_G3, KF_PRO_G3:

            DO  R = 1, NROWSC
            DO  C = 1, NCOLSC

                IF ( G3COUNT(C,R) .GT. 0 ) THEN

C...................   Update KFEVENT, KFSDATE, KFSTIME, KFLNGTH, KFCOUNT

                    DIMS( 1 ) = G3NRECS(C,R) + 1
                    DELS( 1 ) = G3COUNT(C,R)
                    DIMS( 2 ) = C
                    DELS( 2 ) = 1
                    DIMS( 3 ) = R
                    DELS( 3 ) = 1

                    CALL NCVPT( SCANUM, SINDX3(G3AFID), DIMS, DELS,
     &                          G3EVENT(1,C,R), IERR )

                    IF ( IERR .NE. 0 ) THEN
                        WRITE( MESG, '( 5 A, I10 )' )
     &                    'Error writing KFEVENT to ', FLIST3(G3AFID),
     &                    'netCDF error number', IERR
                        CALL M3EXIT( 'KFCPL:KF_OUT', 0, 0, MESG, 2 )
                    END IF                  !  ierr nonzero:  NCVPT() failed

                    CALL NCVPT( PRONUM, SINDX3(G3PFID), DIMS, DELS,
     &                          G3EVENT(1,C,R), IERR )

                    IF ( IERR .NE. 0 ) THEN
                        WRITE( MESG, '( 5 A, I10 )' )
     &                    'Error writing KFEVENT to ', FLIST3(G3PFID),
     &                    'netCDF error number', IERR
                        CALL M3EXIT( 'KFCPL:KF_OUT', 0, 0, MESG, 2 )
                    END IF                  !  ierr nonzero:  NCVPT() failed

                    CALL NCVPT( SCANUM, LINDX3(G3AFID), DIMS, DELS,
     &                          G3SDATE(1,C,R), IERR )

                    IF ( IERR .NE. 0 ) THEN
                        WRITE( MESG, '( 5 A, I10 )' )
     &                    'Error writing KFSDATE to ', FLIST3(G3AFID),
     &                    'netCDF error number', IERR
                        CALL M3EXIT( 'KFCPL:KF_OUT', 0, 0, MESG, 2 )
                    END IF                  !  ierr nonzero:  NCVPT() failed

                    CALL NCVPT( PRONUM, LINDX3(G3PFID), DIMS, DELS,
     &                          G3SDATE(1,C,R), IERR )

                    IF ( IERR .NE. 0 ) THEN
                        WRITE( MESG, '( 5 A, I10 )' )
     &                    'Error writing KFSDATE to ', FLIST3(G3PFID),
     &                    'netCDF error number', IERR
                        CALL M3EXIT( 'KFCPL:KF_OUT', 0, 0, MESG, 2 )
                    END IF                  !  ierr nonzero:  NCVPT() failed

                    CALL NCVPT( SCANUM, XINDX3(G3AFID), DIMS, DELS,
     &                          G3STIME(1,C,R), IERR )

                    IF ( IERR .NE. 0 ) THEN
                        WRITE( MESG, '( 5 A, I10 )' )
     &                    'Error writing KFSTIME to ', FLIST3(G3AFID),
     &                    'netCDF error number', IERR
                        CALL M3EXIT( 'KFCPL:KF_OUT', 0, 0, MESG, 2 )
                    END IF                  !  ierr nonzero:  NCVPT() failed

                    CALL NCVPT( PRONUM, XINDX3(G3PFID), DIMS, DELS,
     &                          G3STIME(1,C,R), IERR )

                    IF ( IERR .NE. 0 ) THEN
                        WRITE( MESG, '( 5 A, I10 )' )
     &                    'Error writing KFSTIME to ', FLIST3(G3PFID),
     &                    'netCDF error number', IERR
                        CALL M3EXIT( 'KFCPL:KF_OUT', 0, 0, MESG, 2 )
                    END IF                  !  ierr nonzero:  NCVPT() failed

                    CALL NCVPT( SCANUM, YINDX3(G3AFID), DIMS, DELS,
     &                          G3LNGTH(1,C,R), IERR )

                    IF ( IERR .NE. 0 ) THEN
                        WRITE( MESG, '( 5 A, I10 )' )
     &                    'Error writing KFLNGTH to ', FLIST3(G3AFID),
     &                    'netCDF error number', IERR
                        CALL M3EXIT( 'KFCPL:KF_OUT', 0, 0, MESG, 2 )
                    END IF                  !  ierr nonzero:  NCVPT() failed

                    CALL NCVPT( PRONUM, YINDX3(G3PFID), DIMS, DELS,
     &                          G3LNGTH(1,C,R), IERR )

                    IF ( IERR .NE. 0 ) THEN
                        WRITE( MESG, '( 5 A, I10 )' )
     &                    'Error writing KFLNGTH to ', FLIST3(G3PFID),
     &                    'netCDF error number', IERR
                        CALL M3EXIT( 'KFCPL:KF_OUT', 0, 0, MESG, 2 )
                    END IF                  !  ierr nonzero:  NCVPT() failed


C...................   Update KFCOUNT for KF_SCA_G3, KF_PRO_G3:

                    G3NRECS(C,R) = G3COUNT(C,R) + G3NRECS(C,R)
                    G3COUNT(C,R) = 0    !  reset for next output step

                END IF

            END DO
            END DO

C...........   Write G3NRECS to KF_SCA_G3, KF_PRO_G3:

            DIMS( 1 ) = 1
            DELS( 1 ) = NCOLSC
            DIMS( 2 ) = 1
            DELS( 2 ) = NROWSC

            CALL NCVPT( SCANUM, NINDX3(G3AFID), DIMS, DELS,
     &                  G3NRECS, IERR )

            IF ( IERR .NE. 0 ) THEN
                WRITE( MESG, '( 5 A, I10 )' )
     &                'Error writing KFLNGTH to ', FLIST3(G3AFID),
     &                'netCDF error number', IERR
                CALL M3EXIT( 'KFCPL:KF_OUT', 0, 0, MESG, 2 )
            END IF                  !  ierr nonzero:  NCVPT() failed

            CALL NCVPT( PRONUM, NINDX3(G3PFID), DIMS, DELS,
     &                  G3NRECS, IERR )

            IF ( IERR .NE. 0 ) THEN
                WRITE( MESG, '( 5 A, I10 )' )
     &                'Error writing KFLNGTH to ', FLIST3(G3PFID),
     &                'netCDF error number', IERR
                CALL M3EXIT( 'KFCPL:KF_OUT', 0, 0, MESG, 2 )
            END IF                  !  ierr nonzero:  NCVPT() failed

            CALL NCSNC( SCANUM, IERR )
            IF ( IERR .NE. 0 ) THEN
                WRITE( MESG, '( 5 A, I10 )' )
     &              'NCSNC() error for file ', FLIST3(G3AFID),
     &              'netCDF error number', IERR
                CALL M3EXIT( 'KFCPL:KF_OUT', 0, 0, MESG, 2 )
            END IF                  !  ierr nonzero:  NCSNC() failed

            CALL NCSNC( PRONUM, IERR )
            IF ( IERR .NE. 0 ) THEN
                WRITE( MESG, '( 5 A, I10 )' )
     &              'NCSNC() error for file ', FLIST3(G3PFID),
     &              'netCDF error number', IERR
                CALL M3EXIT( 'KFCPL:KF_OUT', 0, 0, MESG, 2 )
            END IF                  !  ierr nonzero:  NCSNC() failed

C...........   Reset record-count state variables for window G3:

            G3REC0 = G3REC0 + G3RECS
            G3RECS = 0

            MXREC3( G3AFID ) =  G3REC0
            MXREC3( G3PFID ) =  G3REC0

            WRITE( MESG, '( 3 A, I9.7, A, I6.6 )' )
     &        'Record written to ', KF_SCA_G3,
     &        ' for ', KDATE, ':', KTIME
            CALL M3MESG( MESG )
            WRITE( MESG, '( 3 A, I9.7, A, I6.6 )' )
     &        'Record written to ', KF_PRO_G3,
     &        ' for ', KDATE, ':', KTIME
            CALL M3MESG( MESG )


C...............   KF_SCA_G4, KF_PRO_G4  files:
C...........   Write the variables in the event-records:

44          CONTINUE

            IF ( G4RECS .EQ. 0 ) GO TO  55

            SCANUM = CDFID3( G4AFID )
            PRONUM = CDFID3( G4PFID )

            DIMS( 1 ) = 1
            DELS( 1 ) = 1

            DIMS( 2 ) = G4REC0 + 1
            DELS( 2 ) = G4RECS

            DO  V = 1, NVARS3( G4AFID )

                CALL NCVPT( SCANUM, VINDX3( V,G4AFID ), DIMS, DELS,
     &                      KABUF( 1,V,3 ), IERR )

                IF ( IERR .NE. 0 ) THEN
                    WRITE( MESG, '( 5 A, I10 )' )
     &                  'Error writing variable ', VLIST3( V,G4AFID ),
     &                  'from file ', FLIST3( G4AFID ) ,
     &                  'netCDF error number', IERR
                    CALL M3EXIT( 'KFCPL:KF_OUT', 0, 0, MESG, 2 )
                END IF                  !  ierr nonzero:  NCVPT() failed

            END DO

            DELS( 1 ) = MKX

            DO  V = 1, NVARS3( G4PFID )

                CALL NCVPT( PRONUM, VINDX3( V,G4PFID ), DIMS, DELS,
     &                      KPBUF( 1,1,V,3 ), IERR )

                IF ( IERR .NE. 0 ) THEN
                    WRITE( MESG, '( 5 A, I10 )' )
     &                  'Error writing variable ', VLIST3( V,G4PFID ),
     &                  'from file ', FLIST3( G4PFID ) ,
     &                  'netCDF error number', IERR
                    CALL M3EXIT( 'KFCPL:KF_OUT', 0, 0, MESG, 2 )
                END IF                  !  ierr nonzero:  NCVPT() failed

            END DO


C...........   Write TFLAG for the G4 event-records:

            DIMS( 1 ) = 1
            DELS( 1 ) = 5
            DIMS( 2 ) = G4REC0 + 1
            DELS( 2 ) = G4RECS

            CALL NCVPT( SCANUM, TINDX3(G4AFID), DIMS, DELS,
     &                  TFLAG(1,1,3), IERR )

            IF ( IERR .NE. 0 ) THEN
                WRITE( MESG, '( 5 A, I10 )' )
     &                'Error writing TFLAG to file ', FLIST3(G4AFID) ,
     &                'netCDF error number', IERR
                CALL M3EXIT( 'KFCPL:KF_OUT', 0, 0, MESG, 2 )
            END IF                  !  ierr nonzero:  NCVPT() failed

            CALL NCVPT( PRONUM, TINDX3(G4AFID), DIMS, DELS,
     &                  TFLAG(1,1,3), IERR )

            IF ( IERR .NE. 0 ) THEN
                WRITE( MESG, '( 5 A, I10 )' )
     &                'Error writing TFLAG to file ', FLIST3(G4PFID) ,
     &                'netCDF error number', IERR
                CALL M3EXIT( 'KFCPL:KF_OUT', 0, 0, MESG, 2 )
            END IF                  !  ierr nonzero:  NCVPT() failed


C...........   Update KFEVENT, KFSDATE, KFSTIME, KFLNGTH, KFCOUNT
C...........   for KF_SCA_G4, KF_PRO_G4:

            DO  R = 1, NROWS4
            DO  C = 1, NCOLS4

                IF ( G4COUNT(C,R) .GT. 0 ) THEN

C...................   Update KFEVENT, KFSDATE, KFSTIME, KFLNGTH, KFCOUNT

                    DIMS( 1 ) = G4NRECS(C,R) + 1
                    DELS( 1 ) = G4COUNT(C,R)
                    DIMS( 2 ) = C
                    DELS( 2 ) = 1
                    DIMS( 3 ) = R
                    DELS( 3 ) = 1

                    CALL NCVPT( SCANUM, SINDX3(G4AFID), DIMS, DELS,
     &                          G4EVENT(1,C,R), IERR )

                    IF ( IERR .NE. 0 ) THEN
                        WRITE( MESG, '( 5 A, I10 )' )
     &                    'Error writing KFEVENT to ', FLIST3(G4AFID),
     &                    'netCDF error number', IERR
                        CALL M3EXIT( 'KFCPL:KF_OUT', 0, 0, MESG, 2 )
                    END IF                  !  ierr nonzero:  NCVPT() failed

                    CALL NCVPT( PRONUM, SINDX3(G4PFID), DIMS, DELS,
     &                          G4EVENT(1,C,R), IERR )

                    IF ( IERR .NE. 0 ) THEN
                        WRITE( MESG, '( 5 A, I10 )' )
     &                    'Error writing KFEVENT to ', FLIST3(G4PFID),
     &                    'netCDF error number', IERR
                        CALL M3EXIT( 'KFCPL:KF_OUT', 0, 0, MESG, 2 )
                    END IF                  !  ierr nonzero:  NCVPT() failed

                    CALL NCVPT( SCANUM, LINDX3(G4AFID), DIMS, DELS,
     &                          G4SDATE(1,C,R), IERR )

                    IF ( IERR .NE. 0 ) THEN
                        WRITE( MESG, '( 5 A, I10 )' )
     &                    'Error writing KFSDATE to ', FLIST3(G4AFID),
     &                    'netCDF error number', IERR
                        CALL M3EXIT( 'KFCPL:KF_OUT', 0, 0, MESG, 2 )
                    END IF                  !  ierr nonzero:  NCVPT() failed

                    CALL NCVPT( PRONUM, LINDX3(G4PFID), DIMS, DELS,
     &                          G4SDATE(1,C,R), IERR )

                    IF ( IERR .NE. 0 ) THEN
                        WRITE( MESG, '( 5 A, I10 )' )
     &                    'Error writing KFSDATE to ', FLIST3(G4PFID),
     &                    'netCDF error number', IERR
                        CALL M3EXIT( 'KFCPL:KF_OUT', 0, 0, MESG, 2 )
                    END IF                  !  ierr nonzero:  NCVPT() failed

                    CALL NCVPT( SCANUM, XINDX3(G4AFID), DIMS, DELS,
     &                          G4STIME(1,C,R), IERR )

                    IF ( IERR .NE. 0 ) THEN
                        WRITE( MESG, '( 5 A, I10 )' )
     &                    'Error writing KFSTIME to ', FLIST3(G4AFID),
     &                    'netCDF error number', IERR
                        CALL M3EXIT( 'KFCPL:KF_OUT', 0, 0, MESG, 2 )
                    END IF                  !  ierr nonzero:  NCVPT() failed

                    CALL NCVPT( PRONUM, XINDX3(G4PFID), DIMS, DELS,
     &                          G4STIME(1,C,R), IERR )

                    IF ( IERR .NE. 0 ) THEN
                        WRITE( MESG, '( 5 A, I10 )' )
     &                    'Error writing KFSTIME to ', FLIST3(G4PFID),
     &                    'netCDF error number', IERR
                        CALL M3EXIT( 'KFCPL:KF_OUT', 0, 0, MESG, 2 )
                    END IF                  !  ierr nonzero:  NCVPT() failed

                    CALL NCVPT( SCANUM, YINDX3(G4AFID), DIMS, DELS,
     &                          G4LNGTH(1,C,R), IERR )

                    IF ( IERR .NE. 0 ) THEN
                        WRITE( MESG, '( 5 A, I10 )' )
     &                    'Error writing KFLNGTH to ', FLIST3(G4AFID),
     &                    'netCDF error number', IERR
                        CALL M3EXIT( 'KFCPL:KF_OUT', 0, 0, MESG, 2 )
                    END IF                  !  ierr nonzero:  NCVPT() failed

                    CALL NCVPT( PRONUM, YINDX3(G4PFID), DIMS, DELS,
     &                          G4LNGTH(1,C,R), IERR )

                    IF ( IERR .NE. 0 ) THEN
                        WRITE( MESG, '( 5 A, I10 )' )
     &                    'Error writing KFLNGTH to ', FLIST3(G4PFID),
     &                    'netCDF error number', IERR
                        CALL M3EXIT( 'KFCPL:KF_OUT', 0, 0, MESG, 2 )
                    END IF                  !  ierr nonzero:  NCVPT() failed


C...................   Update KFCOUNT for KF_SCA_G4, KF_PRO_G4:

                    G4NRECS(C,R) = G4COUNT(C,R) + G4NRECS(C,R)
                    G4COUNT(C,R) = 0    !  reset for next output step

                END IF

            END DO
            END DO

C...........   Write G4NRECS to KF_SCA_G4, KF_PRO_G4:

            DIMS( 1 ) = 1
            DELS( 1 ) = NCOLS4
            DIMS( 2 ) = 1
            DELS( 2 ) = NROWS4

            CALL NCVPT( SCANUM, NINDX3(G4AFID), DIMS, DELS,
     &                  G4NRECS, IERR )

            IF ( IERR .NE. 0 ) THEN
                WRITE( MESG, '( 5 A, I10 )' )
     &                'Error writing KFLNGTH to ', FLIST3(G4AFID),
     &                'netCDF error number', IERR
                CALL M3EXIT( 'KFCPL:KF_OUT', 0, 0, MESG, 2 )
            END IF                  !  ierr nonzero:  NCVPT() failed

            CALL NCVPT( PRONUM, NINDX3(G4PFID), DIMS, DELS,
     &                  G4NRECS, IERR )

            IF ( IERR .NE. 0 ) THEN
                WRITE( MESG, '( 5 A, I10 )' )
     &                'Error writing KFLNGTH to ', FLIST3(G4PFID),
     &                'netCDF error number', IERR
                CALL M3EXIT( 'KFCPL:KF_OUT', 0, 0, MESG, 2 )
            END IF                  !  ierr nonzero:  NCVPT() failed

            CALL NCSNC( SCANUM, IERR )
            IF ( IERR .NE. 0 ) THEN
                WRITE( MESG, '( 5 A, I10 )' )
     &              'NCSNC() error for file ', FLIST3(G4AFID),
     &              'netCDF error number', IERR
                CALL M3EXIT( 'KFCPL:KF_OUT', 0, 0, MESG, 2 )
            END IF                  !  ierr nonzero:  NCSNC() failed

            CALL NCSNC( PRONUM, IERR )
            IF ( IERR .NE. 0 ) THEN
                WRITE( MESG, '( 5 A, I10 )' )
     &              'NCSNC() error for file ', FLIST3(G4PFID),
     &              'netCDF error number', IERR
                CALL M3EXIT( 'KFCPL:KF_OUT', 0, 0, MESG, 2 )
            END IF                  !  ierr nonzero:  NCSNC() failed

C...........   Reset record-count state variables for output window G4:

            G4REC0 = G4REC0 + G4RECS
            G4RECS = 0

            MXREC3( G4AFID ) =  G4REC0
            MXREC3( G4PFID ) =  G4REC0

            WRITE( MESG, '( 3 A, I9.7, A, I6.6 )' )
     &        'Record written to ', KF_SCA_G4,
     &        ' for ', KDATE, ':', KTIME
            CALL M3MESG( MESG )
            WRITE( MESG, '( 3 A, I9.7, A, I6.6 )' )
     &        'Record written to ', KF_PRO_G4,
     &        ' for ', KDATE, ':', KTIME
            CALL M3MESG( MESG )


C...............   KF_SCA_G5, KF_PRO_G5  files:
C...........   Write the variables in the event-records:

55          CONTINUE

            IF ( G5RECS .EQ. 0 ) GO TO  66

            SCANUM = CDFID3( G5AFID )
            PRONUM = CDFID3( G5PFID )

            DIMS( 1 ) = 1
            DELS( 1 ) = 1

            DIMS( 2 ) = G5REC0 + 1
            DELS( 2 ) = G5RECS

            DO  V = 1, NVARS3( G5AFID )

                CALL NCVPT( SCANUM, VINDX3( V,G5AFID ), DIMS, DELS,
     &                      KABUF( 1,V,4 ), IERR )

                IF ( IERR .NE. 0 ) THEN
                    WRITE( MESG, '( 5 A, I10 )' )
     &                  'Error writing variable ', VLIST3( V,G5AFID ),
     &                  'from file ', FLIST3( G5AFID ) ,
     &                  'netCDF error number', IERR
                    CALL M3EXIT( 'KFCPL:KF_OUT', 0, 0, MESG, 2 )
                END IF                  !  ierr nonzero:  NCVPT() failed

            END DO

            DELS( 1 ) = MKX

            DO  V = 1, NVARS3( G5PFID )

                CALL NCVPT( PRONUM, VINDX3( V,G5PFID ), DIMS, DELS,
     &                      KPBUF( 1,1,V,4 ), IERR )

                IF ( IERR .NE. 0 ) THEN
                    WRITE( MESG, '( 5 A, I10 )' )
     &                  'Error writing variable ', VLIST3( V,G5PFID ),
     &                  'from file ', FLIST3( G5PFID ) ,
     &                  'netCDF error number', IERR
                    CALL M3EXIT( 'KFCPL:KF_OUT', 0, 0, MESG, 2 )
                END IF                  !  ierr nonzero:  NCVPT() failed

            END DO


C...........   Write TFLAG for the G5 event-records:

            DIMS( 1 ) = 1
            DELS( 1 ) = 5
            DIMS( 2 ) = G5REC0 + 1
            DELS( 2 ) = G5RECS

            CALL NCVPT( SCANUM, TINDX3(G5AFID), DIMS, DELS,
     &                  TFLAG(1,1,4), IERR )

            IF ( IERR .NE. 0 ) THEN
                WRITE( MESG, '( 5 A, I10 )' )
     &                'Error writing TFLAG to file ', FLIST3(G5AFID) ,
     &                'netCDF error number', IERR
                CALL M3EXIT( 'KFCPL:KF_OUT', 0, 0, MESG, 2 )
            END IF                  !  ierr nonzero:  NCVPT() failed

            CALL NCVPT( PRONUM, TINDX3(G5AFID), DIMS, DELS,
     &                  TFLAG(1,1,4), IERR )

            IF ( IERR .NE. 0 ) THEN
                WRITE( MESG, '( 5 A, I10 )' )
     &                'Error writing TFLAG to file ', FLIST3(G5PFID) ,
     &                'netCDF error number', IERR
                CALL M3EXIT( 'KFCPL:KF_OUT', 0, 0, MESG, 2 )
            END IF                  !  ierr nonzero:  NCVPT() failed


C...........   Update KFEVENT, KFSDATE, KFSTIME, KFLNGTH, KFCOUNT
C...........   for KF_SCA_G5, KF_PRO_G5:

            DO  R = 1, NROWS5
            DO  C = 1, NCOLS5

                IF ( G5COUNT(C,R) .GT. 0 ) THEN

C...................   Update KFEVENT, KFSDATE, KFSTIME, KFLNGTH, KFCOUNT

                    DIMS( 1 ) = G5NRECS(C,R) + 1
                    DELS( 1 ) = G5COUNT(C,R)
                    DIMS( 2 ) = C
                    DELS( 2 ) = 1
                    DIMS( 3 ) = R
                    DELS( 3 ) = 1

                    CALL NCVPT( SCANUM, SINDX3(G5AFID), DIMS, DELS,
     &                          G5EVENT(1,C,R), IERR )

                    IF ( IERR .NE. 0 ) THEN
                        WRITE( MESG, '( 5 A, I10 )' )
     &                    'Error writing KFEVENT to ', FLIST3(G5AFID),
     &                    'netCDF error number', IERR
                        CALL M3EXIT( 'KFCPL:KF_OUT', 0, 0, MESG, 2 )
                    END IF                  !  ierr nonzero:  NCVPT() failed

                    CALL NCVPT( PRONUM, SINDX3(G5PFID), DIMS, DELS,
     &                          G5EVENT(1,C,R), IERR )

                    IF ( IERR .NE. 0 ) THEN
                        WRITE( MESG, '( 5 A, I10 )' )
     &                    'Error writing KFEVENT to ', FLIST3(G5PFID),
     &                    'netCDF error number', IERR
                        CALL M3EXIT( 'KFCPL:KF_OUT', 0, 0, MESG, 2 )
                    END IF                  !  ierr nonzero:  NCVPT() failed

                    CALL NCVPT( SCANUM, LINDX3(G5AFID), DIMS, DELS,
     &                          G5SDATE(1,C,R), IERR )

                    IF ( IERR .NE. 0 ) THEN
                        WRITE( MESG, '( 5 A, I10 )' )
     &                    'Error writing KFSDATE to ', FLIST3(G5AFID),
     &                    'netCDF error number', IERR
                        CALL M3EXIT( 'KFCPL:KF_OUT', 0, 0, MESG, 2 )
                    END IF                  !  ierr nonzero:  NCVPT() failed

                    CALL NCVPT( PRONUM, LINDX3(G5PFID), DIMS, DELS,
     &                          G5SDATE(1,C,R), IERR )

                    IF ( IERR .NE. 0 ) THEN
                        WRITE( MESG, '( 5 A, I10 )' )
     &                    'Error writing KFSDATE to ', FLIST3(G5PFID),
     &                    'netCDF error number', IERR
                        CALL M3EXIT( 'KFCPL:KF_OUT', 0, 0, MESG, 2 )
                    END IF                  !  ierr nonzero:  NCVPT() failed

                    CALL NCVPT( SCANUM, XINDX3(G5AFID), DIMS, DELS,
     &                          G5STIME(1,C,R), IERR )

                    IF ( IERR .NE. 0 ) THEN
                        WRITE( MESG, '( 5 A, I10 )' )
     &                    'Error writing KFSTIME to ', FLIST3(G5AFID),
     &                    'netCDF error number', IERR
                        CALL M3EXIT( 'KFCPL:KF_OUT', 0, 0, MESG, 2 )
                    END IF                  !  ierr nonzero:  NCVPT() failed

                    CALL NCVPT( PRONUM, XINDX3(G5PFID), DIMS, DELS,
     &                          G5STIME(1,C,R), IERR )

                    IF ( IERR .NE. 0 ) THEN
                        WRITE( MESG, '( 5 A, I10 )' )
     &                    'Error writing KFSTIME to ', FLIST3(G5PFID),
     &                    'netCDF error number', IERR
                        CALL M3EXIT( 'KFCPL:KF_OUT', 0, 0, MESG, 2 )
                    END IF                  !  ierr nonzero:  NCVPT() failed

                    CALL NCVPT( SCANUM, YINDX3(G5AFID), DIMS, DELS,
     &                          G5LNGTH(1,C,R), IERR )

                    IF ( IERR .NE. 0 ) THEN
                        WRITE( MESG, '( 5 A, I10 )' )
     &                    'Error writing KFLNGTH to ', FLIST3(G5AFID),
     &                    'netCDF error number', IERR
                        CALL M3EXIT( 'KFCPL:KF_OUT', 0, 0, MESG, 2 )
                    END IF                  !  ierr nonzero:  NCVPT() failed

                    CALL NCVPT( PRONUM, YINDX3(G5PFID), DIMS, DELS,
     &                          G5LNGTH(1,C,R), IERR )

                    IF ( IERR .NE. 0 ) THEN
                        WRITE( MESG, '( 5 A, I10 )' )
     &                    'Error writing KFLNGTH to ', FLIST3(G5PFID),
     &                    'netCDF error number', IERR
                        CALL M3EXIT( 'KFCPL:KF_OUT', 0, 0, MESG, 2 )
                    END IF                  !  ierr nonzero:  NCVPT() failed


C...................   Update KFCOUNT for KF_SCA_G5, KF_PRO_G5:

                    G5NRECS(C,R) = G5COUNT(C,R) + G5NRECS(C,R)
                    G5COUNT(C,R) = 0    !  reset for next output step

                END IF

            END DO
            END DO

C...........   Write G5NRECS to KF_SCA_G5, KF_PRO_G5:

            DIMS( 1 ) = 1
            DELS( 1 ) = NCOLS5
            DIMS( 2 ) = 1
            DELS( 2 ) = NROWS5

            CALL NCVPT( SCANUM, NINDX3(G5AFID), DIMS, DELS,
     &                  G5NRECS, IERR )

            IF ( IERR .NE. 0 ) THEN
                WRITE( MESG, '( 5 A, I10 )' )
     &                'Error writing KFLNGTH to ', FLIST3(G5AFID),
     &                'netCDF error number', IERR
                CALL M3EXIT( 'KFCPL:KF_OUT', 0, 0, MESG, 2 )
            END IF                  !  ierr nonzero:  NCVPT() failed

            CALL NCVPT( PRONUM, NINDX3(G5PFID), DIMS, DELS,
     &                  G5NRECS, IERR )

            IF ( IERR .NE. 0 ) THEN
                WRITE( MESG, '( 5 A, I10 )' )
     &                'Error writing KFLNGTH to ', FLIST3(G5PFID),
     &                'netCDF error number', IERR
                CALL M3EXIT( 'KFCPL:KF_OUT', 0, 0, MESG, 2 )
            END IF                  !  ierr nonzero:  NCVPT() failed

            CALL NCSNC( SCANUM, IERR )
            IF ( IERR .NE. 0 ) THEN
                WRITE( MESG, '( 5 A, I10 )' )
     &              'NCSNC() error for file ', FLIST3(G5AFID),
     &              'netCDF error number', IERR
                CALL M3EXIT( 'KFCPL:KF_OUT', 0, 0, MESG, 2 )
            END IF                  !  ierr nonzero:  NCSNC() failed

            CALL NCSNC( PRONUM, IERR )
            IF ( IERR .NE. 0 ) THEN
                WRITE( MESG, '( 5 A, I10 )' )
     &              'NCSNC() error for file ', FLIST3(G5PFID),
     &              'netCDF error number', IERR
                CALL M3EXIT( 'KFCPL:KF_OUT', 0, 0, MESG, 2 )
            END IF                  !  ierr nonzero:  NCSNC() failed

C...........   Reset record-count state variables for output windows G5:

            G5REC0 = G5REC0 + G5RECS
            G5RECS = 0

            MXREC3( G5AFID ) =  G5REC0
            MXREC3( G5PFID ) =  G5REC0

            WRITE( MESG, '( 3 A, I9.7, A, I6.6 )' )
     &        'Record written to ', KF_SCA_G5,
     &        ' for ', KDATE, ':', KTIME
            CALL M3MESG( MESG )
            WRITE( MESG, '( 3 A, I9.7, A, I6.6 )' )
     &        'Record written to ', KF_PRO_G5,
     &        ' for ', KDATE, ':', KTIME
            CALL M3MESG( MESG )


C...............   KF_SCA_G6, KF_PRO_G6  files:
C...........   Write the variables in the event-records:

66          CONTINUE

            IF ( G6RECS .EQ. 0 ) GO TO  77

            SCANUM = CDFID3( G6AFID )
            PRONUM = CDFID3( G6PFID )

            DIMS( 1 ) = 1
            DELS( 1 ) = 1

            DIMS( 2 ) = G6REC0 + 1
            DELS( 2 ) = G6RECS

            DO  V = 1, NVARS3( G6AFID )

                CALL NCVPT( SCANUM, VINDX3( V,G6AFID ), DIMS, DELS,
     &                      KABUF( 1,V,5 ), IERR )

                IF ( IERR .NE. 0 ) THEN
                    WRITE( MESG, '( 5 A, I10 )' )
     &                  'Error writing variable ', VLIST3( V,G6AFID ),
     &                  'from file ', FLIST3( G6AFID ) ,
     &                  'netCDF error number', IERR
                    CALL M3EXIT( 'KFCPL:KF_OUT', 0, 0, MESG, 2 )
                END IF                  !  ierr nonzero:  NCVPT() failed

            END DO

            DELS( 1 ) = MKX

            DO  V = 1, NVARS3( G6PFID )

                CALL NCVPT( PRONUM, VINDX3( V,G6PFID ), DIMS, DELS,
     &                      KPBUF( 1,1,V,5 ), IERR )

                IF ( IERR .NE. 0 ) THEN
                    WRITE( MESG, '( 5 A, I10 )' )
     &                  'Error writing variable ', VLIST3( V,G6PFID ),
     &                  'from file ', FLIST3( G6PFID ) ,
     &                  'netCDF error number', IERR
                    CALL M3EXIT( 'KFCPL:KF_OUT', 0, 0, MESG, 2 )
                END IF                  !  ierr nonzero:  NCVPT() failed

            END DO


C...........   Write TFLAG for the G6 event-records:

            DIMS( 1 ) = 1
            DELS( 1 ) = 5
            DIMS( 2 ) = G6REC0 + 1
            DELS( 2 ) = G6RECS

            CALL NCVPT( SCANUM, TINDX3(G6AFID), DIMS, DELS,
     &                  TFLAG(1,1,5), IERR )

            IF ( IERR .NE. 0 ) THEN
                WRITE( MESG, '( 5 A, I10 )' )
     &                'Error writing TFLAG to file ', FLIST3(G6AFID) ,
     &                'netCDF error number', IERR
                CALL M3EXIT( 'KFCPL:KF_OUT', 0, 0, MESG, 2 )
            END IF                  !  ierr nonzero:  NCVPT() failed

            CALL NCVPT( PRONUM, TINDX3(G6AFID), DIMS, DELS,
     &                  TFLAG(1,1,5), IERR )

            IF ( IERR .NE. 0 ) THEN
                WRITE( MESG, '( 5 A, I10 )' )
     &                'Error writing TFLAG to file ', FLIST3(G6PFID) ,
     &                'netCDF error number', IERR
                CALL M3EXIT( 'KFCPL:KF_OUT', 0, 0, MESG, 2 )
            END IF                  !  ierr nonzero:  NCVPT() failed


C...........   Update KFEVENT, KFSDATE, KFSTIME, KFLNGTH, KFCOUNT
C...........   for KF_SCA_G6, KF_PRO_G6:

            DO  R = 1, NROWS6
            DO  C = 1, NCOLS6

                IF ( G6COUNT(C,R) .GT. 0 ) THEN

C...................   Update KFEVENT, KFSDATE, KFSTIME, KFLNGTH, KFCOUNT

                    DIMS( 1 ) = G6NRECS(C,R) + 1
                    DELS( 1 ) = G6COUNT(C,R)
                    DIMS( 2 ) = C
                    DELS( 2 ) = 1
                    DIMS( 3 ) = R
                    DELS( 3 ) = 1

                    CALL NCVPT( SCANUM, SINDX3(G6AFID), DIMS, DELS,
     &                          G6EVENT(1,C,R), IERR )

                    IF ( IERR .NE. 0 ) THEN
                        WRITE( MESG, '( 5 A, I10 )' )
     &                    'Error writing KFEVENT to ', FLIST3(G6AFID),
     &                    'netCDF error number', IERR
                        CALL M3EXIT( 'KFCPL:KF_OUT', 0, 0, MESG, 2 )
                    END IF                  !  ierr nonzero:  NCVPT() failed

                    CALL NCVPT( PRONUM, SINDX3(G6PFID), DIMS, DELS,
     &                          G6EVENT(1,C,R), IERR )

                    IF ( IERR .NE. 0 ) THEN
                        WRITE( MESG, '( 5 A, I10 )' )
     &                    'Error writing KFEVENT to ', FLIST3(G6PFID),
     &                    'netCDF error number', IERR
                        CALL M3EXIT( 'KFCPL:KF_OUT', 0, 0, MESG, 2 )
                    END IF                  !  ierr nonzero:  NCVPT() failed

                    CALL NCVPT( SCANUM, LINDX3(G6AFID), DIMS, DELS,
     &                          G6SDATE(1,C,R), IERR )

                    IF ( IERR .NE. 0 ) THEN
                        WRITE( MESG, '( 5 A, I10 )' )
     &                    'Error writing KFSDATE to ', FLIST3(G6AFID),
     &                    'netCDF error number', IERR
                        CALL M3EXIT( 'KFCPL:KF_OUT', 0, 0, MESG, 2 )
                    END IF                  !  ierr nonzero:  NCVPT() failed

                    CALL NCVPT( PRONUM, LINDX3(G6PFID), DIMS, DELS,
     &                          G6SDATE(1,C,R), IERR )

                    IF ( IERR .NE. 0 ) THEN
                        WRITE( MESG, '( 5 A, I10 )' )
     &                    'Error writing KFSDATE to ', FLIST3(G6PFID),
     &                    'netCDF error number', IERR
                        CALL M3EXIT( 'KFCPL:KF_OUT', 0, 0, MESG, 2 )
                    END IF                  !  ierr nonzero:  NCVPT() failed

                    CALL NCVPT( SCANUM, XINDX3(G6AFID), DIMS, DELS,
     &                          G6STIME(1,C,R), IERR )

                    IF ( IERR .NE. 0 ) THEN
                        WRITE( MESG, '( 5 A, I10 )' )
     &                    'Error writing KFSTIME to ', FLIST3(G6AFID),
     &                    'netCDF error number', IERR
                        CALL M3EXIT( 'KFCPL:KF_OUT', 0, 0, MESG, 2 )
                    END IF                  !  ierr nonzero:  NCVPT() failed

                    CALL NCVPT( PRONUM, XINDX3(G6PFID), DIMS, DELS,
     &                          G6STIME(1,C,R), IERR )

                    IF ( IERR .NE. 0 ) THEN
                        WRITE( MESG, '( 5 A, I10 )' )
     &                    'Error writing KFSTIME to ', FLIST3(G6PFID),
     &                    'netCDF error number', IERR
                        CALL M3EXIT( 'KFCPL:KF_OUT', 0, 0, MESG, 2 )
                    END IF                  !  ierr nonzero:  NCVPT() failed

                    CALL NCVPT( SCANUM, YINDX3(G6AFID), DIMS, DELS,
     &                          G6LNGTH(1,C,R), IERR )

                    IF ( IERR .NE. 0 ) THEN
                        WRITE( MESG, '( 5 A, I10 )' )
     &                    'Error writing KFLNGTH to ', FLIST3(G6AFID),
     &                    'netCDF error number', IERR
                        CALL M3EXIT( 'KFCPL:KF_OUT', 0, 0, MESG, 2 )
                    END IF                  !  ierr nonzero:  NCVPT() failed

                    CALL NCVPT( PRONUM, YINDX3(G6PFID), DIMS, DELS,
     &                          G6LNGTH(1,C,R), IERR )

                    IF ( IERR .NE. 0 ) THEN
                        WRITE( MESG, '( 5 A, I10 )' )
     &                    'Error writing KFLNGTH to ', FLIST3(G6PFID),
     &                    'netCDF error number', IERR
                        CALL M3EXIT( 'KFCPL:KF_OUT', 0, 0, MESG, 2 )
                    END IF                  !  ierr nonzero:  NCVPT() failed


C...................   Update KFCOUNT for KF_SCA_G6, KF_PRO_G6:

                    G6NRECS(C,R) = G6COUNT(C,R) + G6NRECS(C,R)
                    G6COUNT(C,R) = 0    !  reset for next output step

                END IF

            END DO
            END DO

C...........   Write G1NRECS to KF_SCA_G1, KF_PRO_G1:

            DIMS( 1 ) = 1
            DELS( 1 ) = NCOLS6
            DIMS( 2 ) = 1
            DELS( 2 ) = NROWS6

            CALL NCVPT( SCANUM, NINDX3(G6AFID), DIMS, DELS,
     &                  G6NRECS, IERR )

            IF ( IERR .NE. 0 ) THEN
                WRITE( MESG, '( 5 A, I10 )' )
     &                'Error writing KFLNGTH to ', FLIST3(G6AFID),
     &                'netCDF error number', IERR
                CALL M3EXIT( 'KFCPL:KF_OUT', 0, 0, MESG, 2 )
            END IF                  !  ierr nonzero:  NCVPT() failed

            CALL NCVPT( PRONUM, NINDX3(G6PFID), DIMS, DELS,
     &                  G6NRECS, IERR )

            IF ( IERR .NE. 0 ) THEN
                WRITE( MESG, '( 5 A, I10 )' )
     &                'Error writing KFLNGTH to ', FLIST3(G6PFID),
     &                'netCDF error number', IERR
                CALL M3EXIT( 'KFCPL:KF_OUT', 0, 0, MESG, 2 )
            END IF                  !  ierr nonzero:  NCVPT() failed

            CALL NCSNC( SCANUM, IERR )
            IF ( IERR .NE. 0 ) THEN
                WRITE( MESG, '( 5 A, I10 )' )
     &              'NCSNC() error for file ', FLIST3(G6AFID),
     &              'netCDF error number', IERR
                CALL M3EXIT( 'KFCPL:KF_OUT', 0, 0, MESG, 2 )
            END IF                  !  ierr nonzero:  NCSNC() failed

            CALL NCSNC( PRONUM, IERR )
            IF ( IERR .NE. 0 ) THEN
                WRITE( MESG, '( 5 A, I10 )' )
     &              'NCSNC() error for file ', FLIST3(G6PFID),
     &              'netCDF error number', IERR
                CALL M3EXIT( 'KFCPL:KF_OUT', 0, 0, MESG, 2 )
            END IF                  !  ierr nonzero:  NCSNC() failed

C...........   Reset record-count state variables for output window G6:

            G6REC0 = G6REC0 + G6RECS
            G6RECS = 0

            MXREC3( G6AFID ) =  G6REC0
            MXREC3( G6PFID ) =  G6REC0

            WRITE( MESG, '( 3 A, I9.7, A, I6.6 )' )
     &        'Record written to ', KF_SCA_G6,
     &        ' for ', KDATE, ':', KTIME
            CALL M3MESG( MESG )
            WRITE( MESG, '( 3 A, I9.7, A, I6.6 )' )
     &        'Record written to ', KF_PRO_G6,
     &        ' for ', KDATE, ':', KTIME
            CALL M3MESG( MESG )


77          CONTINUE

!$OMP       END CRITICAL( S_NC )

        ELSE            !  else nest not 1,2:

           RETURN

        END IF

        RETURN
        END
