
C Version "%W% %P% %G% %U%
C.........................................................................
C Version "%W% %P% %G% %U%"
C MCPL output module for MM5.  Copyright (C) 2001 MCNC
C Distributed under the GNU LESSER GENERAL PUBLIC LICENSE version 2.1
C See file "LGPL.txt" for conditions of use.
C.........................................................................

        SUBROUTINE MM2CMAQD( IM, JM, KM,
     &                       P00, TLP, TS0, RDRY, ROVG, SIGH, PTOP,
     &                       PST, MSFD, MSFX, UA, VA, TA, QV, PP, 
     &                       NCOLS, NROWS, NLAYS,
     &                       COL0, ROW0, LAY0,
     &                       UCONT, VCONT )

C***********************************************************************
C  subroutine body starts at line  144
C
C  FUNCTION:  Extract/reorder/decouple/compute the contravariant
C       density-weighted horizontal wind components for the EPQ
C       CMAQ air quality model.
C
C  NOTE that MM5-order I subscripts go from south to north, J subscripts
C       go from west to east, and K subscripts go from top to bottom,
C       whereas IOAPI-order C subscripts go from west to east,
C       R subscripts go from south to north, and L subscripts go
C       from bottom to top.
C
C  PRECONDITIONS REQUIRED:
C	ioapi grid is a "proper window" into the MM5 grid
C
C  SUBROUTINES AND FUNCTIONS CALLED:
C       none
C
C  REVISION  HISTORY:
C       Prototype 12/2001 by Carlie J. Coats, Jr., MCNC Environmental 
C       Modeling Center
C***********************************************************************

      IMPLICIT NONE

C...........   ARGUMENTS and their descriptions:

        !!  input arguments:  

        INTEGER         IM      	   !  MM5 I (north-south) dimension
        INTEGER         JM      	   !  MM5 J (east-west)   dimension
        INTEGER         KM      	   !  MM5 K (top-bottom)  dimension
        REAL            P00  		   !  MM5 reference SL-pres (Pa)
        REAL            TLP		   !  MM5 temp ref lapse rate (K/log(P))
        REAL            TS0		   !  MM5 ref SL temp (K)
        REAL            RDRY		   !  MM5 dry gas constant
        REAL            ROVG		   !  MM5 R/G
        REAL            SIGH( KM )         !  MM5 SIGMA-H
        REAL            PTOP		   !  MM5 ptop (cb)
        REAL            PST( IM, JM )      !  MM5 cross-point ref pstar
        REAL            MSFD( IM, JM )     !  MM5-order dot pt map factor
        REAL            MSFX( IM, JM )     !  MM5 inverse map scale factor
        REAL            TA( IM, JM, KM )   !  MM5 coupled temperature
        REAL            QV( IM, JM, KM )   !  MM5 coupled mixing ratio
        REAL            PP( IM, JM, KM )   !  MM5 P-prime
        REAL            UA( IM, JM, KM )   !  MM5 coupled temperature
        REAL            VA( IM, JM, KM )   !  MM5 coupled mixing ratio
        INTEGER         NCOLS   	   !  dot-point output dimension
        INTEGER         NROWS   	   !  " 
        INTEGER         NLAYS   	   !  "
        INTEGER         COL0    	   !  starting J for output window
        INTEGER         ROW0    	   !  ...      I "
        INTEGER         LAY0    	   !  ...      K "

        !!  output arguments:  

        REAL            UCONT( NCOLS, NROWS, NLAYS )
        REAL            VCONT( NCOLS, NROWS, NLAYS )


C...........   PARAMETERs

        INTEGER         MAX
        REAL	    	CBTOPA

        PARAMETER     ( MAX    = 512 , 
     &                  CBTOPA = 1.0E3 )


C...........   SCRATCH LOCAL VARIABLES and their descriptions:

	INTEGER		SIZE
	INTEGER		C, R, L		        !  IOAPI col subscript
        INTEGER         I, J, K                 !  MM5 subscripts
        INTEGER         I0, J0, K0              !  MM5 offsets
        REAL            TH0, PR0                !  ref temp & press
        REAL            DP0                     !  1/p0
        REAL            DR                      !  1/rdry
        REAL            CRDG                    !  CBTOPA * RDRY / G
        REAL            PREFX, TREFX            !  ref. atm. pres, temp
        REAL            PREFY, TREFY            !  ref. atm. pres, temp
        REAL            LPX, LPY                !  ref atm profile factors
        REAL            PSUMX, PSUMY
        REAL            MSUMX, MSUMY
        
        REAL            DTA, DQV, DPP		!  decoupled TA,QV,PP
        REAL            PRES, TV
        REAL            JX, JY, USUM, VSUM, DSUMX, DSUMY
        REAL            U1, U2, V1, V2

        REAL            XFAC( NCOLS+1, NROWS+1 )   !  decouple factor
        REAL            DFAC( NCOLS+1, NROWS+1 )   !  decouple factor
        REAL            DENS( NCOLS+1, NROWS+1 )   !  density at layer L
        REAL            PSTX( NCOLS+1, NROWS+1 )   !  PST on N-S edges
        REAL            PSTY( NCOLS+1, NROWS+1 )   !  PST on E-W edges
        REAL           JFACX( NCOLS+1, NROWS+1 )   !  J-factor on N-S edges
        REAL           JFACY( NCOLS+1, NROWS+1 )   !  J-factor on E-W edges

#ifndef LANGUAGE_FORTRAN90
#if __sgi  || __sun || __osf__ || __mips__

	INTEGER         LSIZE
        DATA            LSIZE  / -1 /
	POINTER         ( P1, XFAC )
	POINTER         ( P2, DFAC )
	POINTER         ( P3, DENS )
	POINTER         ( P4, PSTX )
	POINTER         ( P5, PSTY )
	POINTER         ( P6, JFACX )
	POINTER         ( P7, JFACY )

	SAVE            P1, P2, P3, P4, P5, LSIZE

C***********************************************************************

	SIZE = 4 * ( NCOLS + 1 ) * ( NROWS + 1 )   !! bytes per 2-D variable
        IF ( SIZE .GT. LSIZE ) THEN
	    IF ( LSIZE .GT. 0 )  CALL FREE( P1 )
	    P1 = MALLOC( 7 * SIZE )
            P2 = P1 + SIZE
            P3 = P1 + SIZE*2
            P4 = P1 + SIZE*3
            P5 = P1 + SIZE*4
            P6 = P1 + SIZE*5
            P7 = P1 + SIZE*6
	    IF ( P1 .EQ. 0 )
     &           CALL M3EXIT( 'MMXDCPL', 0, 0,
     &                        'Memory allocation error', 2 )
	    LSIZE = SIZE
	END IF

#endif    /* if     __sgi  || __sun || __osf__ || __mips__ */
#endif    /* ifndef LANGUAGE_FORTRAN90  */

        I0 = ROW0 - 1                 !  offsets for I/O API grid within MM5
        J0 = COL0 - 1
        K0 = KM - LAY0 + 2

        !!  SETUP COMPUTATIONS for dot-point NCOLSxNROWS grid and for
        !!  cross-point NCOLS+1xNROWS+1 grid enclosing it:
        !!  Decoupling-factors XFAC, DFAC for cross and dot points;
        !!  scratch variables JFAC's for the expressions
        !!  CPA * RDRY * PSTAR( I,J ) * MSFX( I,J )**2 / G
        !!  on both sets (N-S and E-W) of edges

        DP0 = 1.0 / P00
        DR  = 1.0 / RDRY
        CRDG= 0.125 * CBTOPA * ROVG   !! factor  0.5**3  for PSUMX*MSUMX**2

        DO  R = 1, NROWS
            I = I0 + R
            DO  C = 1, NCOLS
                J = J0 + C

                XFAC( C,R ) = 1.0/  PST( I  ,J )
                DFAC( C,R ) = 4.0/( MSFD( I,J )
     &                            * ( PST( I  ,J ) + PST( I  ,J+1 )
     &                              + PST( I+1,J ) + PST( I+1,J+1 ) ) )

                PSUMX =  PST( I,J ) +  PST( I,J+1 )
                PSUMY =  PST( I,J ) +  PST( I+1,J )
                MSUMX = MSFX( I,J ) + MSFX( I,J+1 )
                MSUMY = MSFX( I,J ) + MSFX( I+1,J )
                PSTX ( C,R ) =  0.5 * PSUMX
                PSTY ( C,R ) =  0.5 * PSUMY
                JFACX( C,R ) = CRDG * PSUMX * MSUMX**2
                JFACY( C,R ) = CRDG * PSUMY * MSUMY**2

            END DO

            C = NCOLS + 1
            J = J0 + C
            XFAC( C,R )  = 1.0 /   PST( I  ,J )
            PSUMX =  PST( I,J ) +  PST( I,J+1 )
            PSUMY =  PST( I,J ) +  PST( I+1,J )
            MSUMX = MSFX( I,J ) + MSFX( I,J+1 )
            MSUMY = MSFX( I,J ) + MSFX( I+1,J )
            PSTX ( C,R ) =  0.5 * PSUMX
            PSTY ( C,R ) =  0.5 * PSUMY
            JFACX( C,R ) = CRDG * PSUMX * MSUMX**2
            JFACY( C,R ) = CRDG * PSUMY * MSUMY**2

        END DO

        R = NROWS + 1
        I = I0 + R
        DO  C = 1, NCOLS
            J = J0 + C

            XFAC( C,R )  = 1.0 /   PST( I  ,J )
            PSUMX =  PST( I,J ) +  PST( I,J+1 )
            PSUMY =  PST( I,J ) +  PST( I+1,J )
            MSUMX = MSFX( I,J ) + MSFX( I,J+1 )
            MSUMY = MSFX( I,J ) + MSFX( I+1,J )
            PSTX ( C,R ) =  0.5 * PSUMX
            PSTY ( C,R ) =  0.5 * PSUMY
            JFACX( C,R ) = CRDG * PSUMX * MSUMX**2
            JFACY( C,R ) = CRDG * PSUMY * MSUMY**2

        END DO

        C = NCOLS + 1
        J = J0 + C
        XFAC( C,R )  = 1.0 /   PST( I  ,J )
        PSUMX =  PST( I,J ) +  PST( I,J+1 )
        PSUMY =  PST( I,J ) +  PST( I+1,J )
        MSUMX = MSFX( I,J ) + MSFX( I,J+1 )
        MSUMY = MSFX( I,J ) + MSFX( I+1,J )
        PSTX ( C,R ) =  0.5 * PSUMX
        PSTY ( C,R ) =  0.5 * PSUMY
        JFACX( C,R ) = CRDG * PSUMX * MSUMX**2
        JFACY( C,R ) = CRDG * PSUMY * MSUMY**2

        !!  3-D gridded computation of density-weighted contravariant
        !!  wind components:

        DO  L = 1, NLAYS
            K = K0 - L

            !!  setup at this level: cross-point density on extended grid

            DO  R = 1, NROWS+1
                I = I0 + R
                DO  C = 1, NCOLS+1
                    J = J0 + C

                    DTA  = XFAC( C,R ) * TA( I,J,K )	!  decoupled
                    DQV  = XFAC( C,R ) * QV( I,J,K )	!   "
                    DPP  = XFAC( C,R ) * PP( I,J,K )    !  ", pa
                    TV   = DTA * ( 1.0 + 0.608 * DQV )
                    PRES = CBTOPA * ( PST( I,J )*SIGH( K ) + PTOP )
     &                     + DPP
                    DENS( C,R ) = PRES / ( RDRY * TV )

                END DO
            END DO

            DO  R = 1, NROWS
                I = I0 + R
                DO  C = 1, NCOLS
                    J = J0 + C

                    !!  N-S edge values for UCONT:
                    !!  note opposite summing-formulas for dot-pt, 
                    !!  cross-pt variables.

                    PREFX = CBTOPA*( PSTX( I,J )*SIGH( K ) + PTOP )
                    LPX   = ALOG( PREFX * DP0 )
                    TREFX = TS0 + TLP*LPX
                    JX    = JFACX(C,R) * TREFX / PREFX
                    U1    = UA( I  ,J,K ) * DFAC( C,R   )       !  decoupled
                    U2    = UA( I+1,J,K ) * DFAC( C,R+1 )       !  U-winds
                    USUM  = U1          + U2
                    DSUMX = DENS( C,R ) + DENS( C+1,R )
                    UCONT( C,R,L ) = 0.25 * JX * USUM * DSUMX

                    !!  E-W edge values for VCONT:

                    PREFY = CBTOPA*( PSTY( I,J )*SIGH( K ) + PTOP )
                    LPY   = ALOG( PREFY * DP0 )
                    TREFY = TS0 + TLP*LPY
                    JY    = JFACY(C,R) * TREFY / PREFY
                    V1    = VA( I,J  ,K ) * DFAC( C  ,R )       !  decoupled
                    V2    = VA( I,J+1,K ) * DFAC( C+1,R )       !  V-winds
                    VSUM  = V1          + V2
                    DSUMY = DENS( C,R ) + DENS( C,R+1 )
                    VCONT( C,R,L ) = 0.25 * JY * VSUM * DSUMY

                END DO
            END DO

        END DO          !  end loop on levels (3-D computation)


        RETURN
        END

     
