
C Version "%W% %P% %G% %U%
C.........................................................................
C Version "%W% %P% %G% %U%"
C MCPL output module for MM5.  Copyright (C) 2001 MCNC
C Distributed under the GNU LESSER GENERAL PUBLIC LICENSE version 2.1
C See file "LGPL.txt" for conditions of use.
C.........................................................................

        SUBROUTINE MM2CMAQX( IM, JM, KM,
     &                       G, P00, TLP, TS0, ROVG, RDRY, 
     &                       SIGH, SIGF, PTOP, DX,
     &                       PST, MSFX, TA, QV, PP, WA,
     &                       NCOLS, NROWS, NLAYS,
     &                       COL0, ROW0, LAY0,
     &                       TSFC, TH0, TF0, PSFC, PRH0, PRF0,
     &                       UHAT, VHAT,
     &                       DENSJ, DVAPJ, ENTPJ, WHATJ )

C***********************************************************************
C  subroutine body starts at line  178
C
C  FUNCTION:  Extract/reorder/decouple/compute the cross point half-
C       and full-level Jacobians, the Jacobian-weighted
C       thermodynamics variables and the contravariant
C       density-weighted vertical wind component for the EPQ
C       CMAQ air quality model.
C
C  NOTE that MM5-order I subscripts go from south to north, J subscripts
C       go from west to east, and K subscripts go from top to bottom,
C       whereas IOAPI-order C subscripts go from west to east,
C       R subscripts go from south to north, and L subscripts go
C       from bottom to top.
C
C  PRECONDITIONS REQUIRED:
C      ioapi grid is a "proper window" into the MM5 grid
C
C  SUBROUTINES AND FUNCTIONS CALLED:
C       none
C
C  REVISION  HISTORY:
C       Prototype 12/2001 by Carlie J. Coats, Jr., MCNC Environmental 
C       Modeling Center
C***********************************************************************

      IMPLICIT NONE

C...........   ARGUMENTS and their descriptions:

        !!  input arguments:  

        INTEGER         IM                 !  MM5 I (north-south) dimension
        INTEGER         JM                 !  MM5 J (east-west)   dimension
        INTEGER         KM                 !  MM5 K (top-bottom)  dimension
        REAL            G                  !  MM5 gravitational "constant"
        REAL            P00                !  MM5 reference SL-pres (Pa)
        REAL            TLP                !  MM5 temp ref lapse rate (K/log(P))
        REAL            TS0                !  MM5 ref SL temp (K)
        REAL            ROVG               !  MM5 R/G
        REAL            RDRY               !  MM5 dry gas constant
        REAL            SIGH( KM )         !  MM5 SIGMA-H
        REAL            SIGF( KM )         !  MM5 SIGMA-H
        REAL            PTOP               !  MM5 ptop (cb)
        REAL            DX                 !  MM5 cell-size (m)
        REAL            PST( IM, JM )      !  MM5 cross-point reference pstar
        REAL            MSFX( IM, JM )     !  MM5 inverse map scale factor
        REAL            TA( IM, JM, KM )   !  MM5 coupled temperature
        REAL            QV( IM, JM, KM )   !  MM5 coupled mixing ratio
        REAL            PP( IM, JM, KM )   !  MM5 P-prime
        REAL            WA( IM, JM, KM+1 ) !  MM5 W-wind
        INTEGER         NCOLS              !  cross-point output dimension
        INTEGER         NROWS              !  " 
        INTEGER         NLAYS              !  "
        INTEGER         COL0               !  starting J for output window
        INTEGER         ROW0               !  ...      I "
        INTEGER         LAY0               !  ...      K "
        REAL            TSFC( NCOLS, NROWS )         !  ref sfc temperature
        REAL            TH0 ( NCOLS, NROWS, NLAYS )  !  ref sigh temperature
        REAL            TF0 ( NCOLS, NROWS, NLAYS )  !  ref sigf temperature
        REAL            PSFC( NCOLS, NROWS )         !  ref sfc pressure
        REAL            PRH0( NCOLS, NROWS, NLAYS )  !  ref sigh pressure
        REAL            PRF0( NCOLS, NROWS, NLAYS )  !  ref sigf pressure
        REAL            UHAT( NCOLS+1, NROWS+1, NLAYS )  !  contravariant U
        REAL            VHAT( NCOLS+1, NROWS+1, NLAYS )  !  contravariant V

        !!  (purely) output arguments:  

        REAL            DENSJ( NCOLS, NROWS, NLAYS )
        REAL            DVAPJ( NCOLS, NROWS, NLAYS )
        REAL            ENTPJ( NCOLS, NROWS, NLAYS )
        REAL            WHATJ( NCOLS, NROWS, NLAYS )


C...........   PARAMETERs

        INTEGER         MAX
        PARAMETER     ( MAX = 512 )

        REAL            GRAV             !  graviatational acceleration
        REAL            RDGAS            !  dry gas constant
        REAL            RDWVAP           !  gas constant for water vapor
        REAL            CP               !  "
        REAL            CTOK             !  Celsius-to-Kelvin
        REAL            TBFRZ            !  ice-only threshold
        REAL            TTFRZ            !  water-only threshold
        REAL            STDPRES          !  standard pressure (Pa)
        REAL            STDTEMP          !  standard temperature [ K ]
        REAL            VP0              ! Saturation vapor pressure (PA)
        REAL            CBTOPA           !  conversion factor, Pa to cb
        REAL            DDFRZ            !  1/(ttfrz-tbfrz)
        REAL            ROVCP            !  RDGAS/CP
        REAL            CVD              !  spec heat of dry air at const vol
        REAL            CVWVAP           !  spec heat of QV at const vol
        REAL            DTEMP            !  1/stdtemp

        PARAMETER     ( GRAV    =    9.80622,     !  CMAQ value
     &                  RDGAS   =  287.07548994,  !  CMAQ value
     &                  RDWVAP  =  461.52492604,  !  CMAQ value
     &                  CP      = 1004.7642148 , !  "
     &                  CTOK    =  273.16 ,
     &                  TBFRZ   =  248.16 ,
     &                  TTFRZ   =  268.16 ,
     &                  STDPRES =    1.013250E+5 ,
     &                  STDTEMP =  273.15 ,
     &                  VP0     =  611.29,
     &                  CBTOPA  =    1.0E3 ,
     &                  DDFRZ   = 1.0 / ( TTFRZ - TBFRZ ) ,
     &                  ROVCP   = RDGAS / CP ,
     &                  CVD     = 5.0 * RDGAS / 2.0 ,
     &                  CVWVAP  = 4.0 * RDWVAP ,
     &                  DTEMP   = 1.0 / STDTEMP )


C...........   SCRATCH LOCAL VARIABLES and their descriptions:

        INTEGER         SIZE
        INTEGER         C, D, R, S, L, M        !  IOAPI subscripts
        INTEGER         I, J, K, N              !  MM5 subscripts
        INTEGER         I0, J0, K0              !  MM5 offsets
        REAL            DP0                     !  1/p0
        REAL            DR                      !  1/rdry
        REAL            CRDG                    !  CBTOPA * RDRY / G
        REAL            PREFX, TREFX            !  ref. atm. pres, temp
        REAL            PREFY, TREFY            !  ref. atm. pres, temp
        REAL            LPX, LPY                !  ref atm profile factors
        REAL            PSUMX, PSUMY
        REAL            MSUMX, MSUMY
        REAL            RHO, DRHO, LAMDA, RHOV, DRHOV
        REAL            JCBH, JCBF              !  half, full-level Jacobians
        
        REAL            DTA, DQV, DPP, DWA      !  decoupled TA,QV,PP, WA
        REAL            PRSH, TVH, DENH, DVAP, ENTP
        REAL            PPRF, PRSF, TAF, TVF, QVF, DENF
        REAL            PFAC, TFAC, QFAC, ZFAC, DDX
        REAL            UL, UM, VL, VM, UVF
        REAL            USUM, VSUM, DSUMX, DSUMY

        REAL            DFAC( MAX )   ! decouple factor
        REAL            DENS( MAX )   ! current/last half-level density
        REAL            PRES( MAX )   ! current/last half-level pressure (Pa)
        REAL            TEMP( MAX )   ! current/last half-level temperature (K)
        REAL            PPRM( MAX )   ! current/last half-level PPrime (Pa)
        REAL            QVAP( MAX )   ! current/last half-level QV (kg/kg)
        REAL            XFAC( MAX )   ! 
        REAL            YFAC( MAX )   ! 
        REAL            FBOT( MAX )   ! 
        REAL            FTOP( MAX )   ! 
        REAL            WBOT( MAX )   ! 
        REAL            WTOP( MAX )   ! 
        REAL             PSX( MAX )   ! 
        REAL             PSY( MAX )   ! 
        REAL            SFAC( MAX )   ! 0.25 * ( 1.0 - xx3face(lvl) )
        REAL            JFAC( NCOLS )

C***********************************************************************
C   begin body of subroutine  MM2CMAQX

        IF ( NCOLS .GT. MAX ) THEN
            CALL M3EXIT( 'MM2CMAQX', 0, 0,
     &      'SCRATCH-ARRAY error:  need parameter MAX > NCOLS', 2 )
        END IF

         IF ( NLAYS .GT. MAX ) THEN
            CALL M3EXIT( 'MM2CMAQX', 0, 0,
     &      'SCRATCH-ARRAY error:  need parameter MAX > NLAYS', 2 )
        END IF

        I0 = ROW0 - 1                 !  offsets for I/O API grid within MM5
        J0 = COL0 - 1
        K0 = KM - LAY0 + 2

        !!  STRUCTURE:  Do as a "wave-front" computation in L, so as
        !!  to have L1-cache-resident scratch variables.
        !!  SETUP COMPUTATIONS:  Decoupling-factor DFAC; dpstar/dp[x,y]
        !!  factors XFAC,YFAC for WHATJ

        L = 1
        K = K0 - L
        FBOT( L ) = ( SIGF( K ) - SIGH( K-1 ) )
     &             /( SIGH( K ) - SIGH( K-1 ) )
        FTOP( L ) = 1.0 - FBOT( L )
        WBOT( L ) =  0.25 * SIGF( K ) * FBOT( L )
        WTOP( L ) =  0.25 * SIGF( K ) * FTOP( L )

        DO  L = 2, NLAYS
            K = K0 - L
            FBOT( L ) = ( SIGF( K ) - SIGH( K-1 ) )
     &                 /( SIGH( K ) - SIGH( K-1 ) )
            FTOP( L ) = 1.0 - FBOT( L )
            WBOT( L ) =  0.25 * SIGF( K ) * FBOT( L )
            WTOP( L ) =  0.25 * SIGF( K ) * FTOP( L )
            SFAC( L ) =  0.25 * ( SIGF( K ) - SIGF( K ) )
        END DO

        DDX   = 1.0 / DX
        DP0   = 1.0 / P00
        DR    = 1.0 / RDRY
        CRDG  = 0.125 * CBTOPA * ROVG   !! factor 0.5**3 for PSUMX * MSUMX**2
        RHO   = P00 / ( RDRY * STDTEMP )
        DRHO  = 1.0 / RHO
        LAMDA = RDWVAP * STDTEMP * 17.625 / 243.04
        RHOV  = VP0 / ( RDWVAP * STDTEMP )
        DRHOV = RDWVAP * STDTEMP / VP0

        DO  R = 1, NROWS
            I = I0 + R

            !!  Wave-front 3-D gridded computation:
            !!  Set up constants; compute thermodynamics at half-level L=1:

            L = 1               !!  start L=1 SIG-H wavefront setup
            K = K0 - L

            DO  C = 1, NCOLS
                J = J0 + C
                D = C + 1
                S = R + 1

                DFAC( C )   = 1.0 /  PST( I,J )
                XFAC( C )   = DDX*DFAC( C )*( PST(I,J+1) - PST(I,J-1) )
                YFAC( C )   = DDX*DFAC( C )*( PST(I+1,J) - PST(I-1,J) )

                JFAC( C ) = CRDG * PST( I,J ) * MSFX( I,J )**2
                JCBH = JFAC( C ) * TH0( C,R,L ) / PRH0( C,R,L )
                DTA  = DFAC( C ) * TA( I,J,K )    !  decoupled
                DQV  = DFAC( C ) * QV( I,J,K )    !   "
                DPP  = DFAC( C ) * PP( I,J,K )    !   ", pa
                TVH  = DTA * ( 1.0 + 0.608 * DQV )
                PRSH = PRH0(C,R,L) + DPP
                DENH = PRSH / ( RDRY * TVH )
                DVAP = DQV * DENH
                IF ( DVAP .LT. 1.0E05 ) THEN
                    ENTP = CVD * DENH * LOG( DTEMP * DTA )
     &                      - RDRY * DENH * LOG( DENH * DRHO ) 
                ELSE
                    ENTP = ( CVD * DENH  + CVWVAP * DVAP )
     &                      * LOG( DTEMP * DTA )
     &                      + DVAP * LAMDA
     &                      - RDRY * DENH * LOG( DENH * DRHO )
     &                      - RDWVAP * DVAP * LOG( DVAP * DRHOV )
                END IF

                DENSJ( C,R,L ) = DENH * JCBH
                DVAPJ( C,R,L ) = DVAP * JCBH
                ENTPJ( C,R,L ) = ENTP * JCBH

                DENS( C ) = DENH
                TEMP( C ) = DTA
                PPRM( C ) = DPP
                QVAP( C ) = DQV

            END DO      !  end loop on columns (sfc; layer 1)

            !!  Wave-front 3-D gridded computation:
            !!  thermodynamics at the remainder of half-levels,
            !!  WHATJ at all but last full-level.
            !!  Note that WHATJ at full-level M=L-1 needs thermodynamics at
            !!  half-levels L and L-1; and WHAT at full-level NLAYS is ZERO.

            DO  L = 2, NLAYS
                K = K0 - L
                M = L  - 1
                N = K0 - M
                DO  C = 1, NCOLS
                    J = J0 + C
                    D = C + 1
                    S = R + 1

                    DTA  = DFAC( C ) * TA( I,J,K )    !  decoupled
                    DQV  = DFAC( C ) * QV( I,J,K )    !   "
                    DPP  = DFAC( C ) * PP( I,J,K )    !   ", pa
                    TVH  = DTA * ( 1.0 + 0.608 * DQV )
                    PRSH = PRH0(C,R,L) + DPP
                    DENH = PRSH / ( RDRY * TVH )
                    DVAP = DQV * DENH
                    IF ( DVAP .LT. 1.0E05 ) THEN
                        ENTP = CVD * DENH * LOG( DTEMP * DTA )
     &                          - RDRY * DENH * LOG( DENH * DRHO ) 
                    ELSE
                        ENTP = ( CVD * DENH  + CVWVAP * DVAP )
     &                          * LOG( DTEMP * DTA )
     &                          + DVAP * LAMDA
     &                          - RDRY * DENH * LOG( DENH * DRHO )
     &                          - RDWVAP * DVAP * LOG( DVAP * DRHOV )
                    END IF

                    JCBH = JFAC( C ) * TH0( C,R,L ) / PRH0( C,R,L )
                    JCBF = JFAC( C ) * TF0( C,R,L ) / PRF0( C,R,L )
                    ZFAC = G*DR*PST( I,J )*PRF0( C,R,M ) / TF0( C,R,M )
                    DWA  = ZFAC * JCBF * WA( I,J,L )   !  decoupled
                    PFAC = ( PRF0(C,R,M) - PRH0(C,R,M) ) / 
     &                     ( PRH0(C,R,L) - PRH0(C,R,M) )
                    TFAC = (  TF0(C,R,M) -  TH0(C,R,M) ) / 
     &                     (  TH0(C,R,L) -  TH0(C,R,M) )
                    PPRF = PPRM( C )   + PFAC * ( DPP - PPRM( C ) )
                    QVF  = QVAP( C )   + PFAC * ( DQV - QVAP( C ) )
                    TAF  = TEMP( C )   + TFAC * ( DTA - TEMP( C ) )
                    TVF  = TAF * ( 1.0 + 0.608 * QVF )
                    PRSF = PRF0(C,R,L) + PPRF
                    DENF = PRSF / ( RDRY * TVF )

                    UM = XFAC( C )* ( UHAT( C,R,M ) + UHAT( D,R,M ) )
                    UL = XFAC( C )* ( UHAT( C,R,L ) + UHAT( D,R,L ) )
                    VM = YFAC( C )* ( VHAT( C,R,M ) + VHAT( C,S,M ) )
                    VL = YFAC( C )* ( VHAT( C,R,L ) + VHAT( C,S,L ) )
                    UVF= UM + VM + PFAC * ( UL - UM + VL - VM )

                    DENSJ( C,R,L ) = DENH * JCBH
                    DVAPJ( C,R,L ) = DVAP * JCBH
                    ENTPJ( C,R,L ) = ENTP * JCBH
                    WHATJ( C,R,M ) = DENF * DWA + SFAC( L ) * UVF

                    DENS( L ) = DENH
                    TEMP( L ) = DTA
                    PPRM( L ) = DPP

                END DO

            END DO          !  end loop on levels (wavefront 3-D computation)

            !!  Top model full-level:  set W==0

            L = NLAYS
            DO  C = 1, NCOLS
                J = J0 + C
                WHATJ( C,R,L ) = 0.0
            END DO

        END DO

        RETURN
        END

     
