
C.........................................................................
C Version "%W% %P% %G% %U%"
C MCPL output module for MM5 cpyright (C) 1998-2002 MCNC and
C Carlie J. Coats, Jr., and (C) 2003 Baron Advanced Meteorological Systems
C This routine copyright (C) 2003 BAMS.
C Distributed under the GNU LESSER GENERAL PUBLIC LICENSE version 2.1
C See file "LGPL.txt" for conditions of use.
C.........................................................................

        SUBROUTINE MM2HTR( IM, JM, KM, NEARSFC, TKICE, TKLIQ, TKFRZ,
     &                     TA, QV, QR, QI, PP, PSTAR, RAINC, RAINNC,
     &                     SIGH, PTOP,
     &                     NCOLS, NROWS, NLAYS,
     &                     COL0,  ROW0,  LAY0,
     &                     TWSFC, TWBAR, LFRAC, QRTOT, QITOT,
     &                     SIGW0, RAIN )

C***********************************************************************
C  subroutine body starts at line  130
C
C  FUNCTION:
C       Compute layer-1 and near-surface-averaged wet bulb temperatures
C       TWSFC and TWBAR, liquid-water-fraction TWBAR such that
C       freezing <= 0 < mixed < 1 <= liquid, 
C       and 3x3-aggregated layer 1 QR and QI, using values of temperature, 
C       precipitation, etc. extracted/reordered/converted/decoupleed/computed
C       from MM5 2-D and 3-D arrays.
C
C  NOTE that MM5-order I subscripts go from south to north, J subscripts
C	go from west to east, and K subscripts go from top to bottom,
C	whereas IOAPI-order C subscripts go from west to east,
C	R subscripts go from south to north, and L subscripts go
C	from bottom to top.
C
C  PRECONDITIONS REQUIRED:
C	ioapi grid fits within the MM5 grid
C
C  SUBROUTINES AND FUNCTIONS CALLED: none
C
C  REVISION  HISTORY:
C	Prototype 3/2003 by Carlie J. Coats, Jr., BAMS
C       Version   5/2003 that does wet-bulb calculation
C       Version   7/2003 that calculates SIGMA-value SIG0 such that
C       TWET( SIG0 ) = 0.0 C; total rain = RAINC+RAINNC
C***********************************************************************

      IMPLICIT NONE

C...........   ARGUMENTS and their descriptions:

        INTEGER		IM	!  I (north-south) dimension for AM
        INTEGER		JM	!  J (east-west)   dimension for AM
        INTEGER		KM	!  K (top-bottom)  dimension for AM
        INTEGER         NEARSFC !  # of layers for averaging
        REAL            TKICE, TKLIQ, TKFRZ
        REAL		TA( IM, JM, KM )	!  MM5-order input array
        REAL		QV( IM, JM, KM )	!  MM5-order input array
        REAL		QR( IM, JM, KM )	!  MM5-order input array
        REAL		QI( IM, JM, KM )	!  MM5-order input array
        REAL		PP( IM, JM, KM )	!  MM5-order input array
        REAL		PSTAR( IM, JM )		!  MM5-order pstar array
        REAL		RAINC( IM, JM )		!  MM5-order convective rain
        REAL		RAINNC( IM, JM )	!  MM5-order nonconvective rain
        REAL		SIGH( KM )		!  sigma half-levels
        REAL            PTOP    !  MM5 ptop (cb)
        INTEGER         NCOLS	!  output dimension
        INTEGER         NROWS	!  " 
        INTEGER         NLAYS	!  "
        INTEGER         COL0	!  starting subscript for output window
        INTEGER         ROW0	!  "
        INTEGER         LAY0	!  "
	REAL		TWSFC( NCOLS, NROWS )
	REAL		TWBAR( NCOLS, NROWS )
	REAL		LFRAC( NCOLS, NROWS )
	REAL		QRTOT( NCOLS, NROWS )
	REAL		QITOT( NCOLS, NROWS )
	REAL		SIGW0( NCOLS, NROWS )   !  sigma-height for TWET=0(C)
	REAL		RAIN( NCOLS, NROWS )    !  total rain

C...........   PARAMETERs

        REAL            ICE_T, RAIN_T
        REAL	    	CPA        !  conversion factor, Pa to cb
        REAL	    	C1
        REAL	    	C15
        REAL	    	C2
        REAL	    	CP
        REAL	    	D611
        REAL	    	E1
        REAL	    	E2
        REAL	    	EPS
        REAL	    	F
        INTEGER         MAXC
        PARAMETER     ( ICE_T  =     9.0E-5 ,
     &                  RAIN_T =     9.0E-7 ,
     &                  CPA    =     1.0E3  ,
     &                  C1     =     0.0091379024 ,
     &                  C15    =    26.66082 ,
     &                  C2     =  6106.396 ,
     &                  CP     =  1004.0 ,
     &                  D611   =     1.0 / 6.11 ,
     &                  E1     =  5418.12 ,
     &                  E2     =    19.84659 ,
     &                  EPS    =     0.622 ,
     &                  F      =     0.0006355 ,
     &                  MAXC   = 512 )

C...........   SCRATCH LOCAL VARIABLES and their descriptions:

	INTEGER		C, R, L, N	        !  I/O API col subscript
	INTEGER		I, J, K 		!  MM5 subscripts
	INTEGER		I0, J0, K0 		!  MM5 offsets
        REAL            DLAY, DFRZ
        REAL            RL, RI, T, TW, DZ
        REAL            LPS, LPH, LPF
        REAL            DPP, DQV, PR, QQ, WW, EE, TD, ES, ED, S
        REAL            EW, DE, DER
        REAL            PRH1, PRH2
        REAL            LPH1, LPH2

#ifdef LANGUAGE_FORTRAN90
        REAL		DFAC( NCOLS )
        REAL		SWET( NCOLS, NLAYS )
#endif    /* ifdef LANGUAGE_FORTRAN90  */
#ifndef LANGUAGE_FORTRAN90
        REAL		DFAC( MAXC )
        REAL		SWET( MAXC, MAXC )

C***********************************************************************
C   begin body of subroutine  MMXDCPL

        IF ( NCOLS.GT. MAXC .OR. KM .GT. MAXC ) THEN
            CALL M3EXIT( 'MCPL:MM2MASK', 0, 0,
     &      'SCRATCH-ARRAY error:  need parameter MAXC > NCOLS,KM', 2 )
        END IF
#endif    /* ifndef LANGUAGE_FORTRAN90  */

        I0 = ROW0 - 1			!  offsets for IOAPI grid within MM5
        J0 = COL0 - 1
        K0 = KM - LAY0 + 2
        DLAY = 1.0 / FLOAT( NEARSFC )
        DFRZ = 1.0 / ( TKLIQ - TKICE )

!$OMP       PARALLEL DO DEFAULT( NONE ),
!$OMP&                   SHARED( I0, J0, K0, KM, NROWS, NCOLS, PSTAR,
!$OMP&                           TA, QV, QR, QI, PP, SIGH, PTOP,
!$OMP&                           NEARSFC, DLAY, DFRZ,
!$OMP&                           RAIN, RAINC, RAINNC, SIGW0,
!$OMP&                           TKICE, TKFRZ, TWSFC, TWBAR,
!$OMP&                           QRTOT, QITOT, LFRAC ),
!$OMP&                  PRIVATE( R, L, C, I, J, K, N, DFAC, T, DPP, DQV,
!$OMP&                           PR, QQ, WW, EE, TD, ES, ED, S, TW,
!$OMP&                           EW, DE, DER, RL, RI, PRH1, PRH2, 
!$OMP&                           LPH1, LPH2, DZ, SWET )

        DO  R = 1, NROWS
            I = I0 + R

            DO  C = 1, NCOLS
                J = J0 + C
                RAIN( C,R ) = RAINC( I,J ) + RAINNC( I,J )
            END DO

            K = K0 - 1
            DO  C = 1, NCOLS
                J = J0 + C

                DFAC( C ) = 1.0 / PSTAR( I,J )
                
                !!  iteration for wet-bulb temperature:
                
                T   = DFAC( C ) * TA( I,J,K )
                DPP = DFAC( C ) * PP( I,J,K )
                DQV = DFAC( C ) * QV( I,J,K )
                PR  = 0.01*( (CPA*PSTAR(I,J)*SIGH(K) + PTOP) + DPP )  !  mb
                QQ  = MAX( DQV, 1.0E-15 )
                WW  = QQ / ( 1.0 - QQ )
                EE  = QQ * PR/( EPS + 0.378*QQ )
                TD  = AMAX1( E1 / (E2 - ALOG( EE*D611 ) ), 200. )
                ES  = EXP (C15 - C1*T  - C2/T)
                ED  = EXP (C15 - C1*TD - C2/TD)
                S   = (ES - ED) / (T - TD)
                TW  = (T*F*PR + TD*S) / ( F*PR + S)
                DO  N = 1, 4
                    EW  = EXP (C15 - C1*TW - C2/TW )
                    DE  = F*PR*(T - TW ) - (EW - ED)
                    DER = EW*(C1 - C2/TW**2) - F*PR
                    TW  = TW - DE/DER
                END DO

                TWSFC( C,R ) = TW
                TWBAR( C,R ) = TW
                SWET ( C,1 ) = TW

                RL = QR(I-1,J-1,K ) + QR(I,J-1,K ) + QR(I+1,J-1,K ) +
     &               QR(I-1,J  ,K ) + QR(I,J  ,K ) + QR(I+1,J  ,K ) +
     &               QR(I-1,J+1,K ) + QR(I,J+1,K ) + QR(I+1,J+1,K )

                RI = QI(I-1,J-1,K ) + QI(I,J-1,K ) + QI(I+1,J-1,K ) +
     &               QI(I-1,J  ,K ) + QI(I,J  ,K ) + QI(I+1,J  ,K ) +
     &               QI(I-1,J+1,K ) + QI(I,J+1,K ) + QI(I+1,J+1,K )

                QRTOT( C,R ) = DFAC( C ) * RL
                QITOT( C,R ) = DFAC( C ) * RI
            END DO

            DO  L = 2, NEARSFC

                K = K0 - L

                    DO  C = 1, NCOLS

                    J = J0 + C

                    !!  iteration for wet-bulb temperature:

                    T   = DFAC( C ) * TA( I,J,K )
                    DPP = DFAC( C ) * PP( I,J,K )
                    DQV = DFAC( C ) * QV( I,J,K )
                    PR  = 0.01*(CPA*PSTAR(I,J)*SIGH(K) + PTOP + DPP)  !  mb
                    QQ  = MAX( DQV, 1.0E-15 )
                    WW  = QQ / ( 1.0 - QQ )
                    EE  = QQ * PR/(EPS + 0.378*QQ)
                    TD  = AMAX1( E1/(E2 - ALOG( EE*D611 ) ), 200. )
                    ES  = EXP (C15 - C1*T  - C2/T)
                    ED  = EXP (C15 - C1*TD - C2/TD)
                    S   = (ES - ED) / (T - TD)
                    TW  = (T*F*PR + TD*S) / ( F*PR + S)
                    DO  N = 1, 4
                        EW  = EXP (C15 - C1*TW - C2/TW )
                        DE  = F*PR*(T - TW ) - (EW - ED)
                        DER = EW*(C1-C2/TW**2) - F*PR
                        TW  = TW    - DE/DER
                    END DO

                    TWBAR( C,R ) = TWBAR( C,R ) + TW
                    SWET( C,L ) = TW

                END DO

            END DO

            DO  C = 1, NCOLS

                J = J0 + C

                TWBAR( C,R ) = DLAY *   TWBAR( C,R )
                LFRAC( C,R ) = DFRZ * ( TWBAR( C,R ) - TKICE )

                IF ( SWET( C,1 ) .LT. 273.15 ) THEN

                    SIGW0( C,R ) = 1.0

                ELSE IF ( SWET( C,NEARSFC ) .LT. 273.15 ) THEN

                    DO L = 2, NEARSFC
                        K = K0 - L
                        IF ( SWET( C,L ) .LT. 273.15 ) THEN
                            SIGW0( C,R ) = SIGH( L-1 )
     &                            + ( SIGH( L ) - SIGH( L-1 ) )
     &                              *( SWET( C,L-1 ) - 273.15 )
     &                              /( SWET( C,L-1 ) - SWET( C,L ) )
                            GO TO  11
                        END IF
                    END DO

                ELSE    !  

                    DO L = NEARSFC+1, KM-1

                        K = K0 - L

                        !!  iteration for wet-bulb temperature:

                        T   = DFAC( C ) * TA( I,J,K )
                        DPP = DFAC( C ) * PP( I,J,K )
                        DQV = DFAC( C ) * QV( I,J,K )
                        PR  = 0.01*(CPA*PSTAR(I,J)*SIGH(K) + PTOP + DPP)  !  mb
                        QQ  = MAX( DQV, 1.0E-15 )
                        WW  = QQ / ( 1.0 - QQ )
                        EE  = QQ * PR/(EPS + 0.378*QQ)
                        TD  = AMAX1( E1/(E2 - ALOG( EE*D611 ) ), 200. )
                        ES  = EXP (C15 - C1*T  - C2/T)
                        ED  = EXP (C15 - C1*TD - C2/TD)
                        S   = (ES - ED) / (T - TD)
                        TW  = (T*F*PR + TD*S) / ( F*PR + S)
                        DO  N = 1, 4
                            EW  = EXP (C15 - C1*TW - C2/TW )
                            DE  = F*PR*(T - TW ) - (EW - ED)
                            DER = EW*(C1-C2/TW**2) - F*PR
                            TW  = TW    - DE/DER
                        END DO

                        IF ( TW .LT. 273.15 ) THEN
                            SIGW0( C,R ) = SIGH( L-1 )
     &                            + ( SIGH( L ) - SIGH( L-1 ) )
     &                              *( SWET( C,L-1 ) - 273.15 )
     &                              /( SWET( C,L-1 ) - TW )
                            GO TO  11
                        ELSE
                            SWET( C,L ) = TW
                        END IF

                    END DO

                    SIGW0( C,R ) = 0.0  !  if you get to here:  fallback value

                END IF

11              CONTINUE        !  exit from wet-bulb=zero calculation

            END DO

        END DO

        RETURN
        END

