
C Version "%W% %P% %G% %U%
C.........................................................................
C Version "%W% %P% %G% %U%"
C MCPL output module for MM5.  Copyright (C) 1999 MCNC
C Distributed under the GNU LESSER GENERAL PUBLIC LICENSE version 2.1
C See file "LGPL.txt" for conditions of use.
C.........................................................................

        SUBROUTINE  MM2J( IM, JM, PSTAR, MSFX, P00, ROVG,
     &                    NCOLS, NROWS, NLAYS,
     &                    COL0, ROW0,
     &                    TSFC, TH0, TF0, PSFC, PRH0, PRF0, 
     &                    JSFC, JH, JF )

C***********************************************************************
C  subroutine body starts at line  92
C
C  FUNCTION:  Extract/reorder/convert/decouple/compute the cross point
C	surface, half-level and full-level Jacobians JSFC, JH, JF from
C       the reference atmosphere, for the window specified in terms of
C       grid-window bounds and place the result in I/O API order.
C
C  NOTE that MM5-order I subscripts go from south to north, J subscripts
C       go from west to east, and K subscripts go from top to bottom,
C       whereas I/O API-order C subscripts go from west to east,
C       R subscripts go from south to north, and L subscripts go
C       from bottom to top.
C
C  PRECONDITIONS REQUIRED:
C       ioapi grid fits within the MM5 grid
C       PSFC, DENS, PRH0, PRF0, already calculated in I/O API-order 
C       by MM2REF
C
C  SUBROUTINES AND FUNCTIONS CALLED:
C       none
C
C  REVISION  HISTORY:
C       Prototype  12/2001 by Carlie J. Coats, Jr., MCNC Environmental 
C       Modeling Center
C***********************************************************************

      IMPLICIT NONE

C...........   ARGUMENTS and their descriptions:

        !!  input arguments:  

        INTEGER         IM              !  MM5 I (north-south) dimension
        INTEGER         JM      	!  MM5 J (east-west)   dimension
        REAL            PSTAR( IM, JM ) !  MM5 pstar array
        REAL            MSFX( IM, JM )  !  MM5 inverse map scale factor
        REAL            P00  		!  MM5 reference SL-pres (Pa)
        REAL            ROVG		!  MM5 dry gas constant / gravity
        INTEGER         NCOLS           !  output window dimension
        INTEGER         NROWS           !  " 
        INTEGER         NLAYS           !  "
        INTEGER         COL0            !  starting subscript for output window
        INTEGER         ROW0            !  "
        REAL            TSFC( NCOLS, NROWS )         !  ref sfc temperature
        REAL            TH0 ( NCOLS, NROWS, NLAYS )  !  ref sigh temperature
        REAL            TF0 ( NCOLS, NROWS, NLAYS )  !  ref sigf temperature
        REAL            PSFC( NCOLS, NROWS )         !  ref sfc pressure
        REAL            PRH0( NCOLS, NROWS, NLAYS )  !  ref sigh pressure
        REAL            PRF0( NCOLS, NROWS, NLAYS )  !  ref sigf pressure

        !!  output arguments:  

        REAL            JSFC( NCOLS, NROWS )         !  sfc  Jacobian
        REAL            JH( NCOLS, NROWS, NLAYS )    !  sigh Jacobian
        REAL            JF( NCOLS, NROWS, NLAYS )    !  sigf Jacobian


C...........   PARAMETERs

        REAL	    	CBTOPA        !  conversion factor, Pa to cb
        PARAMETER     ( CBTOPA = 1.0E3 )


C...........   SCRATCH LOCAL VARIABLES and their descriptions:

	INTEGER		C, R, L      !  IOAPI col subscript
        INTEGER         I, J         !  MM5 subscripts
        INTEGER         I0, J0       !  MM5 offsets
        REAL            CRDG         !  CPA * RDRY / G
        REAL            A, B         !  CPA*PSTAR*R*T/(G*P) = CPA*PSTAR/(G*DENS)
                                     !  where DENS = P/(R*T)

C***********************************************************************
C   begin body of subroutine  MM2J

        I0  = ROW0 - 1			!  offsets for IOAPI grid within MM5
        J0  = COL0 - 1
        CRDG= CBTOPA * ROVG

        !!  Use JSFC as a scratch variable for 
        !!  expression  CPA * RDRY * PSTAR( I,J ) * MSFX( I,J )**2 / G

        DO  R = 1, NROWS            !  surface; layer 1 processing
            I = I0 + R
            DO  C = 1, NCOLS
                J = J0 + C
                JSFC( C,R ) = CRDG * PSTAR( I,J ) * MSFX( I,J )**2
            END DO
        END DO

        DO  L = 1, NLAYS            !  upper-layer processing
        DO  R = 1, NROWS
            I = I0 + R
            DO  C = 1, NCOLS
                J = J0 + C
                JH( C,R,L ) = JSFC(C,R) * TH0( C,R,L ) / PRH0( C,R,L )
                JF( C,R,L ) = JSFC(C,R) * TF0( C,R,L ) / PRF0( C,R,L )
            END DO
        END DO
        END DO

        DO  R = 1, NROWS            !  surface; layer 1 processing
            I = I0 + R
            DO  C = 1, NCOLS
                J = J0 + C
                JSFC( C,R ) = JSFC( C,R ) * TSFC( C,R )/ PSFC( C,R )
            END DO
        END DO

        RETURN
        END
