
C Version "%W% %P% %G% %U%

        SUBROUTINE  MM2RAD( IM, JM, RAD, ALB,
     &                     NCOLS, NROWS, COL0, ROW0, ROUT )

C***********************************************************************
C  subroutine body starts at line  59
C
C  FUNCTION:  Extract/reorder/convert/compute net ground radiation
C	in terms of gross radiation and albedo, according to the 
C	formula R_out = RAD / ( 1 - ALB )
C
C  NOTE that MM5-order I subscripts go from south to north, J subscripts
C	go from west to east, and K subscripts go from top to bottom,
C	whereas IOAPI-order C subscripts go from west to east,
C	R subscripts go from south to north, and L subscripts go
C	from bottom to top.
C
C  PRECONDITIONS REQUIRED:
C	ioapi grid fits within the MM5 grid
C
C  SUBROUTINES AND FUNCTIONS CALLED: none
C
C  REVISION  HISTORY:
C	Prototype OCT. 16, 1997 by Carlie J. Coats, Jr., MCNC
C
C***********************************************************************

      IMPLICIT NONE

C...........   ARGUMENTS and their descriptions:

        INTEGER		IM	!  I (north-south) dimension for AM
        INTEGER		JM	!  J (east-west)   dimension for AM
        REAL		RAD( IM, JM )	!  MM5-order input array
        REAL		ALB( IM, JM )	!  MM5-order input array
        INTEGER         NCOLS	!  output dimension
        INTEGER         NROWS	!  " 
        INTEGER         COL0	!  starting subscript for output window
        INTEGER         ROW0	!  "
	REAL	      	ROUT( NCOLS, NROWS )

C...........   SCRATCH LOCAL VARIABLES and their descriptions:

	INTEGER		C     		!  IOAPI col subscript
	INTEGER		R     		!  IOAPI row subscript
	INTEGER		I, J 		!  MM5 subscripts
	INTEGER		I0, J0 		!  MM5 offsets

C***********************************************************************
C   begin body of subroutine  MM2RAD

        I0 = ROW0 - 1			!  offsets for IOAPI grid within MM5
        J0 = COL0 - 1

        DO  22  R = 1, NROWS
            I = I0 + R
            DO  11  C = 1, NCOLS
                J = J0 + C
                ROUT( C,R ) = RAD( I,J ) / ( 1.0 - ALB( I,J ) )
11          CONTINUE
22      CONTINUE

        RETURN
        END

