
C Version "%W% %P% %G% %U%
C.........................................................................
C Version "%W% %P% %G% %U%"
C MCPL output module for MM5.  Copyright (C) 1999 MCNC
C Distributed under the GNU LESSER GENERAL PUBLIC LICENSE version 2.1
C See file "LGPL.txt" for conditions of use.
C.........................................................................

        SUBROUTINE MM2THERM2 ( IM, JM, KM, 
     &                         TA, QV, PP, PST, HFX, QFX, SIGH, PTOP,
     &                         NCOLS, NROWS,
     &                         COL0, ROW0,
     &                         DENS, PRES, TV,
     &                         RHWI, RHLW, RHKF, QFLX, HFLX )

C***********************************************************************
C  subroutine body starts at line  130
C
C  FUNCTION:  Extract/reorder/convert/decouple/compute the cross point
C	3D density, pressure, virtual temperature, relative humidity,
C       and potential temperature for the window specified in terms of 
C       grid-window bounds to I/O API order, using TA, QV, Pprime, Pstar, 
C       SIGH, and Ptop from MM5.  If NLAYS=1, also rescales fluxes 
C       QFLX, HFLX by density.
C
C  NOTE that MM5-order I subscripts go from south to north, J subscripts
C       go from west to east, and K subscripts go from top to bottom,
C       whereas IOAPI-order C subscripts go from west to east,
C       R subscripts go from south to north, and L subscripts go
C       from bottom to top.
C
C  PRECONDITIONS REQUIRED:
C       ioapi grid fits within the MM5 grid
C
C  SUBROUTINES AND FUNCTIONS CALLED:
C       none
C
C  REVISION  HISTORY:
C       Adapted 2/2001 by Carlie J. Coats, Jr., MCNC from "mm2dp2.F".
C           Three different versions of relative humidity supported:
C               RHWI has liquid-water based RH above CTOK, ice-based RH
C               below CTOK
C               RHLW has liquid-water based RH only
C               RHKF uses liquid-water based RH above TTFRZ,  ice-based RH
C               below TBFRZ, and linear mixture of the two in between.
C               (as used in KF cloud parameterization and elsewhere)
C***********************************************************************

      IMPLICIT NONE

C...........   ARGUMENTS and their descriptions:

        INTEGER         IM      !  I (north-south) dimension for MM5
        INTEGER         JM      !  J (east-west)   dimension for MM5
        INTEGER         KM      !  K (top-bottom)  dimension for MM5
        REAL            TA( IM, JM, KM )        !  MM5 coupled temperature
        REAL            QV( IM, JM, KM )        !  MM5 coupled mixing ratio
        REAL            PP( IM, JM, KM )        !  MM5 P-prime
        REAL            PST( IM, JM )           !  MM5 pstar array
        REAL            HFX( IM, JM )           !  MM5 hfx array
        REAL            QFX( IM, JM )           !  MM5 QFX array
        REAL            SIGH( KM )              !  MM5 SIGMA-H
        REAL            PTOP			!  MM5 ptop
        INTEGER         NCOLS   !  output dimension
        INTEGER         NROWS   !  " 
        INTEGER         COL0    !  starting subscript for output window
        INTEGER         ROW0    !  "
        REAL            PRES( NCOLS, NROWS ) !  pressure (cb)
        REAL            DENS( NCOLS, NROWS ) !  density
        REAL            TV  ( NCOLS, NROWS ) !  virtual temperature
        REAL            RHWI( NCOLS, NROWS ) !  relative humidity
        REAL            RHLW( NCOLS, NROWS ) !  rh, liq. water version
        REAL            RHKF( NCOLS, NROWS ) !  rh, kf mixed-phase version
        REAL            QFLX( NCOLS, NROWS ) !  kinematic  latent  heat flux
        REAL            HFLX( NCOLS, NROWS ) !  kinematic sensible heat flux

C...........   PARAMETERs

        REAL	    	CBTOPA
        REAL	    	RDGAS		 !  from param.F, line 1573 & ff
	REAL            CP		 !  "
	REAL            CTOK		 !  Celsius-to-Kelvin
	REAL            TBFRZ		 !  ice-only threshold
	REAL            TTFRZ		 !  water-only threshold
	REAL            DDFRZ		 !  1/(ttfrz-tbfrz)
	REAL            ROVCP		 ! RDGAS/CP

        PARAMETER     ( CBTOPA = 1.0E3 , !  Conversion: CB to Pa
     &                  RDGAS  =  287.04, !  to preserve MM5 compatibility
     &                  CP     = 1004.0 , !  "
     &                  CTOK  = 273.15 ,
     &                  TBFRZ  = 248.16 ,
     &                  TTFRZ  = 268.16 ,
     &                  DDFRZ  = 1.0 / ( TTFRZ - TBFRZ ) ,
     &                  ROVCP  = RDGAS / CP )
	       	
        REAL            ALIQ
        REAL            BLIQ
        REAL            CLIQ
        REAL            DLIQ
        REAL            AICE
        REAL            BICE
        REAL            CICE
        REAL            DICE
        REAL            F622

        PARAMETER     ( ALIQ   =  613.3,
     &                  BLIQ   =   17.502,
     &                  CLIQ   = 4780.8,
     &                  DLIQ   =   32.19,
     &                  AICE   =  613.2,
     &                  BICE   =   22.452,
     &                  CICE   = 6133.0,
     &                  DICE   =    0.61 ,
     &                  F622   =  100.0 / 0.622 )

C...........   SCRATCH LOCAL VARIABLES and their descriptions:

	INTEGER		C, R    	        !  IOAPI col subscript
        INTEGER         I, J, K                 !  MM5 subscripts
        INTEGER         I0, J0                  !  MM5 offsets
        
        REAL            DTA, DQV, DPP		!  decoupled TA,QV,PP
        REAL            ES, ESL, ESI, RAT, DIV, DCP, DDD, RHL

C***********************************************************************
C   begin body of subroutine  MM2DPT

        I0  = ROW0 - 1                   !  offsets for IOAPI grid within MM5
        J0  = COL0 - 1
        K   = KM

        DCP = -1.0 / CP

        DO  R = 1, NROWS
            I = I0 + R
            DO  C = 1, NCOLS
                J = J0 + C
                DDD = 1.0 / PST( I,J )
                DTA = DDD * TA( I,J,K )	!  decoupled
                DQV = DDD * QV( I,J,K )	!   "
                DPP = DDD * PP( I,J,K )     !  ", pa
                TV  ( C,R ) = DTA * ( 1.0 + 0.608 * DQV )
                PRES( C,R ) = CBTOPA * ( PST( I,J ) * SIGH( K )
     &                                   + PTOP ) + DPP
                DENS( C,R ) = PRES( C,R )/( RDGAS*TV( C,R ) )
  
                !!  liquid water single-phase RH formulation:

                ESL = ALIQ * EXP( ( BLIQ * DTA - CLIQ )
     &                              / ( DTA - DLIQ ) )
                RHL = F622*DQV*( PRES( C,R ) - ESL )/ESL
                RHLW( C,R ) = MIN( MAX( RHL, 0.0 ), 100.0 )

                !!  water/ice two-phase RH formulation:

                IF ( DTA .LT. CTOK ) THEN            !  water may be ice
                    ESI = AICE * EXP( ( BICE * DTA - CICE )
     &                              / ( DTA - DICE ) )
                    RHL = F622*DQV*( PRES( C,R ) - ESI )/ESI
                    RHWI( C,R ) = MIN( MAX( RHL, 0.0 ), 100.0 )
                ELSE
                    RHWI( C,R ) = RHLW( C,R )
                END IF

                !!  KF mixed-phase RH formulation:

                IF ( DTA .GE. TTFRZ ) THEN           !  water is liquid
                    RHKF( C,R ) = RHLW( C,R )
                ELSE IF ( DTA .LE. TBFRZ ) THEN      !  else below freezing
                    RHKF( C,R ) = RHWI( C,R )
                ELSE
                    RAT = DDFRZ * ( TTFRZ - DTA )
                    ES  = ESL  +  RAT * ( ESI - ESL )
                    RHL = F622*DQV*( PRES( C,R ) - ES ) / ES
                    RHKF( C,R ) = MIN( MAX( RHL, 0.0 ), 100.0 )
                END IF                              !  if liquid or frozen

                DIV = 1.0 / DENS( C,R )
                HFLX( C,R ) = HFX( I,J ) * DCP * DIV
                QFLX( C,R ) = QFX( I,J ) * DIV
            END DO
        END DO

        RETURN
        END

