
C Version "%W% %P% %G% %U%
C.........................................................................
C Version "%W% %P% %G% %U%"
C MCPL output module for MM5.  Copyright (C) 1999 MCNC
C Distributed under the GNU LESSER GENERAL PUBLIC LICENSE version 2.1
C See file "LGPL.txt" for conditions of use.
C.........................................................................

        SUBROUTINE MM2THERM3( IM,JM,KM, TA, QV, PP, PST, SIGH, PTOP,
     &                        NCOLS, NROWS, NLAYS,
     &                        COL0, ROW0, LAY0, 
     &                        DENS, PRES, TV, RHWI, RHLW, RHKF, THTA )

C***********************************************************************
C  subroutine body starts at line  142
C
C  FUNCTION:  Extract/reorder/convert/decouple/compute the cross point
C	3D density, pressure, virtual temperature, relative humidity,
C       and potential temperature for the window specified in terms of 
C       grid-window bounds to I/O API order, using TA, QV, Pprime, Pstar, 
C       SIGH, and Ptop from MM5. 
C
C  NOTE that MM5-order I subscripts go from south to north, J subscripts
C       go from west to east, and K subscripts go from top to bottom,
C       whereas IOAPI-order C subscripts go from west to east,
C       R subscripts go from south to north, and L subscripts go
C       from bottom to top.
C
C  PRECONDITIONS REQUIRED:
C       ioapi grid fits within the MM5 grid
C
C  SUBROUTINES AND FUNCTIONS CALLED:
C       none
C
C  REVISION  HISTORY:
C       Adapted 2/2001 by Carlie J. Coats, Jr., MCNC from "mm2dp3.F".
C           Three different versions of relative humidity supported:
C               RHWI has liquid-water based RH above CTOK, ice-based RH
C               below CTOK
C               RHLW has liquid-water based RH only
C               RHKF uses liquid-water based RH above TTFRZ,  ice-based RH
C               below TBFRZ, and linear mixture of the two in between.
C               (as used in KF cloud parameterization and elsewhere)
C***********************************************************************

      IMPLICIT NONE

C...........   ARGUMENTS and their descriptions:

        INTEGER         IM      !  I (north-south) dimension for AM
        INTEGER         JM      !  J (east-west)   dimension for AM
        INTEGER         KM      !  K (top-bottom)  dimension for AM
        REAL            TA( IM, JM, KM )        !  MM5 coupled temperature
        REAL            QV( IM, JM, KM )        !  MM5 coupled mixing ratio
        REAL            PP( IM, JM, KM )        !  MM5 P-prime
        REAL            PST( IM, JM )           !  MM5 pstar array
        REAL            SIGH( KM )              !  MM5 SIGMA-H
        REAL            PTOP			!  MM5 ptop
        INTEGER         NCOLS   !  output dimension
        INTEGER         NROWS   !  " 
        INTEGER         NLAYS   !  "
        INTEGER         COL0    !  starting subscript for output window
        INTEGER         ROW0    !  "
        INTEGER         LAY0    !  "
        REAL            PRES( NCOLS, NROWS, NLAYS ) !  pressure (cb)
        REAL            DENS( NCOLS, NROWS, NLAYS ) !  density
        REAL            TV  ( NCOLS, NROWS, NLAYS ) !  virtual temperature
        REAL            RHWI( NCOLS, NROWS, NLAYS ) !  rh, 2-phase version
        REAL            RHLW( NCOLS, NROWS, NLAYS ) !  rh, liq. water version
        REAL            RHKF( NCOLS, NROWS, NLAYS ) !  rh, kf version
        REAL            THTA( NCOLS, NROWS, NLAYS ) !  potential temperature

C...........   EXTERNAL FUNCTIONS:

#if __sgi  || __sun || __osf__ || __mips__
        INTEGER         MALLOC
#endif    /* __sgi  || __sun || __osf__ || __mips__ */


C...........   PARAMETERs

        REAL	    	RDGAS		 !  from param.F, line 1573 & ff
        REAL            CP		 !  "
        REAL            CTOK		 !  Celsius-to-Kelvin
        REAL            TBFRZ		 !  ice-only threshold
        REAL            TTFRZ		 !  water-only threshold
        REAL            DDFRZ		 !  1/(ttfrz-tbfrz)
        REAL            ROVCP		 !  RDGAS/CP
        PARAMETER     ( RDGAS =  287.04, !  to preserve MM5 compatibility
     &                  CP    = 1004.0 , !  "
     &                  CTOK  = 273.15 ,
     &                  TBFRZ = 248.16 ,
     &                  TTFRZ = 268.16 ,
     &                  DDFRZ = 1.0 / ( TTFRZ - TBFRZ ) ,
     &                  ROVCP = RDGAS / CP )
               	
        REAL	    	CBTOPA
        REAL            ALIQ
        REAL            BLIQ
        REAL            CLIQ
        REAL            DLIQ
        REAL            AICE
        REAL            BICE
        REAL            CICE
        REAL            DICE
        REAL            F622

        PARAMETER     ( CBTOPA = 1.0E3 ,
     &                  ALIQ   =  613.3,
     &                  BLIQ   =   17.502,
     &                  CLIQ   = 4780.8,
     &                  DLIQ   =   32.19,
     &                  AICE   =  613.2,
     &                  BICE   =   22.452,
     &                  CICE   = 6133.0,
     &                  DICE   =    0.61 ,
     &                  F622   =  100.0 / 0.622 )

C...........   SCRATCH LOCAL VARIABLES and their descriptions:

        INTEGER		C, R, L		        !  IOAPI col subscript
        INTEGER         I, J, K                 !  MM5 subscripts
        INTEGER         I0, J0, K0              !  MM5 offsets
        
        REAL            DTA, DQV, DPP		!  decoupled TA,QV,PP
        REAL            ES, ESL, ESI, FAC, RHL, RAT
        REAL            DFAC( NCOLS, NROWS )

#if __sgi  || __sun || __osf__ || __mips__
        INTEGER         LSIZE
        DATA            LSIZE  / -1 /
        POINTER         ( P, DFAC )

        SAVE            P, LSIZE
#endif    /* __sgi  || __sun || __osf__ || __mips__ */

C***********************************************************************
C   begin body of subroutine  MM2DPT

#if __sgi  || __sun || __osf__ || __mips__

        IF ( NCOLS*NROWS .GT. LSIZE ) THEN
            IF ( LSIZE .GT. 0 )  CALL FREE( P )
            P = MALLOC( 4 * NCOLS*NROWS )
            IF ( P .EQ. 0 )
     &           CALL M3EXIT( 'MMXDCPL', 0, 0,
     &                       'Memory allocation error', 2 )
            LSIZE = NCOLS*NROWS
        END IF

#endif    /* __sgi  || __sun || __osf__ || __mips__ */
        
        I0  = ROW0 - 1                   !  offsets for IOAPI grid within MM5
        J0  = COL0 - 1
        FAC = 1.0E5**ROVCP

        K0 = KM - LAY0 + 2

        DO  22  R = 1, NROWS
            I = I0 + R
            DO  11  C = 1, NCOLS
                J = J0 + C
                DFAC( C,R ) = 1.0 / PST( I,J )
11              CONTINUE
22          CONTINUE

        DO  55  L = 1, NLAYS
            K = K0 - L
            DO  44  R = 1, NROWS
                I = I0 + R
                DO  33  C = 1, NCOLS
                    J = J0 + C

                    DTA = DFAC( C,R ) * TA( I,J,K )	!  decoupled
                    DQV = DFAC( C,R ) * QV( I,J,K )	!   "
                    DPP = DFAC( C,R ) * PP( I,J,K )     !  ", pa
                    TV  ( C,R,L ) = DTA * ( 1.0 + 0.608 * DQV )
                    PRES( C,R,L ) = CBTOPA * ( PST( I,J )*SIGH( K )
     &                                       + PTOP ) + DPP
                    DENS( C,R,L ) = PRES( C,R,L ) 
     &                              / ( RDGAS * TV( C,R,L ) )
                    THTA( C,R,L ) = FAC*DTA*PRES( C,R,L )**(-ROVCP)

                    !!  liquid water single-phase RH formulation:

                    ESL = ALIQ * EXP( ( BLIQ * DTA - CLIQ )
     &                                  / ( DTA - DLIQ ) )
                    RHL = F622*DQV*( PRES( C,R,L ) - ESL )/ESL
                    RHLW( C,R,L ) = MIN( MAX( RHL, 0.0 ), 100.0 )
                    
                    !!  water/ice two-phase RH formulation:

                    IF ( DTA .LT. CTOK ) THEN            !  water may be ice
                        ESI = AICE * EXP( ( BICE * DTA - CICE )
     &                                  / ( DTA - DICE ) )
                        RHL = F622*DQV*( PRES( C,R,L ) - ESI )/ESI
                        RHWI( C,R,L ) = MIN( MAX( RHL, 0.0 ), 100.0 )
                    ELSE
                        RHWI( C,R,L ) = RHLW( C,R,L )
                    END IF

                    !!  KF mixed-phase RH formulation:

                    IF ( DTA .GE. TTFRZ ) THEN           !  water is liquid
                        RHKF( C,R,L ) = RHLW( C,R,L )
                    ELSE IF ( DTA .LE. TBFRZ ) THEN      !  else below freezing
                        RHKF( C,R,L ) = RHWI( C,R,L )
                    ELSE
                        ESI = AICE * EXP( ( BICE * DTA - CICE )
     &                                  / ( DTA - DICE ) )
                        RAT = DDFRZ * ( TTFRZ - DTA )
                        ES  = ESL  +  RAT * ( ESI - ESL )
                        RHL = F622*DQV*( PRES( C,R,L ) - ES ) / ES
                        RHKF( C,R,L ) = MIN( MAX( RHL, 0.0 ), 100.0 )
                    END IF                              !  if liquid or frozen

33                  CONTINUE
44              CONTINUE
55          CONTINUE

        RETURN
        END

