
C Version "%W% %P% %G% %U%

        SUBROUTINE  MMBREF( IM, JM, KM, PSTAR, SIGH, SIGF, 
     &                      PTOP, P00, TLP, TS0, ROVG, RDRY,
     &                      NCOLS, NROWS, NLAYS, NTHIK, 
     &                      COL0, ROW0, LAY0, 
     &                      TSFC, PSFC,
     &                      DENS, ZH, ZF, DZF, TH0, TF0, PRH0, PRF0 )

C***********************************************************************
C  subroutine body starts at line  68
C
C  FUNCTION:  Extract/reorder/convert/decouple the boundary
C	reference atmosphere:  surface temperature and pressure,
C       3D half-level density and half and full level temperature 
C       and pressure, for the specified boundary window .
C
C  NOTE that MM5-order I subscripts go from south to north, J subscripts
C       go from west to east, and K subscripts go from top to bottom,
C       whereas IOAPI-order C subscripts go from west to east,
C       R subscripts go from south to north, and L subscripts go
C       from bottom to top.
C
C  PRECONDITIONS REQUIRED:
C	ioapi grid fits within the MM5 grid
C       nthik > 0
C
C  SUBROUTINES AND FUNCTIONS CALLED: none
C
C  REVISION  HISTORY:
C       Prototype May 6, 1998 by Carlie J. Coats, Jr., MCNC
C
C***********************************************************************

      IMPLICIT NONE

C...........   ARGUMENTS and their descriptions:

        INTEGER  IM      !  I (north-south) dimension for AM
        INTEGER  JM      !  J (east-west)   dimension for AM
        INTEGER  KM      !  K (top-bottom)  dimension for AM
        REAL     PSTAR( IM, JM )  !  MM5-order PSTAR array
        REAL     SIGH( KM )       !  MM5 SIGMA-H
        REAL     SIGF( KM+1 )     !  MM5 SIGMA-F
        REAL     PTOP    !  MM5 ptop (cb)
        REAL     P00  	 !  MM5 reference SL-pres (Pa)
        REAL     TLP     !  MM5 temp ref lapse rate (K/log(P))
        REAL     TS0	 !  MM5 ref SL temp (K)
        REAL     ROVG	 !  MM5 R/G
        REAL     RDRY	 !  MM5 dry gas constant
        INTEGER  NCOLS   !  output dimension
        INTEGER  NROWS   !  " 
        INTEGER  NLAYS   !  "
        INTEGER  NTHIK   !  perimeter thickness
        INTEGER  COL0    !  starting subscript for output window
        INTEGER  ROW0    !  "
        INTEGER  LAY0    !  "
        REAL     TSFC( 2*NTHIK*( NCOLS + NROWS + 2*NTHIK ) )  !  ref sfc temperature
        REAL     PSFC( 2*NTHIK*( NCOLS + NROWS + 2*NTHIK ) )  !  ref sfc pressure
        REAL     DENS( 2*NTHIK*( NCOLS + NROWS + 2*NTHIK ), NLAYS )  !  ref density
        REAL     TH0 ( 2*NTHIK*( NCOLS + NROWS + 2*NTHIK ), NLAYS )  !  ref sigh temperature
        REAL     TF0 ( 2*NTHIK*( NCOLS + NROWS + 2*NTHIK ), NLAYS )  !  ref sigf temperature
        REAL     PRH0( 2*NTHIK*( NCOLS + NROWS + 2*NTHIK ), NLAYS )  !  ref sigh pressure
        REAL     PRF0( 2*NTHIK*( NCOLS + NROWS + 2*NTHIK ), NLAYS )  !  ref sigf pressure
        REAL     ZH  ( 2*NTHIK*( NCOLS + NROWS + 2*NTHIK ), NLAYS )  !  ref sigh Z
        REAL     ZF  ( 2*NTHIK*( NCOLS + NROWS + 2*NTHIK ), NLAYS )  !  ref sigf Z
        REAL     DZF ( 2*NTHIK*( NCOLS + NROWS + 2*NTHIK ), NLAYS )  !  layer thickness 

C...........   EXTERNAL FUNCTIONS:

#if __sgi  || __sun || __osf__ || __mips__
        INTEGER         MALLOC
#endif    /* __sgi  || __sun || __osf__ || __mips__ */


C...........   PARAMETERs

        REAL	    	CPA        !  conversion factor, Pa to cb
        PARAMETER     ( CPA = 1.0E3 )

C...........   SCRATCH LOCAL VARIABLES and their descriptions:

        INTEGER         C, R, L             !  IOAPI subscripts
        INTEGER         Q                   !  boundary-cell subscript
        INTEGER         I, J, K             !  MM5 subscripts
        INTEGER         I0, J0, K0          !  MM5 offsets
        INTEGER         NPERM               !  ioapi dimensions
        INTEGER         R1, R2, R3, R4      !  perimeter-boundary rows
        INTEGER         C1, C2, C3, C4      !  perimeter-boundary cols
        REAL            DP0                 !  1/p0
        REAL            DR                  !  1/rdry
        REAL            LPS, LPH, LPF       !  log-subexpressions
        

C.......   Terrain height:

        REAL            ZT( 2*NTHIK*( NCOLS + NROWS + 2*NTHIK ) )

#if __sgi  || __sun || __osf__ || __mips__
	INTEGER         LSIZE  / -1 /
	POINTER         ( PT, ZT )

	SAVE            PT, LSIZE
#endif    /* __sgi  || __sun || __osf__ || __mips__ */

C***********************************************************************
C   begin body of subroutine  MMBREF

        NPERM = 2 * NTHIK * ( NCOLS + NROWS + 2*NTHIK )

#if __sgi  || __sun || __osf__ || __mips__
	IF ( NPERM .GT. LSIZE ) THEN
	    IF ( LSIZE .GT. 0 )  CALL FREE( PT )
	    PT = MALLOC( 8 * NPERM )
	    IF ( PT .EQ. 0 )
     &           CALL M3EXIT( 'MMBREF', 0, 0,
     &                       'Memory allocation error', 2 )
	    LSIZE = NPERM
	END IF
#endif    /* __sgi  || __sun || __osf__ || __mips__ */
	
        IF ( NTHIK .GT. 0 ) THEN        ! external bdy
            R1 = 1 - NTHIK
            R2 = 0
            R3 = NROWS + 1
            R4 = NROWS + NTHIK
            C1 = 1 - NTHIK
            C2 = 0
            C3 = NCOLS + 1
            C4 = NCOLS + NTHIK
        ELSE                            !  else nthik negative:  internal bdy
            R1 = 1
            R2 = NTHIK - 1
            R3 = NROWS - NTHIK + 1
            R4 = NROWS
            C1 = 1
            C2 = NTHIK - 1
            C3 = NCOLS - NTHIK + 1
            C4 = NCOLS
        END IF                          !  if nthik positive or not

        I0 = ROW0 - 1                     !  offsets for IOAPI grid within MM5
        J0 = COL0 - 1
        K0 = KM - LAY0 + 2


C.......   First, compute layer-1

        DP0 = 1.0 / P00
        DR  = 1.0 / RDRY
        Q   = 0
        K   = K0 - 1
        L   = 1
           
        DO  22  R = R1, R2          !  south component
            I = I0 + R
            DO  11  C = 1, C4
            
                J = J0 + C
                Q = Q  + 1
                
                PSFC( Q ) = CPA*( PSTAR( I,J ) + PTOP )
                TSFC( Q ) = TS0 + TLP*ALOG( PSFC( Q )*DP0 )
                PRH0( Q,L ) = CPA*(PSTAR( I,J )*SIGH( K ) + PTOP)
                PRF0( Q,L ) = CPA*(PSTAR( I,J )*SIGF( K ) + PTOP)
                LPS = ALOG( PSFC( Q )   * DP0 )
                LPH = ALOG( PRH0( Q,L ) * DP0 )
                LPF = ALOG( PRF0( Q,L ) * DP0 )
                TH0( Q,L ) = TS0 + TLP*LPH
                TF0( Q,L ) = TS0 + TLP*LPF
                ZT ( Q )   = -ROVG*LPS*( 0.5*TLP*LPS + TS0 )
                ZH ( Q,L ) = -ROVG*LPH*( 0.5*TLP*LPH + TS0 )
     &                          - ZT( Q )
                ZF ( Q,L ) = -ROVG*LPF*( 0.5*TLP*LPF + TS0 )
     &                          - ZT( Q )
                DZF ( Q,L ) = ZF( Q,L )
                DENS( Q,L ) = DR * PRH0( Q,L ) / TH0( Q,L )
                
11          CONTINUE
22      CONTINUE                    !  end south

        DO  44  R = 1, R4           !  east component
            I = I0 + R
            DO  33  C = C3, C4
            
                J = J0 + C
                Q = Q  + 1
                
                PSFC( Q ) = CPA*( PSTAR( I,J ) + PTOP )
                TSFC( Q ) = TS0 + TLP*ALOG( PSFC( Q )*DP0 )
                PRH0( Q,L ) = CPA*(PSTAR( I,J )*SIGH( K ) + PTOP)
                PRF0( Q,L ) = CPA*(PSTAR( I,J )*SIGF( K ) + PTOP)
                LPS = ALOG( PSFC( Q )   * DP0 )
                LPH = ALOG( PRH0( Q,L ) * DP0 )
                LPF = ALOG( PRF0( Q,L ) * DP0 )
                TH0( Q,L ) = TS0 + TLP*LPH
                TF0( Q,L ) = TS0 + TLP*LPF
                ZT ( Q )   = -ROVG*LPS*( 0.5*TLP*LPS + TS0 )
                ZH ( Q,L ) = -ROVG*LPH*( 0.5*TLP*LPH + TS0 )
     &                          - ZT( Q )
                ZF ( Q,L ) = -ROVG*LPF*( 0.5*TLP*LPF + TS0 )
     &                          - ZT( Q )
                DZF ( Q,L ) = ZF( Q,L )
                DENS( Q,L ) = DR * PRH0( Q,L ) / TH0( Q,L )
                
33          CONTINUE
44      CONTINUE                    !  end east

        DO  66  R = R3, R4          !  north component
            I = I0 + R
            DO  55  C = C1, NCOLS
            
                J = J0 + C
                Q = Q  + 1
                
                PSFC( Q ) = CPA*( PSTAR( I,J ) + PTOP )
                TSFC( Q ) = TS0 + TLP*ALOG( PSFC( Q )*DP0 )
                PRH0( Q,L ) = CPA*(PSTAR( I,J )*SIGH( K ) + PTOP)
                PRF0( Q,L ) = CPA*(PSTAR( I,J )*SIGF( K ) + PTOP)
                LPS = ALOG( PSFC( Q )   * DP0 )
                LPH = ALOG( PRH0( Q,L ) * DP0 )
                LPF = ALOG( PRF0( Q,L ) * DP0 )
                TH0( Q,L ) = TS0 + TLP*LPH
                TF0( Q,L ) = TS0 + TLP*LPF
                ZT ( Q )   = -ROVG*LPS*( 0.5*TLP*LPS + TS0 )
                ZH ( Q,L ) = -ROVG*LPH*( 0.5*TLP*LPH + TS0 )
     &                          - ZT( Q )
                ZF ( Q,L ) = -ROVG*LPF*( 0.5*TLP*LPF + TS0 )
     &                          - ZT( Q )
                DZF ( Q,L ) = ZF( Q,L )
                DENS( Q,L ) = DR * PRH0( Q,L ) / TH0( Q,L )
                
55          CONTINUE
66      CONTINUE                    !  end north

        DO  88  R = R1, NROWS          !  west component
            I = I0 + R
            DO  77  C = C1, C2
            
                J = J0 + C
                Q = Q  + 1
                
                PSFC( Q ) = CPA*( PSTAR( I,J ) + PTOP )
                TSFC( Q ) = TS0 + TLP*ALOG( PSFC( Q )*DP0 )
                PRH0( Q,L ) = CPA*(PSTAR( I,J )*SIGH( K ) + PTOP)
                PRF0( Q,L ) = CPA*(PSTAR( I,J )*SIGF( K ) + PTOP)
                LPS = ALOG( PSFC( Q )   * DP0 )
                LPH = ALOG( PRH0( Q,L ) * DP0 )
                LPF = ALOG( PRF0( Q,L ) * DP0 )
                TH0( Q,L ) = TS0 + TLP*LPH
                TF0( Q,L ) = TS0 + TLP*LPF
                ZT ( Q )   = -ROVG*LPS*( 0.5*TLP*LPS + TS0 )
                ZH ( Q,L ) = -ROVG*LPH*( 0.5*TLP*LPH + TS0 )
     &                          - ZT( Q )
                ZF ( Q,L ) = -ROVG*LPF*( 0.5*TLP*LPF + TS0 )
     &                          - ZT( Q )
                DZF ( Q,L ) = ZF( Q,L )
                DENS( Q,L ) = DR * PRH0( Q,L ) / TH0( Q,L )
                
77          CONTINUE
88      CONTINUE                    !  end west


C...........   Extract and apply decoupling-factors

        DO  199  L = 2, NLAYS

            K = K0 - L
            Q = 0

            DO  122  R = R1, R2          !  south component
                I = I0 + R
                DO  111  C = 1, C4
                
                    J = J0 + C
                    Q = Q  + 1
                    
                    PRH0( Q,L ) = CPA*( PSTAR( I,J )*SIGH( K ) + PTOP )
                    PRF0( Q,L ) = CPA*( PSTAR( I,J )*SIGF( K ) + PTOP )
                    LPH = ALOG( PRH0( Q,L ) * DP0 )
                    LPF = ALOG( PRF0( Q,L ) * DP0 )
                    TH0 ( Q,L ) = TS0 + TLP*LPH
                    TF0 ( Q,L ) = TS0 + TLP*LPF
                    ZH  ( Q,L ) = -ROVG*LPH*(0.5*TLP*LPH + TS0) - ZT(Q)
                    ZF  ( Q,L ) = -ROVG*LPF*(0.5*TLP*LPF + TS0) - ZT(Q)
                    DZF ( Q,L ) = ZF( Q,L ) - ZF( Q,L-1 )
                    DENS( Q,L ) = DR * PRH0( Q,L ) / TH0( Q,L )

111             CONTINUE
122         CONTINUE                    !  end south

            DO  144  R = 1, R4           !  east component
                I = I0 + R
                DO  133  C = C3, C4
                
                    J = J0 + C
                    Q = Q  + 1
                    
                    PRH0( Q,L ) = CPA*( PSTAR( I,J )*SIGH( K ) + PTOP )
                    PRF0( Q,L ) = CPA*( PSTAR( I,J )*SIGF( K ) + PTOP )
                    LPH = ALOG( PRH0( Q,L ) * DP0 )
                    LPF = ALOG( PRF0( Q,L ) * DP0 )
                    TH0 ( Q,L ) = TS0 + TLP*LPH
                    TF0 ( Q,L ) = TS0 + TLP*LPF
                    ZH  ( Q,L ) = -ROVG*LPH*(0.5*TLP*LPH + TS0) - ZT(Q)
                    ZF  ( Q,L ) = -ROVG*LPF*(0.5*TLP*LPF + TS0) - ZT(Q)
                    DZF ( Q,L ) = ZF( Q,L ) - ZF( Q,L-1 )
                    DENS( Q,L ) = DR * PRH0( Q,L ) / TH0( Q,L )

133             CONTINUE
144         CONTINUE                    !  end east

            DO  166  R = R3, R4          !  north component
                I = I0 + R
                DO  155  C = C1, NCOLS
                
                    J = J0 + C
                    Q = Q  + 1
                    
                    PRH0( Q,L ) = CPA*( PSTAR( I,J )*SIGH( K ) + PTOP )
                    PRF0( Q,L ) = CPA*( PSTAR( I,J )*SIGF( K ) + PTOP )
                    LPH = ALOG( PRH0( Q,L ) * DP0 )
                    LPF = ALOG( PRF0( Q,L ) * DP0 )
                    TH0 ( Q,L ) = TS0 + TLP*LPH
                    TF0 ( Q,L ) = TS0 + TLP*LPF
                    ZH  ( Q,L ) = -ROVG*LPH*(0.5*TLP*LPH + TS0) - ZT(Q)
                    ZF  ( Q,L ) = -ROVG*LPF*(0.5*TLP*LPF + TS0) - ZT(Q)
                    DZF ( Q,L ) = ZF( Q,L ) - ZF( Q,L-1 )
                    DENS( Q,L ) = DR * PRH0( Q,L ) / TH0( Q,L )

155             CONTINUE
166         CONTINUE                    !  end north

            DO  188  R = R1, NROWS          !  west component
                I = I0 + R
                DO  177  C = C1, C2
                
                    J = J0 + C
                    Q = Q  + 1
                    
                    PRH0( Q,L ) = CPA*( PSTAR( I,J )*SIGH( K ) + PTOP )
                    PRF0( Q,L ) = CPA*( PSTAR( I,J )*SIGF( K ) + PTOP )
                    LPH = ALOG( PRH0( Q,L ) * DP0 )
                    LPF = ALOG( PRF0( Q,L ) * DP0 )
                    TH0 ( Q,L ) = TS0 + TLP*LPH
                    TF0 ( Q,L ) = TS0 + TLP*LPF
                    ZH  ( Q,L ) = -ROVG*LPH*(0.5*TLP*LPH + TS0) - ZT(Q)
                    ZF  ( Q,L ) = -ROVG*LPF*(0.5*TLP*LPF + TS0) - ZT(Q)
                    DZF ( Q,L ) = ZF( Q,L ) - ZF( Q,L-1 )
                    DENS( Q,L ) = DR * PRH0( Q,L ) / TH0( Q,L )

177             CONTINUE
188         CONTINUE                    !  end west

199     CONTINUE                        !  end loop on layers

        RETURN
        END

