
C Version "%W% %P% %G% %U%

        SUBROUTINE  MMPDDCPL( FACTOR, IM, JM, KM, AM, PS,
     &                        NCOLS, NROWS, NLAYS,
     &                        COL0, ROW0, LAY0, LEVS, COEF, AOUT )

C***********************************************************************
C  subroutine body starts at line  89
C
C  FUNCTION:  Interpolate/reorder/convert/decouple the DOT-point sub-array 
C       specified in terms of grid-window bounds to I/O API order from 
C       array AM, which is found in standard MM5 subscript order, using
C       FACTOR for units-conversion and PSTAR for decoupling, using the
C       indices LEVS and coeffs COEF.  Stores zeros for levels at which
C       COEF is out of range (i.e., below terrain, if this is pressure
C       surface interpolation.
C
C  NOTE that MM5-order I subscripts go from south to north, J subscripts
C       go from west to east, and K subscripts go from top to bottom,
C       whereas IOAPI-order C subscripts go from west to east,
C       R subscripts go from south to north, and L subscripts go
C       from bottom to top.
C
C  PRECONDITIONS REQUIRED:
C	ioapi grid fits within the MM5 grid
C
C  SUBROUTINES AND FUNCTIONS CALLED: none
C
C  REVISION  HISTORY:
C       Prototype Jan. 26, 1999 by Carlie J. Coats, Jr., MCNC
C
C***********************************************************************

      IMPLICIT NONE

C...........   ARGUMENTS and their descriptions:

        REAL            FACTOR  !  conversion factor
        INTEGER         IM      !  I (north-south) dimension for AM
        INTEGER         JM      !  J (east-west)   dimension for AM
        INTEGER         KM      !  K (top-bottom)  dimension for AM
        REAL            AM( IM, JM, KM ) !  MM5-order input array
        REAL            PS( IM, JM )     !  MM5-order pstar array
        INTEGER         NCOLS   !  output dimension
        INTEGER         NROWS   !  " 
        INTEGER         NLAYS   !  "
        INTEGER         COL0    !  starting subscript for output window
        INTEGER         ROW0    !  "
        INTEGER         LAY0    !  "
        INTEGER         LEVS( NCOLS, NROWS, NLAYS )     !  interp indices
        REAL            COEF( NCOLS, NROWS, NLAYS )     !  interp coeffs
        REAL            AOUT( NCOLS, NROWS, NLAYS )

C...........   EXTERNAL FUNCTIONS:

#if __sgi  || __sun || __osf__ || __mips__
        INTEGER         MALLOC
#endif    /* __sgi  || __sun || __osf__ || __mips__ */


C...........   SCRATCH LOCAL VARIABLES and their descriptions:

	INTEGER		C, R, L		        !  IOAPI col subscript
        INTEGER         I, J, K                 !  MM5 subscripts
        INTEGER         I0, J0, K0              !  MM5 offsets
        
        REAL            DFAC( NCOLS, NROWS )    !  decouple factor
        REAL            PP, QQ


#if __sgi  || __sun || __osf__ || __mips__
	INTEGER         LSIZE  / -1 /
	POINTER         ( P, DFAC )

	SAVE            P, LSIZE
#endif    /* __sgi  || __sun || __osf__ || __mips__ */

C***********************************************************************
C   begin body of subroutine  MMDDCPL

#if __sgi  || __sun || __osf__ || __mips__
	IF ( NCOLS*NROWS .GT. LSIZE ) THEN
	    IF ( LSIZE .GT. 0 )  CALL FREE( P )
	    P = MALLOC( 4 * NCOLS*NROWS )
	    IF ( P .EQ. 0 )
     &           CALL M3EXIT( 'MMXDCPL', 0, 0,
     &                       'Memory allocation error', 2 )
	    LSIZE = NCOLS*NROWS
	END IF
#endif    /* __sgi  || __sun || __osf__ || __mips__ */
	
        I0 = ROW0 - 1                     !  offsets for IOAPI grid within MM5
        J0 = COL0 - 1
        K0 = KM - LAY0 + 2

        DO  22  R = 1, NROWS
            I = I0 + R
            DO  11  C = 1, NCOLS
                J = J0 + C
                DFAC( C,R ) = 4.0 * FACTOR / ( PS( I-1,J-1 ) +
     &                                         PS( I  ,J-1 ) +
     &                                         PS( I-1,J   ) +
     &                                         PS( I  ,J   ) )
11          CONTINUE
22      CONTINUE

        DO  55  L = 1, NLAYS
            DO  44  R = 1, NROWS
                I = I0 + R
                DO  33  C = 1, NCOLS
                    J  = J0 + C
                    K  = K0 - LEVS( C,R,L )
                    PP = COEF( C,R,L )
                    IF ( PP .LT. 1.0 ) THEN
                        QQ = 1.0 - PP
                        AOUT( C,R,L ) = DFAC( C,R )*( PP*AM( I,J,K )
     &                                              + QQ*AM( I,J,K-1))
                    ELSE
                        AOUT( C,R,L ) = 0.0
                    END IF
33              CONTINUE
44          CONTINUE
55      CONTINUE

        RETURN
        END

