
C Version "%W% %P% %G% %U%
C.........................................................................
C Version "%W% %P% %G% %U%"
C MCPL output module for MM5.  Copyright (C) 1999 MCNC
C Distributed under the GNU LESSER GENERAL PUBLIC LICENSE version 2.1
C See file "LGPL.txt" for conditions of use.
C.........................................................................

        SUBROUTINE  MMPDTRH( IM, JM, KM, 
     &                      TA, QV, PP, PST, SIGH, PTOP, P00,  
     &                      PLEV, PDEX, CLIN, CLOG,
     &                      NCOLS, NROWS, NLAYS,
     &                      COL0, ROW0, LAY0, 
     &                      T, DENS, TV, RH )

C***********************************************************************
C  subroutine body starts at line  130
C
C  FUNCTION:  Extract/reorder/convert/decouple/compute/interpolate the 
C       cross point 3D density, virtual temperature, relative humidity, 
C       and temperature to the specified pressure levels PLEV
C       for the window specified in terms of grid-window bounds.
C       Place the result in I/O API order, using
C       TA, QV, Pprime, Pstar, SIGH, and Ptop from MM5
C
C  NOTE that MM5-order I subscripts go from south to north, J subscripts
C       go from west to east, and K subscripts go from top to bottom,
C       whereas IOAPI-order C subscripts go from west to east,
C       R subscripts go from south to north, and L subscripts go
C       from bottom to top.
C
C  PRECONDITIONS REQUIRED:
C       ioapi grid fits within the MM5 grid
C
C  SUBROUTINES AND FUNCTIONS CALLED: none
C
C  REVISION  HISTORY:
C       Prototype  Jan. 29, 1999 by Carlie J. Coats, Jr., MCNC
C       Validated 2/15/99 CJC, JNM.
C
C***********************************************************************

      IMPLICIT NONE

C...........   ARGUMENTS and their descriptions:

        INTEGER         IM      !  I (north-south) dimension for AM
        INTEGER         JM      !  J (east-west)   dimension for AM
        INTEGER         KM      !  K (top-bottom)  dimension for AM
        REAL            TA( IM, JM, KM )        !  MM5 coupled temperature
        REAL            QV( IM, JM, KM )        !  MM5 coupled mixing ratio
        REAL            PP( IM, JM, KM )        !  MM5 P-prime (coupled)
        REAL            PST( IM, JM )           !  MM5 pstar (Cb)
        REAL            SIGH( KM )              !  MM5 SIGMA-H
        REAL            PTOP			!  MM5 ptop, cb
        REAL            P00  		!  MM5 reference SL-pres (Pa)
        REAL            PLEV( NLAYS )   !  requested pressure levels (Pa)
        INTEGER         PDEX( NCOLS, NROWS, NLAYS ) !  interpolation subscripts
        REAL            CLIN( NCOLS, NROWS, NLAYS ) !  lin-interp coeffs
        REAL            CLOG( NCOLS, NROWS, NLAYS ) !  log-interp coeffs
        INTEGER         NCOLS   !  output dimension
        INTEGER         NROWS   !  " 
        INTEGER         NLAYS   !  "
        INTEGER         COL0    !  starting subscript for output window
        INTEGER         ROW0    !  "
        INTEGER         LAY0    !  "
        REAL            T   ( NCOLS, NROWS, NLAYS ) !  temperature, K
        REAL            DENS( NCOLS, NROWS, NLAYS ) !  density
        REAL            TV  ( NCOLS, NROWS, NLAYS ) !  virtual temperature
        REAL            RH  ( NCOLS, NROWS, NLAYS ) !  relative humidity, %

C...........   PARAMETERs

        INTEGER         MAXK             !  dimension for scratch arrays
        REAL	    	CBTOPA           !  CB to Pa
        REAL	    	RDGAS		 !  from param.F, line 1573 & ff
	REAL            CP		 !  "
	REAL            CTOK		 !  Celsius-to-Kelvin
	REAL            ROVCP		 !  for p**kappa interpolation

        PARAMETER     ( CBTOPA = 1.0E3 ,
     &                  MAXK   = 200 ,
     &                  RDGAS =  287.04, !  to preserve MM5 compatibility
     &                  CP    = 1004.0 , !  "
     &                  CTOK  = 273.15 , !  "
     &                  ROVCP = RDGAS / CP  )
	       	
        REAL            ALIQ
        REAL            BLIQ
        REAL            CLIQ
        REAL            DLIQ
        REAL            AICE
        REAL            BICE
        REAL            CICE
        REAL            DICE
        REAL            F622

        PARAMETER     ( ALIQ   =  613.3,
     &                  BLIQ   =   17.502,
     &                  CLIQ   = 4780.8,
     &                  DLIQ   =   32.19,
     &                  AICE   =  613.2,
     &                  BICE   =   22.452,
     &                  CICE   = 6133.0,
     &                  DICE   =    0.61 ,
     &                  F622   =  100.0 / 0.622 )

C...........   SCRATCH LOCAL VARIABLES and their descriptions:

	INTEGER		C, R, L		!  IOAPI subscripts
        INTEGER         I, J, K, M      !  MM5 subscripts
        INTEGER         I0, J0, K0      !  MM5 offsets
        
        REAL            DTA, DPP        !  decoupled TA,PP
        REAL            ES, DFAC
        REAL            THL             !  theta on pressure-levels
        REAL            QVL             !  QV    on pressure-levels
        REAL            AA, BB          !  interpolation coeffs
        REAL            QVK( MAXK )     !  decoupled QV on sigma-levels
        REAL            THK( MAXK )     !  potential temp on sigma-levels
        REAL            TVK( MAXK )     !  virtual   temp on sigma-levels
        REAL            PRK             !  pressure on sigma-levels (PA)
        REAL            PRS1            !  press at lowest half-level (Pa)
        REAL            TL              !  T at requested level (Pa)

C***********************************************************************
C   begin body of subroutine  MM2DPT

        I0 = ROW0 - 1           !  offsets for IOAPI grid within MM5
        J0 = COL0 - 1
        K0 = KM - LAY0 + 2

        DO  22  R = 1, NROWS
            I = I0 + R
            DO  11  C = 1, NCOLS
                J = J0 + C
                DFAC = 1.0 / PST( I,J )

                DO  L = 1, KM   !  loop computing MM5-level values
                    K = K0 - L

                    DPP    = DFAC * PP( I,J,K )    !  ", pa
                    DTA    = DFAC * TA( I,J,K )	   !  decoupled
                    QVK(L) = DFAC * QV( I,J,K )	   !   "
                    PRK    = CBTOPA*(PST( I,J )*SIGH( K ) + PTOP) + DPP   !! PA
                    TVK(L) = DTA * ( 1.0 + 0.608 * QVK(L) )
                    THK(L) = DTA*( 1.0E-5 * PRK )**(-ROVCP)   !! 1.0E-5 is 1/(100000Pa)

                END DO  !  end loop on MM5-levels

                PRS1 = CBTOPA*( PST( I,J )*SIGH( KM ) + PTOP )
     &                    + PP( I,J,KM ) * DFAC

                DO  L = 1, NLAYS        !  loop on output layers

                    !!  ALGORITHM:  If requested pressure is less
                    !!  than lowest model-level pressure (i.e., if 
                    !!  altitude of pressure surface is above ground, 
                    !!  interpolate QV using linear coefficients,
		    !!  THETA (stored in THK above) using log coeffs,
		    !!  and then use them to derive T, TV, and DENS;
		    !!  where T is derived from THETA, TV from THETA
		    !!  and QV, and DENS from the combination....

                    !!  ELSE: Below ground, use -9.999 as missing...
                    
                    IF ( PLEV( L ) .GT. PRS1 ) THEN

                        T   (C,R,L) = -9.999
                        TV  (C,R,L) = -9.999
                        DENS(C,R,L) = -9.999
                        RH  (C,R,L) = -9.999

                    ELSE
                    
                        M   = PDEX( C,R,L )
                        AA  = CLIN( C,R,L )
                        BB  = 1.0 - AA
                        QVL = AA*QVK(M) + BB*QVK(M+1)   ! lin interp QV

                        AA  = CLOG( C,R,L )
                        BB  = 1.0 - AA
                        THL = AA*THK(M) + BB*THK(M+1)   !  log interp theta
                        TL  = THL*( 1.0E-5*PLEV( L ) )**ROVCP

                        T   (C,R,L) = TL
                        TV  (C,R,L) = TL * ( 1.0 + 0.608 * QVL )

                        DENS(C,R,L) = PLEV( L )/( RDGAS*TV( C,R,L ) )

                        IF ( TL .GE. CTOK ) THEN !  liquid
                            ES = ALIQ*EXP( (BLIQ*TL-CLIQ)/(TL-DLIQ) )
                        ELSE              !  else below freezing
                            ES = AICE*EXP( (BICE*TL-CICE)/(TL-DICE) )
                        END IF             !  if liquid or frozen

                        RH(C,R,L) = F622*QVL*( PLEV( L ) - ES ) / ES

                    END IF
                END DO  !  end loop on output layers L

11          CONTINUE
22      CONTINUE

        RETURN
        END
