
C Version "%W% %P% %G% %U%

        SUBROUTINE  MMPUV( FACTOR, IM, JM, KM, UM, VM, PST,
     &                     LEVS, COEF, NCOLS, NROWS, NLAYS,
     &                     COL0, ROW0, LAY0, UWIND, VWIND )

C***********************************************************************
C  subroutine main body starts at line  107
C
C  FUNCTION:  Extract/reorder/compute/decouple the cross-point horizontal
C       wind components on the  sub-arrays specified in terms of grid-window
C       bounds to I/O API order from U and V wind component arrays UM and VM
C       and PSTAR-array PST,  found in standard MM5 subscript order.
C       Returns zero-values when interpolation is out of range (i.e., 
C       when requested pressure surface is below terrain).
C
C  NOTE that MM5-order I subscripts go from south to north, J subscripts
C       go from west to east, and K subscripts go from top to bottom,
C       whereas IOAPI-order C subscripts go from west to east,
C       R subscripts go from south to north, and L subscripts go
C       from bottom to top.
C
C  PRECONDITIONS REQUIRED:
C      ioapi grid fits within the MM5 grid
C
C  SUBROUTINES AND FUNCTIONS CALLED: none
C
C  REVISION  HISTORY:
C       Prototype Feb. 3, 1999 by Carlie J. Coats, Jr., MCNC
C       Validated 2/16/99 CJC, JNM
C
C***********************************************************************

      IMPLICIT NONE

C...........   ARGUMENTS and their descriptions:

        REAL            FACTOR  !  conversion factor
        INTEGER         IM      !  I (north-south) dimension for UM,VM
        INTEGER         JM      !  J (east-west)   dimension for UM,VM
        INTEGER         KM      !  K (top-bottom)  dimension for UM,VM
        REAL            UM( IM, JM, KM )        !  MM5-order input array
        REAL            VM( IM, JM, KM )        !  MM5-order input array
        REAL            PST( IM, JM )            !  MM5-order PSTAR array
        REAL            LEVS( NCOLS+1, NROWS+1, NLAYS )  !  dot-pt interp 
        REAL            COEF( NCOLS+1, NROWS+1, NLAYS )  !  index & coeffs
        INTEGER         NCOLS	!  output dimension
        INTEGER         NROWS	!  " 
        INTEGER         NLAYS	!  "
        INTEGER         COL0	!  starting subscript for output window
        INTEGER         ROW0	!  "
        INTEGER         LAY0	!  "
        REAL            UWIND( NCOLS, NROWS, NLAYS )
        REAL            VWIND( NCOLS, NROWS, NLAYS )

C...........   EXTERNAL FUNCTIONS:

#if __sgi  || __sun || __osf__ || __mips__
        INTEGER         MALLOC
#endif    /* __sgi  || __sun || __osf__ || __mips__ */


C...........   SCRATCH LOCAL VARIABLES and their descriptions:

	INTEGER		C, R, L		        !  IOAPI col subscript
        INTEGER         I, J, K                 !  MM5 subscripts
        INTEGER         I0, J0, K0              !  MM5 offsets
	INTEGER		N			!  single-indexing subscript

        REAL            D( NCOLS+1, NROWS+1 )   !  decouple factor
        REAL            U( NCOLS+1, NROWS+1 )   !  decoupled UM
        REAL            V( NCOLS+1, NROWS+1 )   !  decoupled VM
        REAL            PP, QQ

#if __sgi  || __sun || __osf__ || __mips__
	INTEGER         LSIZE  / -1 /
	POINTER         ( PD, D )
	POINTER         ( PU, U )
	POINTER         ( PV, V )

	SAVE            PD, PU, PV, LSIZE
#endif    /* __sgi  || __sun || __osf__ || __mips__ */

C***********************************************************************
C   begin body of subroutine  MM2SPD

#if __sgi  || __sun || __osf__ || __mips__
	N = ( NCOLS + 1 ) * ( NROWS + 1 )
        IF ( N .GT. LSIZE ) THEN
	    IF ( LSIZE .GT. 0 ) THEN  
                CALL FREE( PD )
                CALL FREE( PU )
                CALL FREE( PV )
            END IF
	    PD = MALLOC( 4 * N )
	    PU = MALLOC( 4 * N )
	    PV = MALLOC( 4 * N )
	    IF ( PD .EQ. 0  .OR. 
     &           PU .EQ. 0  .OR. 
     &           PV .EQ. 0  ) THEN
                CALL M3EXIT( 'MMXDCPL', 0, 0,
     &                       'Memory allocation error', 2 )
	    END IF
	    LSIZE = N
	END IF
#endif    /* __sgi  || __sun || __osf__ || __mips__ */
	
        I0 = ROW0 - 1                     !  offsets for IOAPI grid within MM5
        J0 = COL0 - 1
        K0 = KM - LAY0 + 2

        DO  22  R = 1, NROWS + 1        !  0.25*FACTOR*(uncoupling-factor)**2
            I = I0 + R
            DO  11  C = 1, NCOLS + 1
                J = J0 + C
                D( C,R ) = FACTOR / ( PST( I-1,J-1 )        
     &                              + PST( I  ,J-1 )
     &                              + PST( I-1,J   )
     &                              + PST( I  ,J   ) )
11          CONTINUE
22      CONTINUE

        DO  77  L = 1, NLAYS

            DO  44  R = 1, NROWS + 1    !  compute rescaled p-interp wind
                I = I0 + R
                DO  33  C = 1, NCOLS + 1
                    J  = J0 + C
                    IF ( COEF( C,R,L ) .LE. 1.0 ) THEN
                        K  = K0 - LEVS( C,R,L )
                        PP = D( C,R ) * COEF( C,R,L )
                        QQ = D( C,R ) * ( 1.0 - COEF( C,R,L ) )
                        U( C,R ) =  PP*UM( I,J,K ) + QQ*UM( I,J,K-1 )
                        V( C,R ) =  PP*VM( I,J,K ) + QQ*VM( I,J,K-1 )
                    ELSE
                        U( C,R ) =  0.0
                        V( C,R ) =  0.0
                    END IF
33              CONTINUE
44          CONTINUE

            DO  66  R = 1, NROWS    !  interpolate cross-point winds
            DO  55  C = 1, NCOLS
                UWIND( C,R,L ) = ( U( C  , R   ) + U( C+1, R   )
     &                           + U( C,   R+1 ) + U( C+1, R+1 ) )
                VWIND( C,R,L ) = ( V( C  , R   ) + V( C+1, R   )
     &                           + V( C,   R+1 ) + V( C+1, R+1 ) )
55          CONTINUE
66          CONTINUE

77      CONTINUE        !  end loop on layers

        RETURN
        END

