
C Version "%W% %P% %G% %U%
C.........................................................................
C Version "%W% %P% %G% %U%"
C MCPL output module for MM5.  Copyright (C) 1999 MCNC
C Distributed under the GNU LESSER GENERAL PUBLIC LICENSE version 2.1
C See file "LGPL.txt" for conditions of use.
C.........................................................................

        SUBROUTINE  MMPXDCPL( FACTOR, IM, JM, KM, AM, PSTAR,
     &                        NCOLS, NROWS, NLAYS,
     &                        COL0, ROW0, LAY0, LEVS, COEF, AOUT )

C***********************************************************************
C  subroutine body starts at line  89
C
C  FUNCTION:  Interpolate/reorder/convert/decouple the CROSS-point sub-array 
C       specified in terms of grid-window bounds to I/O API order from 
C       array AM, which is found in standard MM5 subscript order, using
C       FACTOR for units-conversion and PSTAR for decoupling, using the
C       indices LEVS and coeffs COEF.  (Normally used for pressure-level
C       interpolation, using interpolation coeffs and indices from MMX2P)
C       Returns AOUT=-9.999 for out of range COEF, i.e., when PLEV is
C       below terrain).
C
C  NOTE that MM5-order I subscripts go from south to north, J subscripts
C	go from west to east, and K subscripts go from top to bottom,
C	whereas IOAPI-order C subscripts go from west to east,
C	R subscripts go from south to north, and L subscripts go
C	from bottom to top.
C
C  PRECONDITIONS REQUIRED:
C	ioapi grid fits within the MM5 grid
C
C  SUBROUTINES AND FUNCTIONS CALLED: none
C
C  REVISION  HISTORY:
C	Prototype OCT. 16, 1997 by Carlie J. Coats, Jr., MCNC
C       Validated 2/15/99 CJC, JNM
C
C***********************************************************************

      IMPLICIT NONE

C...........   ARGUMENTS and their descriptions:

        REAL		FACTOR  !  conversion factor
        INTEGER		IM	!  I (north-south) dimension for AM
        INTEGER		JM	!  J (east-west)   dimension for AM
        INTEGER		KM	!  K (top-bottom)  dimension for AM
        REAL		AM( IM, JM, KM )	!  MM5-order input array
        REAL		PSTAR( IM, JM )		!  MM5-order pstar array
        INTEGER         NCOLS	!  output dimension
        INTEGER         NROWS	!  " 
        INTEGER         NLAYS	!  "
        INTEGER         COL0	!  starting subscript for output window
        INTEGER         ROW0	!  "
        INTEGER         LAY0	!  "
        INTEGER         LEVS( NCOLS, NROWS, NLAYS )     !  interp indices
        REAL            COEF( NCOLS, NROWS, NLAYS )     !  interp coeffs
	REAL		AOUT( NCOLS, NROWS, NLAYS )

C...........   EXTERNAL FUNCTIONS:

#if __sgi  || __sun || __osf__ || __mips__
        INTEGER         MALLOC
#endif    /* __sgi  || __sun || __osf__ || __mips__ */


C...........   SCRATCH LOCAL VARIABLES and their descriptions:

	INTEGER		C, R, L		        !  IOAPI col subscript
	INTEGER		I, J, K 		!  MM5 subscripts
	INTEGER		I0, J0, K0 		!  MM5 offsets
        
        REAL		DFAC( NCOLS,NROWS ), PP, QQ

#if __sgi  || __sun || __osf__ || __mips__
	INTEGER         LSIZE
        DATA            LSIZE  / -1 /
	POINTER         ( P, DFAC )

	SAVE            P, LSIZE
#endif    /* __sgi  || __sun || __osf__ || __mips__ */

C***********************************************************************
C   begin body of subroutine  MMXDCPL

#if __sgi  || __sun || __osf__ || __mips__
!$OMP   CRITICAL( MCPL_PXDCPL )
	IF ( NCOLS*NROWS .GT. LSIZE ) THEN
	    IF ( LSIZE .GT. 0 )  CALL FREE( P )
	    P = MALLOC( 8 * NCOLS*NROWS )
	    IF ( P .EQ. 0 )
     &           CALL M3EXIT( 'MMXDCPL', 0, 0, 
     &                       'Memory allocation error', 2 )
	    LSIZE = NCOLS*NROWS
	END IF
#endif    /* __sgi  || __sun || __osf__ || __mips__ */
	
        I0 = ROW0 - 1			!  offsets for IOAPI grid within MM5
        J0 = COL0 - 1
        K0 = KM - LAY0 + 2

        DO  22  R = 1, NROWS
            I = I0 + R
            DO  11  C = 1, NCOLS
                J = J0 + C
                DFAC( C,R ) = FACTOR / PSTAR( I,J )
11          CONTINUE
22      CONTINUE

        DO  55  L = 1, NLAYS
            DO  44  R = 1, NROWS
                I = I0 + R
                DO  33  C = 1, NCOLS
                    J = J0 + C
                    K = K0 - LEVS( C,R,L )
                    PP = COEF( C,R,L )
                    IF ( PP .LT. 1.0 ) THEN
                        QQ = 1.0 - PP
                        AOUT( C,R,L ) = DFAC( C,R )*( PP*AM( I,J,K )
     &                                              + QQ*AM( I,J,K-1))
                    ELSE
				! we are over terrain which intersects the constant
                                ! pressure surface requested. For scalars, we either 
                                ! follow current INTERP practice and use values from the 
                                ! lowest MM5 sigma layer on that pressure surface; or 
                                ! or assign a negative "missing" value. At present
                                ! (2/15/99) we choose the missing value option because
                                ! the user can more easily recognize that the pressure
                                ! surfaces "don't exist" underground....Commented formula
                                ! is the INTERP practice...

C...                        AOUT( C,R,L ) = DFAC( C,R ) * AM( I,J,K )
                            AOUT( C,R,L ) = -0.999   !! 
                    END IF
33              CONTINUE
44          CONTINUE
55      CONTINUE

#if __sgi  || __sun || __osf__ || __mips__
!$OMP   END CRITICAL( MCPL_PXDCPL )
#endif    /* __sgi  || __sun || __osf__ || __mips__ */

        RETURN
        END
